
#include <LEDA/graphwin.h>

#include <AGD/FPPLayout.h>
#include <AGD/ConvexLayout.h>
#include <AGD/ConvexDrawLayout.h>
#include <AGD/MixedModelLayout.h>
#include <AGD/PlanarStraightLayout.h>
#include <AGD/PlanarDrawLayout.h>
#include <AGD/SchnyderLayout.h>
#include <AGD/SpringLayout.h>
#include <AGD/TutteLayout.h>
#include <AGD/VisibilityRepresentation.h>
#include <AGD/TreeLayout.h>
#include <AGD/SugiyamaLayout.h>
#include <AGD/PlanarizationGridLayout.h>
#include <AGD/Ortho.h>
#include <AGD/Quod.h>

#include <AGD/PlanAug.h>
#include <AGD/PlanarSubgraph.h>
#include <AGD/LEDAMakeBiconnected.h>
#include <AGD/FastHierarchyLayout.h>

#include <AGD/generators.h>



class GraphWinAGD : public GraphWin
{
public:

	GraphWinAGD();

	virtual ~GraphWinAGD() { }

protected:
	void optionsConvex();
	void optionsConvexDraw();
	virtual void optionsStraight();
	virtual void optionsPlanarDraw();
	virtual void optionsMixedModel();
	void optionsQuod();
	void optionsSpring();
	void optionsTreeLayout();
	void optionsSugiyama();
	virtual void optionsPlanarization();

	FPPLayout fpp;
	ConvexLayout convex;
	ConvexDrawLayout convex_draw;
	PlanarStraightLayout straight;
	PlanarDrawLayout planar_draw;
	MixedModelLayout mixed;
	SchnyderLayout schnyder;
	SpringLayout spring;
	TutteLayout tutte;
	VisibilityRepresentation visib_rep;
	SugiyamaLayout sugiyama;
	TreeLayout tree_layout;
	PlanarizationGridLayout planarization;
	Ortho ortho;
	Quod quod;

	PlanAug plan_aug;
	LEDAMakeBiconnected make_bic;
	PlanarSubgraph plan_sub;
	FastHierarchyLayout fhl;

public:
	static void callFppLayout (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callGridLayout (gwAGD, gwAGD.fpp);
	}
	static void callConvexLayout (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callGridLayout (gwAGD, gwAGD.convex);
	}
	static void callConvexDrawLayout (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callGridLayout (gwAGD, gwAGD.convex_draw);
	}
	static void callPlanarStraightLayout (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callGridLayout (gwAGD, gwAGD.straight);
	}
	static void callPlanarDrawLayout (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callGridLayout (gwAGD, gwAGD.planar_draw);
	}
	static void callMixedModelLayout (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callGridLayout (gwAGD, gwAGD.mixed);
	}
	static void callOrthoLayout (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callGridLayout (gwAGD, gwAGD.ortho);
	}
	static void callQuodLayout (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callGridLayout (gwAGD, gwAGD.quod);
	}
	static void callSchnyderLayout (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callGridLayout (gwAGD, gwAGD.schnyder);
	}
	static void callTutteLayout (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		gwAGD.tutte.bbox (DRect(GW.get_xmin(), GW.get_xmax(),
			-GW.get_ymax(), -GW.get_ymin()));
		callLayout (gwAGD, gwAGD.tutte);
	}
	static void callSpringLayout (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callLayout (gwAGD, gwAGD.spring);
	}
	static void callVisibilityRepresentation (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callGridLayout (gwAGD, gwAGD.visib_rep);
	}
	static void callSugiyamaLayout(GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callLayout(gwAGD, gwAGD.sugiyama);
	}
	static void callTreeLayout(GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callLayout(gwAGD, gwAGD.tree_layout);
	}
	static void callPlanarizationLayout(GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callGridLayout(gwAGD, gwAGD.planarization);
	}


	static void callRandomPlanarTricGraph (GraphWin& GW) {
		gen_random_graph (GW, random_tric_graph,
			"Create Random Triconnected Planar Graph");
	}
	static void callRandomPlanarBicGraph (GraphWin& GW) {
		gen_random_graph (GW, random_bic_graph,
			"Create Random Biconnected Planar Graph");
	}
	static void callRandomGraph (GraphWin& GW) {
		gen_random_graph (GW, random_graph_2,
			"Create Random Graph");
	}
	static void callRandomGraph2 (GraphWin& GW) {
		gen_random_graph (GW, random_graph_3,
			"Create Random Graph (2)");
	}
	static void callRandomFourGraph (GraphWin& GW) {
		gen_random_graph (GW, random_four_graph,
			"Create Random Four-Graph");
	}
	static void callRandomHierarchy (GraphWin& GW);

	static void callPlanAug (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callAugment (gwAGD, gwAGD.plan_aug);
	}
	static void callMakeBic (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callAugment (gwAGD, gwAGD.make_bic);
	}

	static void callPlanSub (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callSubgraph (gwAGD, gwAGD.plan_sub);
	}

	static void callOptionsConvex (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		gwAGD.optionsConvex();
	}
	static void callOptionsConvexDraw (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		gwAGD.optionsConvexDraw();
	}
	static void callOptionsStraight (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		gwAGD.optionsStraight();
	}
	static void callOptionsPlanarDraw (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		gwAGD.optionsPlanarDraw();
	}
	static void callOptionsMixedModel (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		gwAGD.optionsMixedModel();
	}
	static void callOptionsQuod (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		gwAGD.optionsQuod();
	}
	static void callOptionsSpring (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		gwAGD.optionsSpring();
	}
	static void callOptionsTreeLayout(GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		gwAGD.optionsTreeLayout();
	}
	/*static void callOptionsSugiyama (GraphWin &GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		gwAGD.optionsSugiyama();
	}*/
	static void callOptionsPlanarization (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		gwAGD.optionsPlanarization();
	}
	static void callOptionsSugiyama (GraphWin &GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		gwAGD.optionsSugiyama();
	}

	static void animateSpring (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		animate (gwAGD, gwAGD.spring);
	}
	/*static void animateSugiyama (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		animate (gwAGD, gwAGD.sugiyama);
	}*/

	static bool callLayout (GraphWinAGD& GW, LayoutModule& L);
	static bool callGridLayout (GraphWinAGD& GW, GridLayoutModule& GL);
	static bool callAugment (GraphWinAGD& GW, AugmentModule& A);
	static bool callSubgraph (GraphWinAGD& GW, SubgraphModule& S);

	static void fit_in_window (GraphWinAGD& GW, LayoutInterface& A);
	static void anim_action (int n);
	static void wait (GraphWinAGD& GW, panel& P, float delay, int QUIT);
	static void animate(GraphWinAGD& GW, LayoutModule& L);

protected:
	//bool sugiyama_infos;
	int planarization_layouter, planarization_planarizer;
	int sug1_ranking;
	int sug1_cross_min;
	static int anim_status;
	static bool rh_planar, rh_single;

	int layout_menu, animation_menu, generators_menu, options_menu,
		graph_menu, augment_menu, subgraph_menu;

	//void show_results(const AGDModule& M, window& results);

	/*static string sugi_to_string (int x) {
		return (x == SugiyamaHierarchy::NOT_COMPUTED) ?
			string("NOT COMPUTED") : string("%d",x);
	}*/

	static void embed_random(const graph& G, node_array<point>& dest);
	static void stretch(const graph& G, node_array<double>& destx,
        node_array<double>& desty, double x0, double y0, double x1, double y1);
	static void stretch(const graph& G, node_array<point>& dest,
        double x0, double y0, double x1, double y1);
	static void gen_random_graph(GraphWin& gw, void (*f)(graph&,int,int), string s);
	static void gen_random_graph(GraphWin& gw, void (*f)(graph&,int), string s);

	static void random_bic_graph (graph& G, int num_nodes);
	static void make_simple (graph& G);
	static void random_graph_2 (graph& G, int num_nodes, int num_edges);
	static void random_graph_3 (graph& G, int num_nodes, int num_edges);
	static void set_N(node v, bool b, const graph& G, node_array<bool>& N);
	static void random_four_graph (graph& G, int n, int m);
};
