//-------------------------------------------------------------------
//
//                         DPolyline.h
//
//    author: Carsten Gutwenger
//    date:   march 1997
//-------------------------------------------------------------------


#ifndef _AGD_DPOLYLINE_H
#define _AGD_DPOLYLINE_H


#include <AGD/impl/misc.h>
#include <LEDA/list.h>
#include <AGD/DPoint.h>


class DSegment;


/*{\Manpage {DPolyline} {} {Double Polylines} {dpl} }*/

/*{\Mbaseclasses

\begin{hierarchy}{6.5}{2.5}
\hdclass{0}{1.5}{5}{list{\tt <}DPoint{\tt >}}
\arhook{5}{2}{6}
\hdclass{1.5}{0}{5}{DPolyline}
\end{hierarchy}
}*/

class DPolyline : public leda_list<DPoint> {

/*{\Mdefinition
An instance |\Mvar| of type |\Mname| is a list
$|dp|_1,\ldots,|dp|_n$ of double points $|dp|_i$.
}*/

public:

	/*{\Mcreation}*/

	DPolyline () : leda_list<DPoint>() { }
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname|
	initialized to an empty list.}*/

	DPolyline (const DPolyline& dpl) : leda_list<DPoint>(dpl) { }

	virtual ~DPolyline() { }

	/*{\Moperations}*/

	DPoint& get_point (int i);
	/*{\Mop returns a reference to $|dp|_i$ if |\Mvar| is
	$|dp|_1,\ldots,|dp|_n$. \precond $1\leq i\leq n$}*/

	DPoint& get_rev_point (int i);
	/*{\Mop returns a reference to $|dp|_{n-i+1}$ if |\Mvar| is
	$|dp|_1,\ldots,|dp|_n$. \precond $1\leq i\leq n$}*/

	DSegment get_segment (int i) const;
	/*{\Mop returns the double segment $[|dp|_i,|dp|_{i+1}]$ if
	|\Mvar| is $|dp|_1,\ldots,|dp|_n$. \precond $1\leq i< n$}*/

	DSegment get_rev_segment (int i) const;
	/*{\Mop returns the double segment $[|dp|_{n-i+1},|dp|_{n-i}]$ if
	|\Mvar| is $|dp|_1,\ldots,|dp|_n$. \precond $1\leq i< n$}*/

	void reverse () {
		reverse_list (*((leda_list<DPoint> *)(this)));
	}
	/*{\Mop reverses the list of double points.}*/
};

/*{\Mimplementation
All operations for
|list<DPoint>| are also available for |\Mname|.
}*/


#endif
