//-------------------------------------------------------------------
//
//                          DSegment.h
//
//    author: Carsten Gutwenger
//    date:   march 1997
//-------------------------------------------------------------------


#ifndef _AGD_DSEGMENT_H
#define _AGD_DSEGMENT_H


#include <AGD/DPoint.h>


/*{\Manpage {DSegment} {} {Double Line Segments} {ds} }*/

class DSegment {

/*{\Mdefinition
An instance |\Mvar| of type |\Mname| is a line segment $[p,q]$,
where $p$ and $q$ are double points. We call $p$ the source point
and $q$ the target point of the segment.
}*/

public:

	/*{\Mcreation}*/

	DSegment () { }
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname|
	initialized to the segment $[(0,0),(0,0)]$.}*/

	DSegment (const DPoint& p, const DPoint& q) {
		the_p = p; the_q = q;
	}
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname|
	initialized to the segment $[p,q]$.}*/


	/*{\Moperations}*/

	DPoint& source() {
		return the_p;
	}

	const DPoint& source() const {
		return the_p;
	}
	/*{\Mop returns a reference to the source point of |\Mvar|.}*/

	DPoint& target() {
		return the_q;
	}

	const DPoint& target() const {
		return the_q;
	}
	/*{\Mop returns a reference to the target point of |\Mvar|.}*/

	bool horizontal() const {
		return (the_p.ycoord() == the_q.ycoord());
	}
	/*{\Mop returns |true| iff |\Mvar| is horizontal, i.e
	the y-coordinate of the source point equals the
	y-coordinate of the target point.}*/

	bool vertical() const {
		return (the_p.xcoord() == the_p.ycoord());
	}
	/*{\Mop returns |true| iff |\Mvar| is vertical, i.e
	the x-coordinate of the source point equals the
	x-coordinate of the target point.}*/

		
private:
	DPoint the_p, the_q;
};


#endif
