//-------------------------------------------------------------------
//
//                          DfsRanking.h
//
//
//    author: Carsten Gutwenger
//    date:   february 1998
//-------------------------------------------------------------------


#ifndef _AGD_DFS_RANKING_H
#define _AGD_DFS_RANKING_H


#include <AGD/RankAssignment.h>


/*{\Manpage {DfsRanking} {} {Ranking by Depth First Search} {R} }*/

/*{\Mbaseclasses

\begin{hierarchy}{8}{4}
\hdclass{0}{3}{5.5}{AGDModule}
\arhook{5.5}{3.5}{6.5}
\hdclass{1.5}{1.5}{5.5}{RankAssignment}
\arhook{7}{2}{8}
\hdclass{3}{0}{5.5}{DfsRanking}
\end{hierarchy}
}*/

class DfsRanking : public RankAssignment {

/*{\Mdefinition
The |\Mname| module computes the following ranking of a graph $G$:
We build a DFS forest $T$ of the underlying undirected graph
of $G$ and set
\[rank(v) = \left\{\begin{array}{cl}
0 & \mbox{if $v$ is a root of $T$}\\
rank(pred(v))+1 & \mbox{otherwise}
\end{array}\right.\]
where $pred(v)$ is the predecessor of $v$ in $T$.
}*/

public:

	/*{\Mcreation}*/

	DfsRanking();
	/*{\Mcreate  creates an instance |\Mvar| of type |\Mname|.}*/

	~DfsRanking() { }


	/*{\Moperations}*/

	void call (const leda_graph& G, leda_node_array<int>& rank);
	/*{\Mop calls |\Mvar| for graph $G$. Returns the rank for each
	node $v\in G$ in |rank[v]|.}*/

	AGDModule *clone() const;

private:
	void dfs(leda_node v, int r, leda_node_array<int>& rank);
};


#endif
