//-------------------------------------------------------------------
//
//                       LEDAMakeBiconnected.h
//
//    author: AGD-Anpassung: Carsten Gutwenger
//    date:   july 1997
//-------------------------------------------------------------------


#ifndef _AGD_LEDA_MAKE_BICONNECTED_H
#define _AGD_LEDA_MAKE_BICONNECTED_H


#include <AGD/AugmentModule.h>


/*{\Manpage {LEDAMakeBiconnected} {} {LEDA's Make Biconnected} {A} }*/

/*{\Mbaseclasses

\begin{hierarchy}{8}{4}
\hdclass{0}{3}{5.5}{AGDModule}
\arhook{5.5}{3.5}{6.5}
\hdclass{1.5}{1.5}{5.5}{AugmentModule}
\arhook{7}{2}{8}
\hdclass{1}{0}{7.5}{LEDAMakeBiconnected}
\end{hierarchy}
}*/

class LEDAMakeBiconnected : public AugmentModule {

/*{\Mdefinition
The |\Mname| module is a wrapper for LEDA's Make\_Biconnected function.
It augments a graph $G$ to a biconnected graph $G'$. If $G$ was planar,
the output graph $G'$ is also planar.

\horizline
\commoninfo{Make Biconnected}{LEDA's Make Biconnected}{}{Stefan Naeher}
{}{}

\headerline{Pre- and Postcondition}
\begin{tabular}{l@{ = }l}
precondition & $\emptyset$ \\
postcondition(|PRE|) & $\{\, |biconnected| \,\}
\cup \Big( \{\, |planar|,|simple|,|no self loops| \,\} \cap |PRE| \Big)$
\end{tabular}

\horizline
}*/

public:

	/*{\Mcreation A 5}*/

	LEDAMakeBiconnected();
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname|.}*/

	virtual ~LEDAMakeBiconnected() { }


	/*{\Moperations}*/

	leda_string name () const {
		return leda_string ("Make Biconnected");
	}

	leda_string long_name () const {
		return leda_string ("LEDA's Make Biconnected");
	}

	leda_string impl_author () const {
		return leda_string ("Stefan Naeher");
	}


	AGDModule *clone() const;

	/*{\Moptions warnings=no}*/

	/*{\Mtext       
	\headerline{Generic Interface (Inherited Methods)}
	The detailed description of these methods can be found in the
	manual entries of the base class (|AugmentModule|).
	\setlength{\entrysep}{-1ex}}*/

	/* inherited
	bool check (const graph& G, string& message);
	*/
	/*{\Mop}*/

	/* inherited
	bool call (graph& G);
	*/
	/*{\Mop}*/

	/* inherited
	bool call (graph& G, list<edge>& L);
	*/
	/*{\Mop}*/
	
	/*{\Mtext \setlength{\entrysep}{0.5ex}}*/
	

protected:
	void _call (leda_graph& G, leda_list<leda_edge>& L);

};


#endif
