//-------------------------------------------------------------------
//
//                         MaximumPlanarSubgraph.h
//
//
// author: Sergey Fialko
// date:   march 1998
//-------------------------------------------------------------------


#ifndef _AGD_MAXIMUM_PLANAR_SUBGRAPH_H
#define _AGD_MAXIMUM_PLANAR_SUBGRAPH_H


#include <AGD/SubgraphModule.h>


/*{\Manpage {MaximumPlanarSubgraph} {} {Maximum Planar Subgraphs} {S} }*/

/*{\Mbaseclasses
\abacus
\begin{hierarchy}{8}{4}
\hdclass{0}{3}{5.5}{AGDModule}
\arhook{5.5}{3.5}{6.5}
\hdclass{1}{1.5}{6}{SubgraphModule}
\arhook{7}{2}{8}
\hdclass{0.5}{0}{8}{MaximumPlanarSubgraph}
\end{hierarchy}
}*/

class MaximumPlanarSubgraph : public SubgraphModule {

/*{\Mdefinition
The module |\Mname| computes a maximum planar subgraph of an
input graph $G$ (\cite{JuMu}).

\horizline
\commoninfo{Maximum Planar Subgraph}{Maximum Planar Subgraph}{Petra Mutzel}{Sergey Fialko}
{march 1998}{}

\headerline{Pre- and Postcondition}
\begin{tabular}{lcl}
precondition & $=$ & $\emptyset$ \\
postcondition(|PRE|) & $=$ & $\{\, |planar| \,\}
\cup \Big( \{\, |simple|,|no self loops|,|no reversals| \,\} \cap |PRE| \Big)$
\end{tabular}
\horizline
}*/

private:
  int nHeuristicCalls_;// 

public:

	/*{\Mcreation S 5.5}*/

	MaximumPlanarSubgraph();
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname|.}*/

	~MaximumPlanarSubgraph() { }


	/*{\Moperations 1.2 3.5}*/

	leda_string name () const {
		return "Maximum Planar Subgraph";
	}

	leda_string author () const {
		return "Petra Mutzel";
	}

	leda_string impl_author () const {
		return "Sergej Fialko";
	}

	leda_string impl_date () const {
		return "march 1998";
	}

	AGDModule *clone() const;
	
	/*{\Moptions warnings=no}*/

	/*{\Mtext       
	\headerline{Generic Interface (Inherited Methods)} }*/

	/* inherited
	bool check (const graph& G, string& message);
	*/
	/*{\Mop returns |true| if $G$ satisfies the precondition of |\Mvar|.
	Otherwise, |false| is returned, and |message| contains the
	error message.}*/

	/* inherited
	bool call (const graph& G, list<edge>& L);
	*/
	/*{\Mop calls |\Mvar| for graph $G$. Returns in $L$ the
	list of edges $e_1,\ldots,e_k$.}*/

	/* inherited
	bool call (graph& G);
	*/
	/*{\Mop calls |\Mvar| for graph $G$, and deletes all
	edges $e_1,\ldots,e_k$ in $G$.}*/

	/* inherited
	bool call (GraphCopy& G, list<edge>& L);
	*/
	/*{\Mop calls |\Mvar| for graph copy $G$. Deletes all
	edges $e_1,\ldots,e_k$ in $G$, and returns in $L$ the list of
	corresponding edges $|original|(e_1),\ldots,|original|(e_k)$ in
	|original(G)|.}*/

	/* inherited
	int num_del_edges() const;
	*/
	/*{\Mop returns the number $k$ of edges in $E\cap E'$.}*/


	inline void nHeuristicCalls (int n) { nHeuristicCalls_ = n; }

protected:
	void _call (const leda_graph& G, leda_list<leda_edge>& L);
};


#endif
