//-------------------------------------------------------------------
//
//                         PlanAug.h
//
//    author: Sergey Fialko, (AGD-Anpassung: Carsten Gutwenger)
//    date:   july 1997
//-------------------------------------------------------------------


#ifndef _AGD_PLAN_AUG_H
#define _AGD_PLAN_AUG_H


#include <AGD/AugmentModule.h>


/*{\Manpage {PlanAug} {} {Planar Biconnected} {A} }*/

/*{\Mbaseclasses

\begin{hierarchy}{8}{4}
\hdclass{0}{3}{5.5}{AGDModule}
\arhook{5.5}{3.5}{6.5}
\hdclass{1.5}{1.5}{5.5}{AugmentModule}
\arhook{7}{2}{8}
\hdclass{3.5}{0}{5}{PlanAug}
\end{hierarchy}
}*/

class PlanAug : public AugmentModule {

/*{\Mdefinition
The |\Mname| module augments a planar graph to a planar biconnected
graph (\cite{Fialko97}, \cite{Mu95}).

\horizline
\commoninfo{Planar Biconnected}{Planar Biconnected}{}{Sergey Fialko}
{octobre 1997}{}

\headerline{Pre- and Postcondition}
\begin{tabular}{l@{ = }l}
precondition & $\{\, |planar| \,\}$ \\
postcondition(|PRE|) & $\{\, |planar|,|biconnected| \,\}
\cup \Big( \{\, |simple|,|no self loops| \,\} \cap |PRE| \Big)$
\end{tabular}

\headerline{Optional Parameters}
Instances of |\Mname| provide the following optional parameters:
\begin{options}
\item |bool| {\bf streng\_cond} = |true|\\
...
\end{options}\horizline
}*/

public:

	/*{\Mcreation}*/

	PlanAug();
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname|.}*/

	virtual ~PlanAug() { }


	/*{\Moperations}*/

	leda_string name () const {
		return "Planar Biconnected";
	}

	leda_string impl_author () const {
		return "Sergey Fialko";
	}

	leda_string impl_date () const {
		return "octobre 1997";
	}


	bool streng_cond() const {
		return _streng_cond;
	}

	void streng_cond (bool b) {
		_streng_cond = b;
	}

	AGDModule *clone() const;

	/*{\Moptions warnings=no}*/

	/*{\Mtext       
	\headerline{Generic Interface (Inherited Methods)}
	The detailed description of these methods can be found in the
	manual entries of the base class (|AugmentModule|).
	\setlength{\entrysep}{-1ex}}*/

	/* inherited
	bool check (const graph& G, string& message);
	*/
	/*{\Mop}*/

	/* inherited
	bool call (graph& G);
	*/
	/*{\Mop}*/

	/* inherited
	bool call (graph& G, list<edge>& L);
	*/
	/*{\Mop}*/
	
	/*{\Mtext \setlength{\entrysep}{0.5ex}}*/

protected:
	void _call (leda_graph& G, leda_list<leda_edge>& L);

private:
	bool _streng_cond;
};


#endif
