//-------------------------------------------------------------------
//
//                         PlanAugOpt.h
//
//    author: Sergey Fialko
//    date:   octobre 1997
//-------------------------------------------------------------------


#ifndef _AGD_PLAN_AUG_OPT_H
#define _AGD_PLAN_AUG_OPT_H

#include <AGD/PlanAug.h>


/*{\Manpage {PlanAugOpt} {} {Optimal Planar Biconnected} {A} }*/

/*{\Mbaseclasses
\abacus
\begin{hierarchy}{8}{5.5}
\hdclass{0}{4.5}{5.5}{AGDModule}
\arhook{5.5}{5}{6.5}
\hdclass{1.5}{3}{5.5}{AugmentModule}
\arhook{7}{3.5}{8}
\hdclass{3.5}{1.5}{5}{PlanAug}
\arhook{8.5}{2}{9.5}
\hdclass{5}{0}{5}{PlanAugOpt}
\end{hierarchy}
}*/

class PlanAugOpt : public PlanAug {

/*{\Mdefinition
The |\Mname| module augments a planar graph to a planar biconnected
graph by adding the minimal number of edges.

\horizline
\commoninfo{Optimal Planar Biconnected}{Optimal Planar Biconnected}{}{Sergey Fialko}
{octobre 1997}{}

\headerline{Pre- and Postcondition}
\begin{tabular}{l@{ = }l}
precondition & $\{\, |planar| \,\}$ \\
postcondition(|PRE|) & $\{\, |planar|,|biconnected| \,\}
\cup \Big( \{\, |simple|,|no self loops| \,\} \cap |PRE| \Big)$
\end{tabular}

\headerline{Optional Parameters}
Instances of |\Mname| provide the following optional parameters:
\begin{options}
\item |bool| {\bf streng\_cond} = |true|\\
...
\end{options}\horizline
}*/

private:
  bool heurist_optim_;// = true, if approx. solution was optimal 
  bool only_bac; //for test purposes, call only branch and cut routine without heuristics


public:

	/*{\Mcreation}*/

  PlanAugOpt ();
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname|.}*/

  ~PlanAugOpt () { }
  
	/*{\Moperations}*/


  leda_string name () const {
     return "Optimal Planar Biconnected";
  }

  inline bool heurist_optim () { return heurist_optim_; }

	/*{\Moptions warnings=no}*/

	/*{\Mtext       
	\bigskip
	\headerline{b) Generic Interface (Inherited Methods)} }*/

	/* inherited
	bool check (const graph& G, string& message);
	*/
	/*{\Mop returns |true| if $G$ satisfies the precondition of |\Mvar|.
	Otherwise, |false| is returned, and |message| contains the
	error message.}*/

	/* inherited
	bool call (graph& G);
	*/
	/*{\Mop calls the |\Mname| module for graph $G$.}*/

	/* inherited
	bool call (graph& G, list<edge>& L);
	*/
	/*{\Mop calls the |\Mname| module for graph $G$. Returns the
	list of added edges in |L|.}*/

  void set_bac_only () { only_bac = true; }
	
protected:
  void _call (leda_graph& G, leda_list<leda_edge>& L);
};


#endif
