//-------------------------------------------------------------------
//
//                         PlanarSubgraph.h
//
//
// author: Carsten Gutwenger, Thomas Ziegler (pre- and postprocessing)
//         Sebastian Leipert (actual algorithm)
// date:   novembre 1997
//-------------------------------------------------------------------


#ifndef _AGD_PLANAR_SUBGRAPH_H
#define _AGD_PLANAR_SUBGRAPH_H


#include <AGD/SubgraphModule.h>


/*{\Manpage {PlanarSubgraph} {} {Planar Subgraphs} {S} }*/

/*{\Mbaseclasses

\begin{hierarchy}{8}{4}
\hdclass{0}{3}{5.5}{AGDModule}
\arhook{5.5}{3.5}{6.5}
\hdclass{1}{1.5}{6}{SubgraphModule}
\arhook{7}{2}{8}
\hdclass{2.5}{0}{6}{PlanarSubgraph}
\end{hierarchy}
}*/

class PlanarSubgraph : public SubgraphModule {

/*{\Mdefinition
The module |\Mname| computes a planar subgraph of an
input graph $G$ (\cite{JLM96}).

\horizline
\commoninfo{Planar Subgraph}{Planar Subgraph}{}{Sebastian Leipert}
{}{novembre 1997}

\headerline{Pre- and Postcondition}
\begin{tabular}{lcl}
precondition & $=$ & $\{\,  \,\}$ \\
postcondition(|PRE|) & $=$ & $\{\, |planar| \,\}
\cup \Big( \{\, |simple|,|no self loops|,|no reversals| \,\} \cap |PRE| \Big)$
\end{tabular}
\horizline
}*/

public:

	/*{\Mcreation S 4.5}*/

	PlanarSubgraph();
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname|.}*/

	virtual ~PlanarSubgraph() { }


	/*{\Moperations}*/

	leda_string name () const {
		return leda_string ("Planar Subgraph");
	}

	leda_string impl_author () const {
		return leda_string ("Sebastian Leipert");
	}

	AGDModule *clone() const;

	/*{\Moptions warnings=no}*/

	/*{\Mtext       
	\headerline{Generic Interface (Inherited Methods)} }*/

	/* inherited
	bool check (const graph& G, string& message);
	*/
	/*{\Mop returns |true| if $G$ satisfies the precondition of |\Mvar|.
	Otherwise, |false| is returned, and |message| contains the
	error message.}*/

	/* inherited
	bool call (const graph& G, list<edge>& L);
	*/
	/*{\Mop calls |\Mvar| for graph $G$. Returns in $L$ the
	list of edges $e_1,\ldots,e_k$.}*/

	/* inherited
	bool call (graph& G);
	*/
	/*{\Mop calls |\Mvar| for graph $G$, and deletes all
	edges $e_1,\ldots,e_k$ in $G$.}*/

	/* inherited
	bool call (GraphCopy& G, list<edge>& L);
	*/
	/*{\Mop calls |\Mvar| for graph copy $G$. Deletes all
	edges $e_1,\ldots,e_k$ in $G$, and returns in $L$ the list of
	corresponding edges $|original|(e_1),\ldots,|original|(e_k)$ in
	|original(G)|.}*/

	/* inherited
	int num_del_edges() const;
	*/
	/*{\Mop returns the number $k$ of edges in $E\cap E'$.}*/


	
protected:
	void _call (const leda_graph& G, leda_list<leda_edge>& L);

};


#endif
