//-------------------------------------------------------------------
//
//                       SchnyderLayout.h
//
//    author: Dirk Ambras, AGD modifications David Alberts
//    date:   february 1997
//-------------------------------------------------------------------


#ifndef _AGD_SCHN_LAYOUT_H
#define _AGD_SCHN_LAYOUT_H


#include <AGD/GridLayoutModule.h>


/*{\Manpage {SchnyderLayout} {} {The Schnyder Algorithm} {L} }*/

/*{\Mbaseclasses

\begin{hierarchy}{9.5}{5.5}
\hdclass{0}{4.5}{5}{AGDModule}
\arhook{5}{5}{6}
\hdclass{1.5}{3}{5}{LayoutModule}
\arhook{6.5}{3.5}{7.5}
\hdclass{2}{1.5}{6}{GridLayoutModule}
\arhook{8}{2}{9}
\hdclass{4.5}{0}{5}{SchnyderLayout}
\end{hierarchy}
\hfill\agdeps{schnyder.epsi}{5.5cm}
}*/

class SchnyderLayout : public GridLayoutModule
{

/*{\Mdefinition
The class |\Mname| represents the layout algorithm by
Schnyder~\cite{s-epgg-90}.

This algorithm draws a planar graph |G| straight-line
without crossings. |G| may not contain self-loops or mulitple edges.
The grid layout
size is $(n-2) \times (n-2)$ for a graph with $n$ nodes ($n \geq 3$).

\horizline
\commoninfo{Schnyder}{Schnyder}{W. Schnyder}{Dirk Ambras, David Alberts}
{1995/1997}{}

\headerline{Pre- and Postcondition}
\begin{tabular}{lcl}
precondition & $=$ & $\{\, |planar|,|simple|,|no self loops| \,\}$ \\
postcondition(|PRE|) & $=$ & $\{\, \mbox{|straight|-|line|},|no crossings| \,\}$
\end{tabular}

\horizline
}*/

  public:

	/*{\Mcreation}*/

    SchnyderLayout();
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname|.}*/

    ~SchnyderLayout() { }

	/*{\Moperations}*/


    leda_string name () const { return "Schnyder"; }

    leda_string long_name () const { return "Schnyder"; }

    leda_string author () const { return "W. Schnyder"; }

	leda_string impl_author () const {
		return "Dirk Ambras, David Alberts";
	}

  	leda_string impl_date () const {
		return "february 1997";
	}

	AGDModule *clone () const;

	/*{\Moptions warnings=no}*/

	/*{\Mtext       
	\headerline{Generic Interface (Inherited Methods)}
	The detailed description of these methods can be found in the
	manual entries of the base classes (|LayoutModule|, |GridLayoutModule|).
	\setlength{\entrysep}{-1ex}}*/

	/* inherited
	bool check (const graph& G, string& message);
	*/
	/*{\Mop}*/

	/* inherited
	void call (const graph& G, LayoutInterface& A);
	*/
	/*{\Mop}*/

	/* inherited
	virtual void call (const graph& G, face f, LayoutInterface& A);
	*/
	/*{\Mop}*/

	/* inherited
	virtual void call (PlaneGraphCopy& PG, face f, LayoutInterface& A);
	*/
	/*{\Mop}*/

	/* inherited
	void call_grid (const graph& G, GridLayout &gl);
	*/
	/*{\Mop}*/

	/* inherited
	void call_grid (const graph& G, face f, GridLayout &gl);
	*/
	/*{\Mop}*/

	/* inherited
	void call_grid (PlaneGraphCopy& PG, face f, GridLayout &gl);
	*/
	/*{\Mop}*/

	/* inherited
	IRect get_grid_area ();
	*/
	/*{\Mop}*/

	/* inherited
	void set_coordinate_mapper (const GridCoordinateMapper& gcm);
	*/
	/*{\Mop}*/
	
	/*{\Mtext \setlength{\entrysep}{0.5ex}}*/

protected:

	void _call_grid (PlaneGraphCopy& PG, leda_face F, GridLayout &gl);

  private:

    void Contract(leda_graph& G, leda_node a, leda_node b , leda_node c,
		leda_list<leda_node>& L);

    void Realizer(leda_graph& G, const leda_list<leda_node>& L, leda_node a,
		leda_node b, leda_node c, GRAPH<leda_node, int>& T,
		leda_node_array<leda_node>& v_in_T);

    void Subtree_Sizes(GRAPH<leda_node, int>& T, int i, leda_node r,
        leda_node_array<int>& size);

    void Prefix_Sum(GRAPH<leda_node, int>& T, int i, leda_node r,
        const leda_node_array<int>& val, leda_node_array<int>& sum);

    int schnyder_embedding(GraphCopy& G,leda_node_map<int>& xcoord,
        leda_node_map<int>& ycoord);

    void schnyder_embedding(GraphCopy& G,leda_node_map<double>& x,
        leda_node_map<double>& y);
};


#endif
