//-------------------------------------------------------------------
//
//                        SubgraphPlanarizer.h
//
//
//    author: Carsten Gutwenger
//    date:   august 1997
//-------------------------------------------------------------------


#ifndef _AGD_SUBGRAPH_PLANARIZER_H
#define _AGD_SUBGRAPH_PLANARIZER_H


#include <AGD/PlanarizerModule.h>
#include <AGD/SubgraphModule.h>
#include <AGD/EdgeInsertModule.h>
#include <AGD/AGDModuleOption.h>


/*{\Manpage {SubgraphPlanarizer} {} {Planarization with Planar Subgraph} {P} }*/

/*{\Mbaseclasses

\begin{hierarchy}{7}{4}
\hdclass{0}{3}{5.5}{AGDModule}
\arhook{5.5}{3.5}{6.5}
\hdclass{1}{1.5}{6}{PlanarizerModule}
\arhook{7}{2}{8}
\hdclass{2.5}{0}{6}{SubgraphPlanarizer}
\end{hierarchy}
}*/

class SubgraphPlanarizer : public PlanarizerModule {

/*{\Mdefinition
...

\horizline
\commoninfo{Subgraph Planarizer}{Planarization with Planar Subgraph}{}{Carsten Gutwenger}
{august 1997}{}

\headerline{Pre- and Postcondition}
\begin{tabular}{l@{ = }l}
precondition & $\{\, |simple|,|no self loops| \,\}$ \\
postcondition(|PRE|) & $\{\, |planar|,|simple|,|no self loops| \,\}$
\end{tabular}

\headerline{Exchangeable Modules}
Instances of type |\Mname| provide the following module options:
\begin{options}
\item {\bf subgraph}\\
a |SubgraphModule| option used to compute a planar subgraph of
the input graph.

\begin{tabular}{ll}
guaranteed precondition: & $\{\, |simple|, |no self loops|,|no reversals| \,\}$\\
required postcondition: & $\{\, |planar|, |simple|,
|no self loops|,|no reversals| \,\}$\\
initial module: & |SubgraphPlanarizer|
\end{tabular}

\item {\bf inserter}\\
an |EdgeInsertModule| option used to reinsert the deleted edges.

\begin{tabular}{ll}
guaranteed precondition: & $\emptyset$ \\
required postcondition: & $\emptyset$ \\
initial module: & |ShortestPathInserter|
\end{tabular}
\end{options}\horizline
}*/

	AGD_DECLARE_MODULE(SubgraphModule,subgraph)
	AGD_DECLARE_MODULE(EdgeInsertModule,inserter)

public:

	/*{\Mcreation P 4.5}*/

	SubgraphPlanarizer ();

	/*{\Mcreate creates an instance |\Mvar| of type |\Mname|.}*/

	virtual ~SubgraphPlanarizer () { }


	/*{\Moperations 1.5 3.5}*/

	leda_string name () const {
		return leda_string ("Subgraph Planarizer");
	}

	leda_string long_name () const {
		return leda_string ("Planarization with Planar Subgraph");
	}

	virtual AGDModule *clone () const;

	/*{\Moptions warnings=no}*/

	/*{\Mtext       
	\headerline{Generic Interface (Inherited Methods)}
	The detailed description of these methods can be found in the
	manual entries of the base class (|PlanarizerModule|).
	\setlength{\entrysep}{-1ex}}*/

	/* inherited
	bool check (const GraphCopy& G, string& message);
	*/
	/*{\Mop}*/

	/* inherited
	bool call (GraphCopy& G, edge_array<list<edge> >& chain);
	*/
	/*{\Mop}*/

protected:

	virtual void _call (const leda_graph& G, PlaneGraphCopy& PG);

};



#endif
