//-------------------------------------------------------------------
//
//                        TopSortRanking.h
//
//
//    author: Carsten Gutwenger
//            Michael Krger
//    date:   february 1998
//-------------------------------------------------------------------


#ifndef _AGD_TOP_SORT_RANKING_H
#define _AGD_TOP_SORT_RANKING_H


#include <AGD/RankAssignment.h>


/*{\Manpage {TopSortRanking} {} {Ranking by Topological Sorting} {R} }*/

/*{\Mbaseclasses

\begin{hierarchy}{8}{4}
\hdclass{0}{3}{5.5}{AGDModule}
\arhook{5.5}{3.5}{6.5}
\hdclass{1.5}{1.5}{5.5}{RankAssignment}
\arhook{7}{2}{8}
\hdclass{2.5}{0}{6}{TopSortRanking}
\end{hierarchy}
}*/

class TopSortRanking : public RankAssignment {

/*{\Mdefinition
The |\Mname| module computes the following ranking of a graph $G=(V,E)$:
First, an acyclic graph $G=(V,E')$ is computed such that $G$ and
$G'$ have the same underlying undirected graph. If $G$ is acyclic,
then $G'$ equals $G$. We set
\[rank(v) = \left\{\begin{array}{cl}
1+\max \{rank(w)\mid w\in pred(v)\} & \mbox{if } pred(v)\neq\emptyset\\
0 & \mbox{otherwise}
\end{array}\right.\]
where $pred(v)=\{w\in V\mid (w,v)\in E'\}$.
}*/

public:

	/*{\Mcreation}*/

	TopSortRanking();
	/*{\Mcreate  creates an instance |\Mvar| of type |\Mname|.}*/

	~TopSortRanking() { }


	/*{\Moperations}*/

	void call (const leda_graph& G, leda_node_array<int>& rank);
	/*{\Mop calls |\Mvar| for graph $G$. Returns the rank for each
	node $v\in G$ in |rank[v]|.}*/

	AGDModule *clone() const;

private:

	int compute_depth (leda_node v, const leda_edge_array<bool>& reversed,
		leda_node_array<int>& rank);
};


#endif
