//-------------------------------------------------------------------
//
//                        generators.h
//
//    author: Carsten Gutwenger
//    date:   april 1998
//-------------------------------------------------------------------


#ifndef _AGD_GENERATORS_H
#define _AGD_GENERATORS_H


#include <LEDA/basic.h>

class leda_graph;
template<class E> class leda_node_array;


void random_tric_graph (leda_graph &G, int n, int m);
/*{\Mfuncl   creates a random triconnected planar graph $G$ with $n$
nodes and $m$ edges. The number of edges can differ slightly from $m$.
Note: For a triconnected planar graph with $n$ nodes $(n\geq 4)$ and $m$ edges
always holds $\left\lceil\frac{3}{2}n\right\rceil \leq m \leq 3n-6.$}*/

void random_hierarchy(leda_graph& G,leda_node_array<int>& rank,leda_node_array<int>& pos,
		      int n,int m,bool planar=false,bool singleSource=false);
/*{\Mfuncl creates a random hierarchical graph $G$ with $n$ nodes
and $m$ edges. Assigns for each node $v$ the layer of $v$ to |rank[v]| and
the position of $v$ on the layer to |pos[v]|. The hierarchy is planar, if
|planar| is set to |true|. It contains a single source, if |singleSource|
is set to |true|.}*/

#endif
