//-------------------------------------------------------------------
//
//                         CompOrder.h
//
//    author: Carsten Gutwenger
//    date:   february 1997
//-------------------------------------------------------------------


#include <LEDA/face_array.h>


//////////////////////////////////////////////////////////////////////////////
// Die Klasse CompOrder untersttzt die Berechnung der kanonoschen Ordnung
// Sie enthlt die Felder sepf und visited, sowie die Listen der mglichen
// Knoten bzw. Kanten. Auerdem untersttzt sie das Erhhen, Erniedrigen, und
// Aktualisieren der Variablen sepf, visited, outv, oute

class CompOrder {
public:
	// Konstruktor
	CompOrder (graph &G, bool pref_nodes) {
		prefer_nodes = pref_nodes;
		g = &G;
		// Initialisierung der Felder
		visited    .init (G, 0);
		sepf       .init (G, 0);
		v_update   .init (G, false);
		on_base    .init (G, false);
		v_link     .init (G, nil);
		outv       .init (G, 0);
		oute       .init (G, 0);
		f_link     .init (G, nil);
		is_sep_face.init (G, false);
		f_update   .init (G, false);
		outer_nodes.init (G);
	}
	~CompOrder () // Destruktor (nichts zu tun)
		{ }

	void init_outer_nodes (face F_out, edge e_left, edge e_right);
	void init_outer_edges (face F_out);
	// Initialisierung der possible-Listen: vn einziger mglicher Knoten
	void init_possible (node vn) {
		v_link [vn] = possible_nodes.append (vn);
	}
	// Gibt es noch mgliche Knoten oder Kanten ?
	bool is_possible () {
		return !(possible_nodes.empty() && possible_faces.empty());
	}
	// Ist beim letzten Aufruf von get_next_possible ein Knoten ausgewhlt
	// worden ? (sonst Flche)
	bool is_node () {
		return next_is_node;
	}
	// Testet, ob F nur noch eine Kante (u,w) an der Auenflche besitzt und
	// damit keine Seperationsflche mehr ist
	bool is_only_edge (face f) {
		return (outv [f] == 2 && oute [f] == 1);
	}
	// Bestimme nchsten Knoten oder nchste Flche (siehe auch is_node() oben)
	void get_next_possible (node &vk, face &Fk) {
		// Falls Knoten und Flchen mglich, werden Flchen bevorzugt
		if (possible_nodes.empty() || (!possible_faces.empty() &&
			prefer_nodes == false))
		{
			next_is_node = false;
			Fk = possible_faces.pop ();
		} else {
			next_is_node = true;
			vk = possible_nodes.pop ();
		}
	}
	void add_outer_node (node v, face f) {
		inc_outv (f);
		outer_nodes[f].append (v);
		if (is_sep_face[f]) sepf [v] ++;
	}
	// Erhhe visited[v]
	void inc_visited (node v) {
		visited [v] ++;
		set_update (v);
	}
	// Erhhe sepf[v]
	void inc_sepf (node v) {
		sepf [v] ++;
		set_update (v);
	}
	// Erniedrige sepf[v]
	void dec_sepf (node v) {
		sepf [v] --;
		set_update (v);
	}
	// Erhhe outv[f]
	void inc_outv (face f) {
		outv [f] ++;
		set_update (f);
	}
	// Erhhe oute[f]
	void inc_oute (face f) {
		oute [f] ++;
		set_update (f);
	}
	node get_outer_deg_2 (face f, graph &G,
		node_array<edge> &e_pred, node_array<edge> &e_succ);

	int get_outv (face f) {
		return outv [f];
	}

	void do_update ();

private:
	void set_update (node v);
	void set_update (face f);

	node_array<int>			visited, sepf;
	node_array<list_item>	v_link;           // link[v] ist das Item in possible_nodes, das v enthlt, falls
	node_array<bool> on_base;

	face_array<int>			outv, oute;
	face_array<list_item>	f_link;
	face_array<bool>		is_sep_face, f_update;
	face_array<list<node> > outer_nodes;
														 // v in possible_nodes ist, sonst nil
	list<node>	possible_nodes; // alle Knoten, die im nchsten Schritt weggenommen werden knnen
	list<face>	possible_faces; // alle Flchen,                     - " -

	node_array<bool>	v_update;     // update[v] = true <=> v in Liste update_nodes
	list<node>			update_nodes; // enthlt alle Knoten, deren Variablen im aktuellen Schritt verndert wurden
	list<face>			update_faces; //      - " -   Flchen,                       - " -
	bool				next_is_node; // wurde im aktuellen Schritt ein Knoten ausgewhlt ?
  //	node				v1, v2;       // vorgegebene v1, v2 (beiden untern Knoten des Layout)

	graph *g;
	bool prefer_nodes;

  node v_left, v_right;
  face Fout;
};
