/************************************************************************

         Filename   :   adjlists.h

         Version    :   09.1994

         Author     :   Sebastian Leipert

         Language   :   C++


 File containing the header of a class adjlists.
 It manages the adjacency lists for graphs.

************************************************************************/ 



#ifndef ADJLISTS_H
#define ADJLISTS_H

#include <AGD/impl/MaxPlanSub/def.glb>


class adjlists {
     int       size;          // number of vertices, is as well the
                              // the number of lists needed to store the
                              // graph
     int       *degree;       // array containing node degrees
     adjptr    *first;        // array containing pointers to the begining
                              // of each list
     adjptr    *last;         // array containing pointers to the end of
                              // each list
     adjptr    *position;     // array containg pointers to an arbitrary 
                              // position in the list
  public:
     adjlists(int m);
     ~adjlists();

     int end_of_list(int m);    // True, if end of list m is reached.
     int read_next_node(int m); // Read next neighbor in list m. 
     void insert_edge(int node1,int node2);
                                // Insert edge by saving the new neighbors
                                // in the lists of node1 and node2.
     void mark_old_edge(int m); // Mark vertex before position pointer as visited.
     int old_edge_marked(int m);// True if vertex before position pointer is 
                                // already visited.
     void start_at_first_node(int m);  
                                // Set position = first.
     int node_degree(int m);    // Get vertex degree.

     void delete_old_edge(int m);
                                // Remove in list m the neighbor just before
                                // the position pointer. Find list of
                                // neighbor and remove correspondingly.
     void delete_edge(int node1,int node2);
                                // Delete edge (node1,node2) in both
                                // adjacency lists.

     int get_size() const {return size;}
};
    


#endif

