/************************************************************************

         Filename   :   pq_node.h

         Version    :   09.1994

         Author     :   Sebastian Leipert

         Language   :   C++


 File containing the header for the class pq_node.
 This class is used to represent a node in a PQ-tree.
 It can be a Q-node, a P-node or a leaf.
 All elements are public.

************************************************************************/ 


#ifndef PQ_NODE_H
#define PQ_NODE_H

#include <AGD/impl/MaxPlanSub/def.glb>
#include <AGD/impl/MaxPlanSub/stack.h>


class pq_node {

  public: 
    int           typ;               // type of node: Q-NODE, P-NODE, LEAF
    int           label;             // label of node: EMPTY, FULL, PARTIAL,
                                     // PERTINENT, IGNORED
    int           key;               // number of the represented vertex in bushform B_k
    int           leafnum;           // number of leaf
    int           total_count;       // total number of node
    int           w_nr;              // w-number
    int           a_nr;              // a-number
    int           h_nr;              // h-number
    int           del_typ;           // type w, b, h, a
    int           child_count;       // number of children
    int           visited_child_count; // number of visited children
    int           not_visited_child_count; // number of not visited children
    int           pert_child_count;  // number of relevant children
    int           pert_leaf_count;   // number of relevant leaves in frontier

    int           debugTreeNumber;    // Added on behalve of debugging the
                                      // with the TreeInterface.
					 
    pq_node       *first_full;       // pointer to the first full child
                                     // of a Q-node
    pq_node       *sib_next;         // pointer to a brother 
    pq_node       *sib_pre;          // pointer to a second brother
                                     // gets a doubly linked list
                                     // of children of a P-node 
    pq_node       *child_sib_list;   // pointer of parent to the
                                     // doubly linked list of children.
    pq_node       *sib_list_father;  // pointer of children of a doubly
                                     // linked list to parent, used by
                                     // just one child of the list.
    pq_node       *parent;           // pointer to parent
    pq_node       *h_child1;         // pointer to h-child1
    pq_node       *h_child2;         // pointer to h-child2
    pq_node       *a_child;          // pointer to a-child
    stack<pq_node*> *full_children;    // stack with full children
    stack<pq_node*> *partial_children; // stack with partial children
    stack<pq_node*> *endmost_children; // stack with endmost children if
                                       // node is Q-node
    stack<pq_node*> *imm_siblings;     // stack with next siblings if node
                                       // is child of a Q-node
                                      

    pq_node(int m,int n,int count,int type, int lbl);
                                       // Creates a new node
    ~pq_node();                        // Deletes a node. 
};




#endif


