/************************************************************************

         Filename   :   stack.h

         Version    :   09.1994

         Author     :   Sebastian Leipert

         Language   :   C++


 File containing the header for the class stack, a last-in first-out stack.

************************************************************************/ 




#ifndef STACK_H
#define STACK_H

#include <stdio.h>
#include <stdlib.h>
#include <AGD/impl/MaxPlanSub/def.glb>


template<class T>
class T_elem{
 public:
   T      key;
   T_elem *next;
   T_elem *pre;
};



template<class T>
class stack {
    T_elem<T>  *first;        // pointer to the bottom element of the stack
    T_elem<T>  *last;         // pointer to the top element of the stack 
    T_elem<T>  *position;     // pointer to an arbitrary element of the stack,
                              // is used to read all elements of the stack
    int        size;          // number of elements in the stack       
  public:
    stack();                  // constructor of the class stack
    ~stack();                 // destructor of the class stack 

    int stack_empty();        // true if stack is empty, false otherwise
    void empty_stack();       // empties the complete stack and frees the
                              // memory
    void set_empty();         // empties the complete stack without freeing
                              // the memory
    void push(T m);           // push element T on top of the stack
    T pop();                  // pop top element out of stack
    void start_at_bottom();   // put position pointer to bottom element
    T read_next();            // get element at position 
    int read_last();          // true if the last element was read,
                              // otherwise false        
    int count();              // get the number of elements in the stack
    int search_and_change(T oldel,T newel);
                              // search for the old element and replace it
                              // by the new element
};




template<class T> stack<T>::stack()
{
   first = NULL;
   last = NULL;
   position = NULL;
   size = 0;
} // constructor


template<class T> stack<T>::~stack()
{
   T_elem<T>   *helpptr;
   
   while (first != NULL)
   {
      helpptr = first;
      first = first->next;
      delete helpptr;
   }
} //destructor


template<class T> int stack<T>::stack_empty()
{
   if (first == NULL)
     return TRUE;
   else
     return FALSE;
}  // stack_empty


template<class T> void stack<T>::empty_stack()
{
   T_elem<T>   *helpptr;
 
   while (first != NULL)
   {
      helpptr = first;
      first = first->next;
      delete helpptr;
   }
   last = NULL;
   position = NULL;
   size = 0;
} // empty_stack



template<class T> void stack<T>::set_empty()
{
   first = NULL;
   last = NULL;
   position = NULL;
   size = 0;
} // set_empty




template<class T> void stack<T>::push(T m)
{
   T_elem<T>   *helpptr;

   size++;
   helpptr = new T_elem<T>;
   helpptr->key = m;
   helpptr->next = NULL;
   if (first == NULL)
   {
      first = helpptr;
      last = helpptr;
      position = helpptr;
      helpptr->pre = NULL;
   }  
   else
   {
      last->next = helpptr;
      helpptr->pre = last;
      last = helpptr;
   }
} // push
      





template<class T> T stack<T>::pop()
{
   T   helpelem;
   T_elem<T>   *helpptr;


   if (last != NULL)
   {
      if (first == last)
         first = NULL;
      if (position == last)
         position = NULL;
      helpelem = last->key;
      helpptr = last;
      last = last->pre;
      if (last != NULL)
         last->next = NULL;
      size--;
      delete helpptr;
      return helpelem;
   }
   else
   {
      printf("template stack: stack is empty. wrong access\n");
      return 0;
   }
} // pop




template<class T> void  stack<T>::start_at_bottom()
{
   position = first;
} // start_at_bottom



template<class T> T stack<T>::read_next()
{
   T   helpelem;

   if (position != NULL)
   {
      helpelem = position->key;
      position = position->next;
      return helpelem;
   }
   else
   {
      printf("template stack: Tried to read nonexisting element\n");
      return 0;
   }
} // read_next



template<class T> int stack<T>::read_last()
{
   if (position == NULL)
      return TRUE;
   else 
      return FALSE;
} // read_last




template<class T> int stack<T>::count()
{
   return size;
} // count




template<class T> int stack<T>::search_and_change(T oldel,T newel)
{
   int  found = FALSE;
   
   position = first;
   while (position != NULL && !found)
   {
      if (position->key == oldel)
      {
          position->key = newel;
          found = TRUE;
      }
      position = position->next;
   }
   return found;
} // search_and_change
   

#endif


    
