
#include <LEDA/array2.h>
#include <LEDA/list.h>
#include <LEDA/graph.h>

class mm_ordering;
class CanonicalOrder;

struct conc_edge_struct {
	int id, src, tgt;

	LEDA_MEMORY(conc_edge_struct)
};

ostream& operator<< (ostream& os, const conc_edge_struct& e);
istream& operator>> (istream& is, conc_edge_struct&);

ostream& operator<< (ostream&, const list<int>&);
istream& operator>> (istream&, list<int>&);
ostream& operator<< (ostream&, const list<edge>&);
istream& operator>> (istream&, list<edge>&);


class conc_graph {
public:
	conc_graph (const CanonicalOrder& C, const mm_ordering& mmo);

	static const int undef, down, up;

	int num_nodes() const {
		return _n;
	}

	int num_edges() const {
		return _m;
	}

	int source(int e) const {
		return _edges[e].src;
	}

	int target(int e) const {
		return _edges[e].tgt;
	}

	int dir(int e) const {
		return (source(e) < target(e)) ? up : down;
	}

	int pred_path(int i) const;

	bool splitable(edge e) {
		return _splitable[e];
	}

	int dep(edge e) {
		return _dep[e];
	}

	void dep(edge e, int ec) {
		_dep[e] = ec;
		_must_split[ec].append(e);
	}

	void cannot_split(int e, edge es) {
		if (e != undef && es != nil) {
			_cannot_split[e].append(es);
			//_excl[es].append(e);
		}
	}

	void print_edge (ostream& os, int e) const {
		if (e == undef) os << "undef"; else os << _edges[e];
	}
	void print_split (ostream& os);
	void print_dependencies (ostream& os);

	int in(int v, int dir) const {
		return _in(v,dir);
	}

	int out(int v, int dir) const {
		return _out(v,dir);
	}

	int new_edge (int from, int to);

	bool del_edge (int e);
	bool del_edge (int e, list<int>& dels);

	void select_edge (int e) {
		list<int> del;
		select_edge(e,del);
	}
	void select_edge (int e, list<int>& del);
	void select_split (edge es, list<int>& del);
	void remove_split (edge es, list<int>& del);

	void get_splitable();
	void get_dependencies();

	void init_must();

private:
	int _make_edge(int from, int to);
	bool common_neighbour(int i, int j);

	array<conc_edge_struct> _edges;
	array2<int> _in, _out;

	edge_array<edge> _next_dep_split, _prev_dep_split, _other_split;
	edge_array<bool> _splitable;
	edge_array<int> _dep;
	//edge_array<list<int> > _excl;
	//array<list<int> > _exl_edges;
	array<list<edge> > _must_split, _cannot_split;

	int _n;
	int _m;
	const CanonicalOrder *_C;
	const mm_ordering *_mmo;
};
