/************************************************************************

            Filename    :  tree.h

	    Version     :  AGD-Version 01.1997

	    Author      :  Sebastian Leipert

	    Language    :  C++

            Purpose     :  A derived class from basic_tree. Computes
	                   and manages the coordinates of all nodes
			   of a tree for a nice layout. Uses an
			   algorithm by John Q. Walker published in:
			   Software-Practice and Experience, vol. 20(7),
			   685-705 (July 1990) under the titel:
			   A Node-positioning Algorithm for General Trees.

************************************************************************/


#ifndef _AGD_IMPL_TREE_H
#define _AGD_IMPL_TREE_H

#include <AGD/impl/TreeLayout/basic_tree.h>
#include <AGD/LayoutModule.h>

class Tree : public basic_tree  {

public:

   Tree();
   ~Tree();


// AGD-Lib Funktions - begin

   void    allocateTree(int nodeCount,int edgeCount);
   void    allocate_node_sizes(leda_node *nodes,LayoutInterface& a);
   bool    enter_edge(int source,int target,int edgeNumber);

// AGD-Lib Funktions -end


   void    positiontree(double*, double*);
   void    newPositionLevel(int, double*, double*);

   void    enter_new_node(int, int, int);
   void    clean_tree();
   
   int     get_level_count() const;
   double  get_actual_level_separation() const;
   double  get_max_X_coord() const;
   double  get_max_Y_coord() const;

   
   static double get_sib_separation();
   static double get_level_separation();
   static double get_subtree_separation();
   static double get_left_margin();
   static double get_upper_margin();

   void set_separation_values(double sibling,double level,double subtree);
   void set_sib_separation(double);
   void set_level_separation(double);
   void set_subtree_separation(double);
   void set_left_margin(double);
   void set_upper_margin(double);

private:

   double   root_X_coord;
   double   root_Y_coord;

   int      level_count;
   double   actual_level_sep;
   double   max_X_coord;
   double   max_Y_coord;

   double*  level_adjust;
   double*  real_level;
   
   static double sibling_separation;
   static double level_separation;
   static double subtree_separation;
   static double left_margin;
   static double upper_margin;
      
   void   compute_coord(vbc_node *,double*,double*,double*);
   void   firstwalk(vbc_node &,int,vbc_node*[]);
   void   secondwalk(vbc_node *, int, double,double*,double*,double*);

   void       apportion(vbc_node &,int);
   vbc_node*  get_left_most(vbc_node *,int,int);
   
   void   get_max_XYvalue(vbc_node*, double*, double*, double*);

   void   deapth_first_search(vbc_node *,int);
   void   del_deapth_first(vbc_node *);
   double mean_node_size(vbc_node *,vbc_node *);
   void   introduce_child(vbc_node *,vbc_node *);


   void  level_correction_step();
};


#endif




