#ifndef CLASS_EDGEVAR
#define CLASS_EDGEVAR


#undef set
#undef list

#include <abacus/numvar.h>
#include <abacus/srowcon.h>

#define set leda_set
#define list leda_list


#include <LEDA/graph.h>

class PLANARISATOR;
class PLAU_MASTER;

enum kind_of_edge {
  NOT_CONNECTED,    // Edge added to connect two components (in MAKE_CONNECTED)
  OUTSIDE_PENDANTS, // connects two nodes belonging to different pendants
  NON_PENDANTS,     // connects two non-pendant blocks (later called "BadBlocks")
  MIXED_BLOCKS      // connects a BadBlock to pendant
};
/*****************************************************************************/
// EDGEVAR describes variable in this Branch-and-Cut Algorithm.
// EDGEVAR's corresponds to edges in the Input graph, that belong to big (>4 nodes)
// blocks. An EDGEVAR stores the source and target of the corresponding edge.
// Sometimes an EDGEVAR can be activated, it means the corresponding edge will be
// added to the scratch graph. Then edge_ will points to the added edge.
 
class EDGEVAR : public ABA_NUMVAR 
{
  public:
    EDGEVAR (PLANARISATOR *m, node s, node t, edge e, int num, double weit = 1.0) :
    ABA_NUMVAR ((ABA_MASTER*)m, 0, num, false, false, weit, 0.0, 1.0, ABA_VARTYPE:: Binary),
    source_(s), target_(t), edge_ (e){}

    inline edge Edge () { return edge_; }
    inline edge reversal () { return reversal_; }
    inline node source () { return source_; }
    inline node target () { return target_; }
    inline void del_edge () { edge_ = nil; reversal_ = nil; }
    inline void set_edge (edge e) { edge_= e; }
    inline void set_reversal (edge e) { reversal_ = e; }

  protected:
    node source_;
    node target_;
    edge edge_;
    edge reversal_;
};
/*****************************************************************************/
// PLAU_EDGEVAR describes a Variable in this Branch-and-Cut Algorithm.
// PLAU_EDGEVAR's corresponds to new edges in the Input graph. With these new
// edges we augment the input graph. A PLAU_EDGEVAR stores always the
// source and target of the corresponding edge. A PLAU_EDGEVAR can sometimes be activated,
// it means the corresponding edge will be added to the scratch
// graph. In this case the member edge_ will be initialised. Member planar_ is true
// if adding of edge to original Graph preserve planarity.
 
class PLAU_EDGEVAR : public ABA_NUMVAR
{
  public:
    PLAU_EDGEVAR(PLAU_MASTER *m, node start, node end, int num, bool plan = false, 
	    kind_of_edge ek = OUTSIDE_PENDANTS);
    inline edge Edge () { return edge_; }
    inline kind_of_edge kind () { return kind_; }
    inline node source () { return source_; }
    inline node target () { return target_; }
    inline void del_edge ()  { edge_ = nil; }
    inline void set_edge (edge e) { edge_= e; }
    inline bool is_planar ()  { return planar_; }
    inline void set_planar ()  { planar_ = true; }
    void print ();

  private:
    edge edge_;
    node source_;
    node target_;
    kind_of_edge kind_;
    bool planar_;
};
//**************************************************************************
class KURATOWSKI_EDGE : public EDGEVAR
{
  friend  class PLANARISATOR;
  public:
    KURATOWSKI_EDGE (PLANARISATOR *m, node s, node t, edge e, int num, double w) :
	EDGEVAR (m, s, t, e, num, w) {}
};
#endif
