#ifndef CLASS_PLANARISATOR
#define CLASS_PLANARISATOR


#undef set

#include <abacus/master.h>
#include <abacus/sub.h>
#include <abacus/numvar.h>
#include <abacus/srowcon.h>
#include <abacus/cutbuffer.h>
#include <abacus/poolslot.h>

#define set leda_set


#include <LEDA/graph.h>
#include <LEDA/h_array.h>

#include <AGD/impl/bctree.h>
#include <AGD/impl/cons.h>
#include <AGD/impl/edgevar.h>


class PLANAR_SUB;
class PLANARISATOR;

typedef list<edge> elist;

//**************************************************************************
class VarImportance {
  private:
     array<int> numRefs_;
  public:
     VarImportance (int i): numRefs_ (i) { numRefs_.init (0); }
     inline void incrImportance (int var) { (numRefs_[var])++; } 
     inline void decrImportance (int var) { (numRefs_[var])--; }
     inline void reduceImportance (int var) { (numRefs_[var])--; }
     inline int Importance (int var)  { return numRefs_[var]; }
     int find_max ();
     inline int nVars () { return numRefs_.high (); }
     inline void print () { numRefs_.print (); }
};

//***************************************************************************
// The derived Class Master for Planarization Problem

class PLANARISATOR : public ABA_MASTER
{
    friend class PLANAR_SUB;
    friend class KURATOWSKI_EDGE;
    friend class PLANAR_CONSTRAINT;
 private:
    ABA_SUB* firstSub ();
    graph *G;
    
    edge_map <KURATOWSKI_EDGE*> edge2evar;

    edge_array <int> weights_;

    inline KURATOWSKI_EDGE* evar_for (edge e) { return edge2evar[e]; }
    
    int LowerB; //result of Approximation

    void initializeOptimization ();

    void terminateOptimization ();

    list<int> best_found_solution;// If a solution which is better than approxim.
				// solution (LowerB) can't be found then the list remains empty.

    void output ();
    
    int numEdges; //debug

    int numLastCons; //Number of last generated PLANAR_CONSTRAINT

    int nKuratowski; // statistics

    int separatedFromPool; // statistics

    long int destroyedFromPool; // statistics

    ABA_CPUTIMER *searchInPool; //Timer for pool searching

    ABA_STANDARDPOOL<ABA_CONSTRAINT, ABA_VARIABLE> *CutPool;

    PLANAR_CONSTRAINT* createCon (list<edge>& edges);
    
    void update_solution (list<int>&);

    inline KURATOWSKI_EDGE* variable (int i)
    { return (KURATOWSKI_EDGE*)(varPool()->slot(i)->conVar ()); } 

    inline edge restore_edge (KURATOWSKI_EDGE* evar); // Adds evar->edge_ to the graph

    inline void remove_edge (KURATOWSKI_EDGE *evar); // Removes evar->edge_ from the graph

    void print_evar (KURATOWSKI_EDGE* evar, double val = 0 );

    void print_constraint (ABA_SROWCON* c, ABA_SUB* s);

    bool LogStatisticsToFile; // Parameter

    ABA_STRING StatisticsFileName;

    bool MoreSeparate;

    bool BranchMoreReferences ;

  public:
    PLANARISATOR (graph &Gr,
         int LowerBound = 0,
	 double eps = 1.0e-7,
	 double machineEps = 1.0e-10);
//    ~PLANARISATOR ();

    void set_weights (edge_array<int>&);
    
    list<node> solution (); // Solution is a list of pairs of nodes

    inline bool found ()
       { return !best_found_solution.empty () ? true : false; }

    inline int nCuts () { return CutPool->number (); }

    inline int nVars () { return varPool()->number (); }

  protected:
    int equalSubCompare(ABA_SUB *s1, ABA_SUB *s2);
};

//****************************************************************************
class PLANAR_SUB : public ABA_SUB //Derived from ABA_SUB Class
{
    friend class PLANARISATOR;
/*    enum CONSTRAINT_STATUS { 
      ACTIVE, // is active
      POOL,   // is stored in cutPool, but dont belong to the set of active constraint
      NEUTRALIZED // one of variables presented in this constraint is set to 0
    };*/
      
  public :
    PLANAR_SUB (ABA_MASTER*);

    PLANAR_SUB (ABA_MASTER* , ABA_SUB *father, ABA_BRANCHRULE *brule);

    bool feasible ();

    ABA_SUB* generateSon (ABA_BRANCHRULE *brule)
	{ return new PLANAR_SUB (master_, this, brule); }

  private:
    VarImportance vI;
    
    check_importances (); // debug
    
    inline void incrImportances (PLANAR_CONSTRAINT* con);

    h_array<PLANAR_CONSTRAINT*, bool> *neutralized;

// some shortcuts
    inline PLANARISATOR* planarisator () { return (PLANARISATOR*)master_; }

    inline graph* Graph () { return planarisator ()->G; }

    inline KURATOWSKI_EDGE* edge_var ( ABA_VARIABLE* var ) { return (KURATOWSKI_EDGE*)var; }

    inline KURATOWSKI_EDGE* edge_var (int i) { return (KURATOWSKI_EDGE*)variable (i);  }

    inline KURATOWSKI_EDGE* edge_var ( edge e ) { return planarisator()->evar_for (e); }

    inline int number (edge e) { return edge_var (e)->number (); }
    
    inline void remove_edge (int i) { planarisator()->remove_edge (edge_var (i)); }
    
    inline edge restore_edge (int i) { return planarisator()->restore_edge (edge_var (i)); }

    inline edge Edge(ABA_VARIABLE* var) {return ((KURATOWSKI_EDGE*)var)->Edge (); }

// Divide a found solution into variables with small and large values
    void filter_solution ( list<int>& large, list<int>& small );

   int SeparationFromCutsPool (list<int>&, ABA_BUFFER<ABA_CONSTRAINT*>&);

   int DestroyKuratowskiG (PLANAR_CONSTRAINT* con, list<int>& l);

   bool branch; // geht es damit??

#undef set    
    inline bool IsLocalSet (int i) { return fsVarStat (i)->fixedOrSet () || 
                                     variable (i)->fsVarStat ()->fixedOrSet (); }
#define set leda_set
    

  protected:

    int separate ();

    bool exceptionBranch();

    void initializeCons(int maxCon);

    void fathom (bool reoptimize);
    
    inline bool MoreSeparate () { return planarisator()->MoreSeparate ; }

    inline int selectBranchingVariable(int &variable);
};

/**********************************************************************/
#endif
