//-------------------------------------------------------------------
//
//                       agd_serv.C
//
//    author: Gunter Koch
//    date:   Juni 1998
//-------------------------------------------------------------------


#include <LEDA/agd_serv_base.h>
#include <stdio.h>
//#include "agd_serv_base.c"
#include <LEDA/graphwin.h>

#include <AGD/FPPLayout.h>
#include <AGD/SchnyderLayout.h>
#include <AGD/ConvexLayout.h>
#include <AGD/ConvexDrawLayout.h>
#include <AGD/PlanarStraightLayout.h>
#include <AGD/PlanarDrawLayout.h>
#include <AGD/MixedModelLayout.h>
#include <AGD/TreeLayout.h>
#include <AGD/PlanarizationLayout.h>
#include <AGD/SugiyamaLayout.h>
#include <AGD/SpringLayout.h>
#include <AGD/TutteLayout.h>
#include <AGD/Ortho.h>
#include <AGD/Quod.h>
#include <AGD/VisibilityRepresentation.h>
#include <AGD/VisibilityMapper.h>
#include <AGD/GraphWinInterface.h>

/*
#include <AGD/fastHierarchyLayout.h>
*/
#include <AGD/FastHierarchyLayout.h>

#include <AGD/DfsRanking.h>
#include <AGD/TopSortRanking.h>
#include <AGD/BarycenterHeuristic.h>
#include <AGD/MedianHeuristic.h>
#include <AGD/GreedyInsertHeuristic.h>
#include <AGD/GreedySwitchHeuristic.h>
#include <AGD/SplitHeuristic.h>

#include <AGD/SubgraphPlanarizer.h>
#include <AGD/PlanarSubgraph.h>
//#include <AGD/MaximumPlanarSubgraph.h>
#include <AGD/ShortestPathInserter.h>

#include <fstream.h>

class agd_serv_class : agd_serv_base {

private:

   FPPLayout	 		fpp;
   SchnyderLayout   		schn;
   ConvexLayout     		convex;
   ConvexDrawLayout 		convexd;
   PlanarStraightLayout 	planarst;
   PlanarDrawLayout     	planardr;
   MixedModelLayout  		mix;
   TreeLayout 			tree;
   PlanarizationLayout	        planar;
   SugiyamaLayout		sugiyama;
   SpringLayout 		spring;
   TutteLayout 			tutte;
   VisibilityRepresentation 	visibility;
   Ortho			ortho;
   Quod				quod;
 //fastHierarchyLayout 		hierarchy;
   FastHierarchyLayout 		hierarchy;


   bool error;
   int argc;
   string in_file_type;
   string out_file_type;
   bool infos;
   char ** argv;
   GraphWin gw;

   void callmake_std_infos (ofstream& opt,LayoutModule& F);

   void callmake_infos_only (ofstream& opt,LayoutModule& F);

   void call_make_fpp (ofstream& opt);
   void call_make_schn (ofstream& opt);
   void call_make_convex (ofstream& opt);
   void call_make_convexd (ofstream& opt);        
   void call_make_planarst (ofstream& opt);  
   void call_make_planardr (ofstream& opt);  
   void call_make_mix (ofstream& opt);
   void call_make_tree (ofstream& opt);
   void call_make_sugiyama (ofstream& opt);
   void call_make_spring (ofstream& opt);
   void call_make_tutte (ofstream& opt);
   void call_make_visibility (ofstream& opt);
   void call_make_ortho (ofstream& opt);
   void call_make_quod (ofstream& opt);
   void call_make_planar (ofstream& opt);

   bool calltest(); 

   bool callfunktion (LayoutModule& L);
   
   bool callfunktionG(GridLayoutModule& F);

   bool callfunktionGv(GridLayoutModule& F);

   void search_struct(LayoutModule& F,agd_serv_struct_def& opt_fkt_struct);

   bool callfppfunktion (GridLayoutModule& F); 

   bool callschnyderfunktion (GridLayoutModule& F);

   bool callmixfunktion (GridLayoutModule& F);

   bool callplanarstfunktion (GridLayoutModule& F);
 
   bool callsugiyamafunktion (LayoutModule& F);

   bool callconvexfunktion (GridLayoutModule& F);

   bool callconvexdrawfunktion (GridLayoutModule& F);

   bool callplanardrfunktion (GridLayoutModule& F);

   bool calltreefunktion (LayoutModule& F);

   bool callspringfunktion (LayoutModule& F);

   bool calltuttefunktion (LayoutModule& F);

   bool callorthofunktion (LayoutModule& F);

   bool callquodfunktion (LayoutModule& F);

   bool callvisibilityfunktion (GridLayoutModule& F);

   bool callplanarfunktion (LayoutModule& F);

   bool errorfkt (string&  msg);
   
   string inline out_bool(bool b);
   
   bool read_graph();
   
   bool write_graph(node_array<int> & n_index,edge_array<int> & e_index , graph& G);


public:

agd_serv_class(int argc_h, char* argv_h[]) : 
                                      gw(string("%s(%s)",argv_h[0],argv_h[1]))
{
   if (argc_h >= 4) 
      in_file_type = argv_h[4];
   else
      in_file_type = "gw";

   if (argc_h >= 5) 
     out_file_type = argv_h[5];
   else
     out_file_type = "gw";

   argc = argc_h - 1; 
   argv = argv_h;
   infos = false;
}




   ~agd_serv_class() {}


   void init (int& argc_h, char * argv_h[]) {
      argc = argc_h - 1; 
      argv = argv_h;
   }   
   
   void init (int& argc_h, char * argv_h[],string _in_file_type,string _out_file_type) {
      in_file_type = _in_file_type;
      out_file_type = _out_file_type;
      argc = argc_h - 1; 
      argv = argv_h;
   } 

   void file_typ (string _in_file_type,string _out_file_type) {
      in_file_type = _in_file_type;
      out_file_type = _out_file_type;   
   }

   string name () const {
      return string ("agd_server");
   }

   string long_name () const {
      return string ("agd_server");
   }

   string author () const {
      return (" ");
   }

   string impl_author () const {
      return string ("Gunter Koch");
   }

   string impl_version () const {
      return "1.0";
   }

   string impl_date () const {
      return string ("Juni 1998");
   }

   
   bool agd_serv();



};


inline void del_file(string fname)
{
#if defined(__win32__)
  system ("del " + fname + " > NUL");
#else
  system ("rm -f " + fname);
#endif
}



inline string agd_serv_class::out_bool(bool b) {
   if (b) return "\"true\"";
   else return "\"false\"" ;
}
      

/******************** Funktion callmake_std_infos ********************/
                                   // schreibt ins bergebene Ausgabefile
                                   // Informationen ber die jeweilige
                                   // AGD-Funktion 

void agd_serv_class::callmake_std_infos (ofstream& opt,LayoutModule& F){
   opt << "\t\twork_infos " << out_bool(infos) << endl;
   opt << "\t\tinfos [" << endl;
      opt << "\t\t\tname: \"" << F.long_name() << "\"" <<  endl;
      opt << "\t\t\tauthor: \"" << F.author() << "\"" << endl;
      opt << "\t\t\tImplementation: \"" << F.impl_author() << "\"" << endl;
      opt << "\t\t\tdate: \"" << F.impl_date() << "\"" << endl;
      opt << "\t\t\tversion: \"" << F.impl_version() << "\"" << endl;
    opt << "\t\t]" << endl;
}
 


void agd_serv_class::callmake_infos_only (ofstream& opt,LayoutModule& F) {
   	opt << "\tfunction \"" <<  F.name() << "\" ["<< endl;
            callmake_std_infos (opt,F);       
        opt << "\t]" << endl;     
}



/********* Funktion calltest fr Funtionsbergabe  ********************/  
 

bool agd_serv_class::calltest() {

  if (argc > 1) {	
                                    
     ofstream out (argv[2],ios::trunc | ios::app);
     if ( ! out ) return false;  		  	 
            
     
     out << "Creator \"AGD_SERVER\""<< endl;
     out << "Version " << impl_version() << endl;
     out << "AGD_function_list [" << endl;
     out << "\tfunction \"" <<fpp.name() << "\"" << endl;
     out << "\tfunction \""<< schn.name() << "\""<< endl;
     out << "\tfunction \""<< convex.name() << "\""<< endl;
     out << "\tfunction \""<< convexd.name()<< "\"" << endl;
     out << "\tfunction \""<< planarst.name() << "\""<< endl;
     out << "\tfunction \""<< planardr.name() << "\""<< endl;
     out << "\tfunction \""<< mix.name() << "\""<< endl;
     out << "\tfunction \""<< ortho.name() << "\""<< endl;
     out << "\tfunction \""<< quod.name() << "\""<< endl;
     out << "\tfunction \""<< tree.name() << "\""<< endl;
     out << "\tfunction \""<< sugiyama.name() << "\""<< endl;
     out << "\tfunction \""<< planar.name() << "\""<< endl;
     out << "\tfunction \""<< spring.name() << "\""<< endl;
     out << "\tfunction \""<< tutte.name() << "\""<< endl;
     out << "\tfunction \""<< visibility.name() << "\""<< endl;
     out << "]" << endl;
     
     out.setf(ios::showpoint);

     out << "AGD_option_list [" << endl;    
        call_make_fpp (out);
        call_make_schn (out);
        call_make_convex (out);
        call_make_convexd (out);        
        call_make_planarst (out);  
        call_make_planardr (out);  
        call_make_mix (out);
        call_make_ortho (out);
        call_make_quod (out);
        call_make_tree (out);
        call_make_sugiyama (out);
        call_make_planar(out);
        call_make_spring (out);
        call_make_tutte (out);
        call_make_visibility (out);
     out << "]" << endl;
     out.close(); 
     return true;	
 } 
 return false;
}



//****************************** callfunktion ******************************/
// fr Aufruf der einzelnen Funktionen verantwortlich



bool agd_serv_class::read_graph() {
 if (in_file_type == "gml") {
          	if (gw.read_gml (argv[2])) { 
                   string msg (" can't read Graph-gml-file ");
                   errorfkt(msg); return false;
                }return true;
 }
 if (in_file_type == "gw") {
          	if (gw.read (argv[2])) { 
                   string msg (" can't read Graph-graphwin-file ");
                   errorfkt(msg);return false;
                } return true;
 }
 if (in_file_type == "layout") {
                ifstream in (argv[2]);
                if(gw.load_layout (in)) {
                   string msg (" can't read Graph-graphwin-file ");
                   errorfkt(msg);return false;
                } return true;
  }
  string msg (" can't read this fileformat"); 
  errorfkt(msg);return false;
}


bool agd_serv_class::write_graph(node_array<int> & n_index,edge_array<int> & e_index , graph& G) {
        if (out_file_type == "gml")  return gw.save_gml (argv[2]);
        if (out_file_type == "gw")  return gw.save(argv[2]);
        if (out_file_type == "ps")  return gw.save_ps(argv[2]);
        if (out_file_type == "layout") {
    	    ofstream out (argv[2]);
            G.sort_nodes(n_index);
            G.sort_edges(e_index);
            gw.save_layout(out);
            return true;
           }
        string msg (" can't read this fileformat"); 
        errorfkt(msg);return false;
}
        



bool agd_serv_class::callfunktion (LayoutModule& F) {
 string msg;
 node v;
 streambuf* strm_buffer;

    
  if (!read_graph()) return false;
  
  del_file(argv[2]);

  
   graph& G = gw.get_graph();

   node_array<int> n_index(G);
   edge_array<int> e_index(G);

   int i = 0;
   forall_nodes(v,G) { 
     n_index[v] = index(v);
     edge e;
   forall_adj_edges(e,v) e_index[e] = i++;
   }  

 GraphWinInterface I (gw);
 
 if (!F.check (gw.get_graph(),msg)) 
 { errorfkt(msg);
   return false;
  }
 
#if !defined(__win32__)

       if ((infos==true) && (argc > 2)) { 
     	  ofstream ans (argv[3], ios::trunc | ios::app);
          if (! ans) {
             return false;
          } 
          ans << "AGD_Information\t\"";
          strm_buffer = cout.rdbuf();
	  cout.rdbuf(ans.rdbuf()); 
          F.call(gw.get_graph(),I); 
	  cout.rdbuf(strm_buffer);
          ans << "\"\n";     
        }else {

          string ansfile (tmpnam(NULL));
     	  ofstream ans (ansfile, ios::trunc | ios::app);
          strm_buffer = cout.rdbuf();
	  cout.rdbuf(ans.rdbuf());
          F.call(gw.get_graph(),I); 
	  cout.rdbuf(strm_buffer);
          ans.close();
          del_file(ansfile);
        }    
#endif

  return write_graph(n_index,e_index,G);
}



bool agd_serv_class::callfunktionG(GridLayoutModule& F) {
   MaxAllMapper mapper (5,5,5);
   F.set_coordinate_mapper (mapper);
   return (callfunktion(F));
}



bool agd_serv_class::callfunktionGv(GridLayoutModule& F) {
   node v;
   VisibilityMapper vis_mapper (5,5,5,5,5);
   F.set_coordinate_mapper (vis_mapper);
   gw.set_node_shape (rectangle_node);
   return (callfunktion(F));
}


void agd_serv_class::search_struct(LayoutModule& F,
                        	agd_serv_struct_def& opt_fkt_struct) {

      	agd_serv_struct_def opt_fkt_struct_h;
	
        forall (opt_fkt_struct_h,fkt_opt_list) {
           if(opt_fkt_struct_h.fkt == F.name())
                 	opt_fkt_struct = opt_fkt_struct_h;
        }
}


/********************* Ausgabe der Fehlermeldungen ************************/

bool agd_serv_class::errorfkt (string&  msg) {
   if (argc > 1) {
      ofstream out (argv[2]);
      if ( ! out )  return false;
      out << "AGD_error_file\t\"";
      out << msg; 
      out << "\"\n";
      out.close(); return true;
   } else return false;
}



bool agd_serv_class::agd_serv() {

 infos = false;

 if (argc < 2 ) return false;

 string funktion = argv[1];
  
 if (funktion == "check") return(!calltest());
  
  if (argc < 3 ) {
     string msg = "error in parameterlist";
     errorfkt (msg);return false;
  }
	
  if ( funktion == fpp.name()) { error = callfppfunktion (fpp);}	     
  else if (funktion == schn.name()) { error = callschnyderfunktion (schn);}
  else if (funktion == convex.name()) { error = callconvexfunktion (convex);}
  else if (funktion == convexd.name()) { error = callconvexdrawfunktion (convexd);}
  else if (funktion == planarst.name()) { error = callplanarstfunktion (planarst);}    
  else if (funktion == planardr.name()) { error = callplanardrfunktion (planardr);}
  else if (funktion == mix.name()) { error = callmixfunktion (mix);}
  else if (funktion == tree.name()) { error = calltreefunktion (tree);}
  else if (funktion == planar.name()) { error = callplanarfunktion (planar);}
  else if (funktion == sugiyama.name()) { error = callsugiyamafunktion (sugiyama);}
  else if (funktion == spring.name()) { error = callspringfunktion (spring);}
  else if (funktion == tutte.name()) { error = calltuttefunktion (tutte);}
  else if (funktion == visibility.name()) { error = callvisibilityfunktion (visibility);}
  else if (funktion == ortho.name()) { error = callorthofunktion (ortho);}
  else if (funktion == quod.name()) { error = callquodfunktion (quod);}
  else {string msg = "no agd-server-function";errorfkt(msg);error = false; }
  ofstream opt (argv[3],ios::app);
  opt << "\nCreator \"AGD_SERVER\"\nVersion " << impl_version() << '\n';
  opt.close();
//  infos = false;
  return(!error);  
}




/************************* FPPLayout *******************************/

void agd_serv_class::call_make_fpp (ofstream& opt) {
   	callmake_infos_only (opt,fpp);  
}


bool agd_serv_class::callfppfunktion (GridLayoutModule& F) {
   if ((argc > 2) && (read_optfile(argv[3]))){
	
      	agd_serv_struct_def opt_fkt_struct;
        search_struct( F, opt_fkt_struct);
        list_item it;
        it = opt_fkt_struct.optbool_list.first();
        infos = opt_fkt_struct.optbool_list[it].wert.bool_wert;
   }
   return (callfunktionG (F));
}


                       
/************************* SchnyderLayout *******************************/

void agd_serv_class::call_make_schn (ofstream& opt) {
   	callmake_infos_only (opt,schn);  
}

bool agd_serv_class::callschnyderfunktion (GridLayoutModule& F) {
   if ((argc > 2) && (read_optfile(argv[3]))){	
      	agd_serv_struct_def opt_fkt_struct;
        search_struct( F, opt_fkt_struct);

        list_item it;
        it = opt_fkt_struct.optbool_list.first();
        infos = opt_fkt_struct.optbool_list[it].wert.bool_wert;
    }
   return (callfunktionG (F));
}


/************************* ConvexLayout *******************************/

void agd_serv_class::call_make_convex(ofstream& opt) {                         

   	opt << "\tfunction \"" <<  convex.name() << "\" ["<< endl;
           opt << "\t\tsize_opt " << out_bool(convex.size_opt()) << endl;                   
           opt << "\t\tbase_ratio " << convexd.base_ratio() << endl;                     
        
           callmake_std_infos (opt,convex); 
      
        opt << "\t]" << endl;
}


bool agd_serv_class::callconvexfunktion (GridLayoutModule& F) {
   if ((argc > 2) && (read_optfile(argv[3]))){	
      	agd_serv_struct_def opt_fkt_struct;
        
        search_struct( F, opt_fkt_struct);
        
        list_item it;
        it = opt_fkt_struct.optbool_list.first();
        convex.size_opt (opt_fkt_struct.optbool_list[it].wert.bool_wert);
        it = opt_fkt_struct.optbool_list.succ(it);
        infos = ((opt_fkt_struct.optbool_list[it]).wert.bool_wert);
        it = opt_fkt_struct.optdouble_list.first();
        convex.base_ratio (opt_fkt_struct.optdouble_list[it].wert.double_wert);
   }
   return (callfunktionG (F));
}

/************************* ConvexDrawLayout *****************************/

void agd_serv_class::call_make_convexd(ofstream& opt) {                

   opt << "\tfunction \"" << convexd.name() << "\" [" << endl;
      opt << "\t\tsize_opt " << out_bool(convexd.size_opt()) << endl;      
      opt << "\t\ttside_opt " << out_bool(convexd.side_opt()) << endl;
      opt << "\t\tbase_ratio " << convexd.base_ratio() << endl;       
        callmake_std_infos (opt,convexd); 
   opt << "\t]" << endl;        
 
}


bool agd_serv_class::callconvexdrawfunktion (GridLayoutModule& F) {
   if ((argc > 2) && (read_optfile(argv[3]))){	
      	agd_serv_struct_def opt_fkt_struct;
        
        search_struct( F, opt_fkt_struct);
        
        list_item it;
        it = opt_fkt_struct.optbool_list.first();
        convexd.size_opt (opt_fkt_struct.optbool_list[it].wert.bool_wert);
        it = opt_fkt_struct.optbool_list.succ(it);
        convexd.side_opt (opt_fkt_struct.optbool_list[it].wert.bool_wert);
        it = opt_fkt_struct.optbool_list.succ(it);
        infos = ((opt_fkt_struct.optbool_list[it]).wert.bool_wert);
        it = opt_fkt_struct.optdouble_list.first();
        convexd.base_ratio (opt_fkt_struct.optdouble_list[it].wert.double_wert);
   }
   return (callfunktionG (F));
}

/*********************** PlanarStraightLayout **************************/

void agd_serv_class::call_make_planarst(ofstream& opt) {


   opt << "\tfunction \"" << planarst.name() << "\" [" << endl;
      opt << "\t\tsize_opt " << out_bool(planarst.size_opt()) << endl; 
      opt << "\t\tmax_outer " << out_bool(planarst.max_outer()) << endl; 
      opt << "\t\tbase_ratio " << planarst.base_ratio() << endl;
        callmake_std_infos (opt,planarst); 
   opt << "\t]" << endl;          
}


bool agd_serv_class::callplanarstfunktion (GridLayoutModule& F) {
   if ((argc > 2) && (read_optfile(argv[3]))){	
      	agd_serv_struct_def opt_fkt_struct;
        
        search_struct( F, opt_fkt_struct);
        
        list_item it;
        it = opt_fkt_struct.optbool_list.first();
        planarst.size_opt (opt_fkt_struct.optbool_list[it].wert.bool_wert);
        it = opt_fkt_struct.optbool_list.succ(it);
        planarst.max_outer (opt_fkt_struct.optbool_list[it].wert.bool_wert);
        it = opt_fkt_struct.optbool_list.succ(it);
        infos = ((opt_fkt_struct.optbool_list[it]).wert.bool_wert);
        it = opt_fkt_struct.optdouble_list.first();
        planarst.base_ratio (opt_fkt_struct.optdouble_list[it].wert.double_wert);
   }
   return (callfunktionG (F));
}

/************************ PlanarDrawLayout ****************************/

void agd_serv_class::call_make_planardr(ofstream& opt){


   opt << "\tfunction \"" << planardr.name() << "\" [" << endl;
      opt << "\t\tsize_opt " << out_bool(planardr.size_opt()) << endl;
      opt << "\t\tside_opt " << out_bool(planardr.side_opt()) << endl;
      opt << "\t\tmax_outer " << out_bool (planardr.max_outer()) << endl;
      opt << "\t\tbase_ratio " << planardr.base_ratio() << endl;
        callmake_std_infos (opt,planardr); 
   opt << "\t]" << endl;          
}


bool agd_serv_class::callplanardrfunktion (GridLayoutModule& F) {
   if ((argc > 2) && (read_optfile(argv[3]))){	
      	agd_serv_struct_def opt_fkt_struct;
        
        search_struct( F, opt_fkt_struct);
        
        list_item it;
        it = opt_fkt_struct.optbool_list.first();
        planardr.size_opt (opt_fkt_struct.optbool_list[it].wert.bool_wert);
        it = opt_fkt_struct.optbool_list.succ(it);
        planardr.side_opt (opt_fkt_struct.optbool_list[it].wert.bool_wert);
        it = opt_fkt_struct.optbool_list.succ(it);
        planardr.max_outer (opt_fkt_struct.optbool_list[it].wert.bool_wert);
        it = opt_fkt_struct.optbool_list.succ(it);
        infos = ((opt_fkt_struct.optbool_list[it]).wert.bool_wert);
        it = opt_fkt_struct.optdouble_list.first();
        planardr.base_ratio (opt_fkt_struct.optdouble_list[it].wert.double_wert);
   }
   return (callfunktionG (F));
}

/************************ MixedModelLayout *****************************/

void agd_serv_class::call_make_mix(ofstream& opt){
      

   opt << "\tfunction \"" << mix.name() << "\" [" << endl;
      opt << "\t\tco_typ [" << endl;
         opt << "\t\t\tco_typ_wahl " << (int)mix.co_type() << endl;
         opt << "\t\t\tco_typ_lst [" << endl;
            opt << "\t\t\t\tlst \"triconnected\"" << endl;
            opt << "\t\t\t\tlst \"biconnected\"" << endl;
         opt <<"\t\t\t]" << endl;
      opt << "\t\t]" << endl;       
      opt << "\t\tprefer_nodes " << out_bool(mix.prefer_nodes()) << endl;     
      opt << "\t\tdo_postprocessing " << out_bool (mix.do_postprocessing()) << endl;
      opt << "\t\tbase_ratio " << mix.base_ratio() << endl;
        callmake_std_infos (opt,mix); 
   opt << "\t]" << endl;          
}


bool agd_serv_class::callmixfunktion (GridLayoutModule& F) {
   if ((argc > 2) && (read_optfile(argv[3]))){
      	agd_serv_struct_def opt_fkt_struct;
                
        search_struct( F, opt_fkt_struct);

        list_item it;
        it = (opt_fkt_struct.optenum_list).first();
        mix.co_type ((CanonicalOrderType)
                (opt_fkt_struct.optenum_list[it].wert.int_wert));
        it = opt_fkt_struct.optbool_list.first();
        mix.prefer_nodes ((opt_fkt_struct.optbool_list[it]).wert.bool_wert);
        it = opt_fkt_struct.optbool_list.succ(it);
        mix.do_postprocessing ((opt_fkt_struct.optbool_list[it]).wert.bool_wert);
        it = opt_fkt_struct.optbool_list.succ(it);
        infos = ((opt_fkt_struct.optbool_list[it]).wert.bool_wert);
        it = opt_fkt_struct.optdouble_list.first();
        mix.base_ratio ((opt_fkt_struct.optdouble_list[it]).wert.double_wert);
   }
   return (callfunktionG (F));
}

/************************* TreeLayout *******************************/

void agd_serv_class::call_make_tree(ofstream& opt) {

   opt << "\tfunction \"" << tree.name() << "\" [" << endl; 
      opt << "\t\tsib_separation " << tree.sib_separation() << endl;
      opt << "\t\tsubtree_separation " << tree.subtree_separation() << endl;
      opt << "\t\tlevel_separation " << tree.level_separation() << endl;
      opt << "\t\tleft_margin " << tree.left_margin() << endl;
      opt << "\t\tupper_margin " << tree.upper_margin() << endl;
        callmake_std_infos (opt,tree); 
   opt << "\t]" << endl;          
}


bool agd_serv_class::calltreefunktion (LayoutModule& F) {
   if ((argc > 2) && (read_optfile(argv[3]))){	
      	agd_serv_struct_def opt_fkt_struct;
        
        search_struct( F, opt_fkt_struct);
        
        list_item it;
        it = opt_fkt_struct.optbool_list.first();
        infos = opt_fkt_struct.optbool_list[it].wert.bool_wert;

        it = opt_fkt_struct.optdouble_list.first();
        tree.sib_separation(opt_fkt_struct.optdouble_list[it].wert.double_wert); 
        it = opt_fkt_struct.optdouble_list.succ(it);
        tree.subtree_separation (opt_fkt_struct.optdouble_list[it].wert.double_wert);
        it = opt_fkt_struct.optdouble_list.succ(it);
        tree.level_separation (opt_fkt_struct.optdouble_list[it].wert.double_wert);

        it = opt_fkt_struct.optdouble_list.succ(it);
        tree.left_margin (opt_fkt_struct.optdouble_list[it].wert.double_wert);

        it = opt_fkt_struct.optdouble_list.succ(it);
        tree.upper_margin (opt_fkt_struct.optdouble_list[it].wert.double_wert);

   }
   return (callfunktion (F));
}

/************************* SugiyamaLayout ******************************/


void agd_serv_class::call_make_sugiyama(ofstream& opt){

   int heuristic = 0; 
   int heuristic1 = 0;
   int heuristic2 = 0; 
   bool sugiyama_infos = false;
   int ranking = 1;
   int cross_min = 1;

   opt << "\tfunction \"" << sugiyama.name() << "\" [" << endl;
      opt << "\t\tranking [" << endl;
         opt << "\t\t\tranking_wahl " << ranking << endl;
         opt << "\t\t\tranking_lst [" << endl;
            opt << "\t\t\t\tlst \"TopSort\"" << endl;
            opt << "\t\t\t\tlst \"DFS\"" << endl;
         opt <<"\t\t\t]" << endl;
      opt << "\t\t]" << endl;    
      opt << "\t\theuristic [" << endl;
         opt << "\t\t\theuristic_wahl " << heuristic << endl;
         opt << "\t\t\theuristic_lst [" << endl;
            opt << "\t\t\t\tlst \"heuristic1\"" << endl;
            opt << "\t\t\t\tlst \"heuristic2\"" << endl;         
         opt <<"\t\t\t]" << endl;
      opt << "\t\t]" << endl;         
      opt << "\t\theuristic1 [" << endl;
         opt << "\t\t\theuristic1_wahl " << heuristic1 << endl;
         opt << "\t\t\theuristic1_lst [" << endl;                  
            opt << "\t\t\t\tlst \"Split\"" << endl;
            opt << "\t\t\t\tlst \"Barycenter\"" << endl;
            opt << "\t\t\t\tlst \"Median\"" << endl;
         opt <<"\t\t\t]" << endl;
      opt << "\t\t]" << endl;             
      opt << "\t\theuristic2 [" << endl;
         opt << "\t\t\theuristic2_wahl " << heuristic2 << endl;
         opt << "\t\t\theuristic2_lst [" << endl;              
            opt << "\t\t\t\tlst \"Greedy Insert\"" << endl;
            opt << "\t\t\t\tlst \"Greedy Switch\"" << endl;
         opt <<"\t\t\t]" << endl;
      opt << "\t\t]" << endl;    
      opt << "\t\tfixed_level_distance " << 
                	out_bool( hierarchy.fixed_layer_distance()) << endl; 
      opt << "\t\tnode_distance " <<  hierarchy.node_distance() << endl;
      opt << "\t\tlevel_distance " << hierarchy.layer_distance() << endl;
        callmake_std_infos (opt,sugiyama); 
   opt << "\t]" << endl;          
}


bool agd_serv_class::callsugiyamafunktion (LayoutModule& F) {
   int ranking;
   int heuristic;
   int heuristic1;
   int heuristic2; 
   int yu;

   if ((argc > 2) && (read_optfile(argv[3]))){
      	agd_serv_struct_def opt_fkt_struct;
        search_struct( F, opt_fkt_struct);
        list_item it;

        it = opt_fkt_struct.optbool_list.first();
        hierarchy.fixed_layer_distance(opt_fkt_struct.optbool_list[it].wert.bool_wert);
        it = opt_fkt_struct.optbool_list.succ(it);
        infos = ((opt_fkt_struct.optbool_list[it]).wert.bool_wert);
        it = opt_fkt_struct.optint_list.first();
        hierarchy.node_distance(opt_fkt_struct.optint_list[it].wert.int_wert);
  	it = opt_fkt_struct.optint_list.succ(it);
	hierarchy.layer_distance(opt_fkt_struct.optint_list[it].wert.int_wert);
	sugiyama.set_comp_coord(hierarchy);


        it = opt_fkt_struct.optenum_list.first();


        ranking = (opt_fkt_struct.optenum_list[it]).wert.int_wert;

	switch (ranking) {
			case 0: {
   				sugiyama.set_ranking(TopSortRanking());
				break;
				}
			case 1: {
				sugiyama.set_ranking(DfsRanking());
				break;
				}
	}
        it = opt_fkt_struct.optenum_list.succ(it);

        heuristic = (opt_fkt_struct.optenum_list[it]).wert.int_wert;
        it = opt_fkt_struct.optenum_list.succ(it);
        heuristic1 = (opt_fkt_struct.optenum_list[it]).wert.int_wert;
        it = opt_fkt_struct.optenum_list.succ(it);
        heuristic2 = (opt_fkt_struct.optenum_list[it]).wert.int_wert;
        if (heuristic > 0) heuristic1= heuristic2 + 3;        
        
	switch (heuristic1) {

			case 0:
				sugiyama.set_cross_min(SplitHeuristic());
				break;
			case 1:
				sugiyama.set_cross_min(BarycenterHeuristic());
				break;

			case 2:
				sugiyama.set_cross_min(MedianHeuristic());
				break;
			case 3:
				sugiyama.set_cross_min(GreedyInsertHeuristic());
				break;
			case 4:
				sugiyama.set_cross_min(GreedySwitchHeuristic());
				break;

	}
   }             
   return callfunktion (F); 
}


/************************* SpringLayout *******************************/     

void agd_serv_class::call_make_spring (ofstream& opt) {
        

   opt << "\tfunction \"" << spring.name() << "\" [" << endl;
      opt << "\t\tnoise " << out_bool(spring.noise()) << endl;
      opt << "\t\titerations " << spring.iterations() << endl;
        callmake_std_infos (opt,spring); 
   opt << "\t]" << endl;          
}

bool agd_serv_class::callspringfunktion (LayoutModule& F) {
   if ((argc > 2) && (read_optfile(argv[3]))){
      	agd_serv_struct_def opt_fkt_struct;
        
        search_struct( F, opt_fkt_struct);
        
        list_item it;
        it = opt_fkt_struct.optbool_list.first();
        spring.noise (opt_fkt_struct.optbool_list[it].wert.bool_wert);
        it = opt_fkt_struct.optbool_list.succ(it);
        infos = ((opt_fkt_struct.optbool_list[it]).wert.bool_wert);
        it = opt_fkt_struct.optint_list.first();
        spring.iterations (opt_fkt_struct.optint_list[it].wert.int_wert);
   }
   return (callfunktion (F));
}

/************************* TutteLayout *******************************/

void agd_serv_class::call_make_tutte (ofstream& opt) {

     callmake_infos_only (opt,tutte);
}


bool agd_serv_class::calltuttefunktion (LayoutModule& F) {
   if ((argc > 2) && (read_optfile(argv[3]))){	
      	agd_serv_struct_def opt_fkt_struct;
        search_struct( F, opt_fkt_struct);

        list_item it;
        it = opt_fkt_struct.optbool_list.first();
        infos = opt_fkt_struct.optbool_list[it].wert.bool_wert;
   }
   return (callfunktion (F));
}


/*********************** VisibilityLayout *****************************/

void agd_serv_class::call_make_visibility (ofstream& opt) {

     callmake_infos_only (opt,visibility);
}    


bool agd_serv_class::callvisibilityfunktion (GridLayoutModule& F) {
   if ((argc > 2) && (read_optfile(argv[3]))){	
      	agd_serv_struct_def opt_fkt_struct;
        search_struct( F, opt_fkt_struct);

        list_item it;
        it = opt_fkt_struct.optbool_list.first();
        infos = opt_fkt_struct.optbool_list[it].wert.bool_wert;
    }
   return (callfunktionGv (F));
}



/************************* OrthoLayout *******************************/     

void agd_serv_class::call_make_ortho (ofstream& opt) {
        
   opt << "\tfunction \"" << ortho.name() << "\" [" << endl;
      opt << "\t\tdissect " << out_bool(ortho.dissect()) << endl;
      opt << "\t\tcheck " << out_bool(ortho.verify()) << endl; 
        callmake_std_infos (opt,ortho); 
   opt << "\t]" << endl;          
}

bool agd_serv_class::callorthofunktion (LayoutModule& F) {
   if ((argc > 2) && (read_optfile(argv[3]))){
      	agd_serv_struct_def opt_fkt_struct;
        
        search_struct( F, opt_fkt_struct);
        
        list_item it;
        it = opt_fkt_struct.optbool_list.first();
        ortho.dissect (opt_fkt_struct.optbool_list[it].wert.bool_wert);
        it = opt_fkt_struct.optbool_list.succ(it);
        ortho.verify((opt_fkt_struct.optbool_list[it]).wert.bool_wert);
        it = opt_fkt_struct.optbool_list.succ(it);
        infos = ((opt_fkt_struct.optbool_list[it]).wert.bool_wert);

   }
   return (callfunktion (F));
}

/************************* QuodLayout *******************************/     

void agd_serv_class::call_make_quod (ofstream& opt) {
        
        
   opt << "\tfunction \"" << quod.name() << "\" [" << endl;
      opt << "\t\tdissect " << out_bool(quod.dissect()) << endl;      
      opt << "\t\tcheck " << out_bool(quod.verify()) << endl;
        callmake_std_infos (opt,quod); 
   opt << "\t]" << endl;          
}


bool agd_serv_class::callquodfunktion (LayoutModule& F) {
   if ((argc > 2) && ((read_optfile(argv[3])))){
      	agd_serv_struct_def opt_fkt_struct;
        search_struct( F, opt_fkt_struct);
        list_item it;
        it = opt_fkt_struct.optbool_list.first();
        quod.dissect (opt_fkt_struct.optbool_list[it].wert.bool_wert);
        it = opt_fkt_struct.optbool_list.succ(it);
        quod.verify((opt_fkt_struct.optbool_list[it]).wert.bool_wert);
        it = opt_fkt_struct.optbool_list.succ(it);
        infos = ((opt_fkt_struct.optbool_list[it]).wert.bool_wert);
   }
   return (callfunktion (F));
}


/************************ PlanarizationLayout *****************************/

void agd_serv_class::call_make_planar(ofstream& opt){
	int layouter=2; 
        int layouter1=0;
        int layouter2=0; 
        int layouter3=0;
        
	int planarizer =1;     

	SubgraphPlanarizer 	subgraph;
	PlanarSubgraph	  	planarsubgraph;
//	MaximumPlanarSubgraph 	maxplanarsubgrah;    		Abacus & Cplex
	ShortestPathInserter	shortestpathinserter;


   opt << "\tfunction \"" << planar.name() << "\" [" << endl;
      opt << "\t\tlayouter [" << endl;
         opt << "\t\t\tlayouter_wahl " << layouter << endl;     
         opt << "\t\t\tlayouter_lst [" << endl;
            opt << "\t\t\t\tlayouterlist \"layouter1\"" << endl;
            opt << "\t\t\t\tlayouterlist \"layouter2\"" << endl;
            opt << "\t\t\t\tlayouterlist \"layouter3\"" << endl;
         opt <<"\t\t\t]" << endl;
      opt << "\t\t]" << endl; 
      opt << "\t\tlayouter1 [" << endl;
         opt << "\t\t\tlayouter1_wahl " << layouter1 << endl;     
         opt << "\t\t\tlayouter1_lst [" << endl;
            opt << "\t\t\t\tlayouterlist1 \"" << "Fraysseix" << "\"" << endl;
            opt << "\t\t\t\tlayouterlist1 \"" << schn.name() << "\"" << endl;
            opt << "\t\t\t\tlayouterlist1 \"" << convex.name() << "\"" << endl;
         opt <<"\t\t\t]" << endl;
      opt << "\t\t]" << endl; 
      opt << "\t\tlayouter2 [" << endl;
         opt << "\t\t\tlayouter2_wahl " << layouter2 << endl;
         opt << "\t\t\tlayouter2_lst [" << endl;
            opt << "\t\t\t\tlayouterlist2 \"" << convexd.name() << "\"" << endl;
            opt << "\t\t\t\tlayouterlist2 \"" << planarst.name() << "\"" << endl;
            opt << "\t\t\t\tlayouterlist2 \"" << planardr.name() << "\"" << endl;
         opt <<"\t\t\t]" << endl;
      opt << "\t\t]" << endl;    
      opt << "\t\tlayouter3 [" << endl;
         opt << "\t\t\tlayouter3_wahl " << layouter3 << endl;
         opt << "\t\t\tlayouter3_lst [" << endl;
            opt << "\t\t\t\tlayouterlist3 \"" << mix.name() << "\"" << endl;
            opt << "\t\t\t\tlayouterlist3 \"" << "Orthogonal" << "\"" << endl;
            opt << "\t\t\t\tlayouterlist3 \"" << quod.name() << "\"" << endl;
         opt <<"\t\t\t]" << endl;
      opt << "\t\t]" << endl;  
        callmake_std_infos (opt,planar); 
   opt << "\t]" << endl;          

/*    
      opt << "\t\tplanarizer [" << endl;
         opt "\t\t\planarizer_wahl " << planarizer << endl;
         opt "\t\t\planarizer_lst [" << endl;
            opt << "\t\t\t\tlst \"" << subgraph.name() << "\"" << endl;
            opt << "\t\t\t\tlst \"" << planarsubgraph.name() << "\"" << endl;
            opt << "\t\t\t\tlst \"" << shortestpathinserter.name() << "\"" << endl;
//            opt << "\t\t\t\tlst \"" << maxplanarsubgraph.name() << "\"" << endl;
         opt <<"\t\t\t]" << endl;
*/

}


bool agd_serv_class::callplanarfunktion (LayoutModule& F) {

	int layouter; 
        int layouter1;
        int layouter2; 
        int layouter3;    
	int planarizer;     
/*
	SubgraphPlanarizer 	subgraph;
	PlanarSubgraph	  	planarsubgraph;
//	MaximumPlanarSubgraph 	maxplanarsubgrah;    		Abacus & Cplex
	ShortestPathInserter	shortestpathinserter;
*/


   if ((argc > 2) && (read_optfile(argv[3]))){
      	agd_serv_struct_def opt_fkt_struct;
        
        search_struct( F, opt_fkt_struct);

        list_item it;
        it = (opt_fkt_struct.optenum_list).first();
        layouter = opt_fkt_struct.optenum_list[it].wert.int_wert;
        it = opt_fkt_struct.optenum_list.succ(it);
        layouter1 = opt_fkt_struct.optenum_list[it].wert.int_wert;
        it = opt_fkt_struct.optenum_list.succ(it);
        layouter2 = opt_fkt_struct.optenum_list[it].wert.int_wert;
        it = opt_fkt_struct.optenum_list.succ(it);
        layouter3 = opt_fkt_struct.optenum_list[it].wert.int_wert;

/*
        it = opt_fkt_struct.optenum_list.succ(it);
        layouter = opt_fkt_struct.optenum_list[it].wert.int_wert;
*/

   	if (layouter == 1) layouter1= 3 + layouter2;
   	if (layouter == 2) layouter1= 6 + layouter3;
	switch (layouter1) {
			case 0: {
   				planar.set_planar_layouter(fpp);
				break;
				}
			case 1: {
   				planar.set_planar_layouter(schn);
				break;
				}
			case 2: {
   				planar.set_planar_layouter(convex);
				break;
				}
			case 3: {
   				planar.set_planar_layouter(convexd);
				break;
				}
			case 4: {
   				planar.set_planar_layouter(planarst);
				break;
				}
			case 5: {
   				planar.set_planar_layouter(planardr);
				break;
				}
			case 6: {
   				planar.set_planar_layouter(mix);
				break;
				}
			case 7: {
   				planar.set_planar_layouter(ortho);
				break;
				}
			case 8: {
   				planar.set_planar_layouter(quod);
				break;
				}
	}

/*	switch (planarizer) {
			case 0: {
   				planar.set_planarizer(subgraph);
				break;
				}
			case 1: {
   				planar.set_planarizer(planarsubgraph);
				break;
				}
			case 2: {
   				planar.set_planarizer(shortestpathinserter);
				break;
				}
//			case 3: {
//   				planar.set_planarizer(maxplanarsubgrah);
//				break;
//				}

	}
*/
        it = opt_fkt_struct.optbool_list.first();
        infos = opt_fkt_struct.optbool_list[it].wert.bool_wert;


   }
   return (callfunktion (F));
}




int main(int argc, char * argv[]) {

int i;

/*
for(i = 0; i< argc; i++) cout << argv[i] << " ";
cout <<endl;
*/

   agd_serv_class* serv = new agd_serv_class(argc,argv);

   i = serv->agd_serv();

   delete serv;

   return i;
}

