//-------------------------------------------------------------------
//
//                        CompOrder.cpp
//
//    author: Carsten Gutwenger
//    date:   february 1997
//-------------------------------------------------------------------


#include <AGD/impl/CompOrder.h>
#include <AGD/CanonicalOrder.h>


// Bestimme Knoten v der Flche f, der an die Auenflche grenzt und im aktuellen Teilgraph den Grad 2
// besitzt (v != v2, oder v = v1)
node CompOrder::get_outer_deg_2 (face f, graph &G,
	node_array<edge> &e_pred, node_array<edge> &e_succ)
// die Felder e_pred und e_succ bestimmen die Verkettung der Knoten auf der Auenflche
{
	list<node> &L = outer_nodes [f];
	list_item it;
	node v;

	forall_items (it, L) {
		v = L[it];
		if (v == v_right) continue;
		if (v == v_left) return v;
		// Hat v Grad 2 ? (da keine Knoten in G gelscht werden, kann nicht getestet werden, ob deg(v) = 2 ist)
		if (e_pred[v] == G.cyclic_adj_pred(e_succ[v]))
			return v;
		}
	//cerr << "Kein Knoten mit Grad 2 in f->outer_nodes!"; // Dieser Fehler sollte nicht vorkommen!
	return nil;
}

// Initialisiere Knoten der Auenflache, initialisere v1, v2
void CompOrder::init_outer_nodes (face F_out, edge e_left, edge e_right)
{
	node v;
	edge e, e2;
	face f;
	bool first = true;

	//v1 = v1_ini; v2 = v2_ini;
	// Fr alle v in F_out erhhe outv[f] fr alle f != F_out mit v in f
	//e = g->face_cycle_succ (e_left);
	e = e_left;
	while (first || e != e_right) {
	  //forall_face_edges (e, F_out) {
		v = target(e);
		forall_out_edges (e2, v) {
			f = g->face_of(e2);
			if (f != F_out) {
				outv [f] ++;
				outer_nodes[f].append (v); // aktualisiere f->outer_nodes
			}
		}
		e = g->face_cycle_succ(e);
		first = false;
	}

	// Initialisiere on_base[v]
	for (e = e_right; e != e_left; e = g->face_cycle_succ(e))
	  on_base [source(e)] = true;
	on_base [source(e_left)] = on_base [target(e_left)] = true;

	v_left = target(e_left);
	v_right = source(e_right);
	Fout = F_out;
}

// Initialisere Kanten der Auenflche
void CompOrder::init_outer_edges (face F_out)
{
	edge e;

	// Fr alle e in F_out mit e != (v1,v2) erhhe oute[f] fr alle f != F_out mit e in f
	forall_face_edges (e, F_out)
		if (on_base[source(e)] == false || on_base[target(e)] == false) {
			oute [g->face_of(g->reversal(e))] ++;
		}

}

// Falls v noch nicht in update-Liste, fge v in update-Liste ein
void CompOrder::set_update (node v)
{
	if (v_update [v] == false) {
		update_nodes.append (v);
		v_update [v] = true;
	}
}

// Falls f noch nicht in update-Liste, fge f in update-Liste ein
void CompOrder::set_update (face f)
{
	if (f_update [f] == false) {
		update_faces.append (f);
		f_update [f] = true;
	}
}

// Fuhre Update durch: Teste, ob
//		a) v bzw. f nach dem aktuellen Schritt in  die possible Liste genommen werden mu
//		b) v bzw. f nach dem aktuellen Schritt aus der possible Liste genommen werden mu
//		c) f im aktuellen Schritt zur Seperations-Flche geworden ist
void CompOrder::do_update ()
{
	bool can_take, is_sep;
	node v;
	face f;

	// Test fr Flchen mu zuerst gemacht werden, Variablen von Knoten gendert werden knnen!
	while (!update_faces.empty()) {
		f = update_faces.pop ();
		f_update [f] = false;
		can_take = (outv[f] == oute[f] + 1 && oute[f] >= 2 && f != Fout);	// ist f mgliche Kante ?
		if (f_link[f] == nil) {  // ist f noch nicht in possible-Liste
			if (can_take) f_link[f] = possible_faces.append (f);  // eventuell einfgen
		} else
			if (!can_take) {  // eventuell wieder herausnehmen
				possible_faces.del_item (f_link[f]);
				f_link[f] = nil;
			}
		is_sep = (outv[f] >= 3 || (outv[f] == 2 && oute[f] == 0));  // is f Seperations-Flche ?
		if (is_sep_face[f] == false) {  // f war vor diesem Schritt noch keine Sep.-Flche
			if (is_sep == true) {  // und ist jetzt eine geworden
				is_sep_face [f] = true;
				forall (v, outer_nodes[f])  // erhhe sepf[v] fr alle v in f der Auenflche
					inc_sepf (v);
			}
		} else  // Falls jetzt keine mehr, dann nur is_sep_face korrigieren, der Rest (ernidrigen von sepf) wurde
			if (is_sep == false) is_sep_face [f] = false;  // schon in CanonicalOrder() erledigt
	}

	while (!update_nodes.empty()) {
		v = update_nodes.pop ();
		v_update [v] = false;
		can_take = (visited [v] >= 1 && sepf [v] == 0 && on_base[v] == false); // ist v mglicher Knoten ? (vermeide,
																									 // v1 und v2 zu nehemn)
		if (v_link [v] == nil) {  // ist v noch nicht in possible-Liste
			if (can_take) v_link [v] = possible_nodes.append (v);  // eventuell einfgen
		} else
			if (!can_take) {  // eventuell wieder herausnehmen
				possible_nodes.del_item (v_link [v]);
				v_link [v] = nil;
			}
	}
}


//
// I/O Operators for class OrderSet as required by LEDA 3.4 (dummy only)
//

ostream& operator<< (ostream& out, OrderSet)
{
	return out;
}

istream& operator>> (istream& in, OrderSet)
{
	return in;
}
