//-------------------------------------------------------------------
//
//                  GridCoordinateMapper.cpp
//
//    author: Carsten Gutwenger
//    date:   octobre 1997
//-------------------------------------------------------------------


#include <AGD/GridCoordinateMapper.h>
#include <AGD/PlaneGraphCopy.h>


DRect GridCoordinateMapper::call (const graph& G, const GridLayout &gl,
	const IRect& gbbox, LayoutInterface& A)
{
	_graph_type = 0;
	_graph_A = _graph_B = &G;
	_A = &A;

	DRect bbox = _do_call (gl, gbbox);

	return bbox;
}

DRect GridCoordinateMapper::call (const GraphCopy& GC, const GridLayout &gl,
	const IRect& gbbox, LayoutInterface& A)
{
	_graph_type = 2;
	_graph_A = &GC.original();
	_graph_B = &GC;
	_PG = &GC;
	_A = &A;

	DRect bbox = _do_call (gl, gbbox);

	return bbox;
}

DRect GridCoordinateMapper::_do_call (const GridLayout &gl, const IRect& gbbox)
{
	_gl = &gl;
	_gbbox = gbbox;

	init();
	map_nodes();
	map_edges();
	DRect bbox;
	get_layout_size(bbox);

	cleanup();
	return bbox;
}

void GridCoordinateMapper::map_nodes()
{
	const graph& G = graph_A();
	node v;

	forall_nodes (v, G)
		_A->set_position (v, map_node(v));
}

DPoint GridCoordinateMapper::map_node (node v)
{
	return DPoint (map_x(x(v)), map_y(y(v)));
}

void GridCoordinateMapper::map_edges()
{
	const graph& G = graph_A();
	edge e;

	forall_edges (e, G) {
		DPolyline dpl;
		DPoint s_a, t_a;
		map_edge (e, dpl, s_a, t_a);
		_A->set_bend_points (e, dpl);
		_A->set_source_anchor (e, s_a);
		_A->set_target_anchor (e, t_a);
	}
}

void GridCoordinateMapper::map_edge (edge e, DPolyline& dpl,
	DPoint& /*s_a*/, DPoint& /*t_a*/)
{
	IPolyline ipl = bends(e);
	IPoint ip;

	forall (ip, ipl)
		dpl.append (DPoint(map_x(ip.xcoord()),map_y(ip.ycoord())));
}

IPolyline GridCoordinateMapper::bends (edge e)
{
	switch (_graph_type) {
	case 0: // graph
		return _gl->get_compact_bends(e);

	case 2: // GraphCopy
		return _gl->get_compact_bends(*_PG, e);
		/*
		IPolyline ipl;
		IPoint ip;
		const list<edge>& ch = _PG->chain(e);
		edge e2, e1 = ch.head();

		forall (e2, ch) {
			if (e2 != e1)
				ipl.append(IPoint(_gl->x(source(e2)),_gl->y(source(e2))));

			if (!_gl->bends(e2).empty()) {
				forall (ip, _gl->bends(e2))
					ipl.append(ip);
			} else if (_PG->is_map()) {
				forall_rev (ip, _gl->bends(_PG->reversal(e2)))
					ipl.append(ip);
			}

		}
		if (_PG->is_reversed(e))
			ipl.reverse_items();

		return ipl;*/
	}

	error_handler (1, "GridCoordinateMapper::bends : illegal _graph_type");
	return IPolyline();
}

node GridCoordinateMapper::in_B (node v) const
{
	switch (_graph_type) {
	case 0:
		return v;
	case 2:
		return _PG->copy(v);
	}

	error_handler (1, "GridCoordinateMapper::in_B : illegal _graph_type");
	return nil;
}
