//-------------------------------------------------------------------
//
//                    LayoutInterface.cpp
//
//    author: Carsten Gutwenger
//    date:   may 1998
//-------------------------------------------------------------------


#include <AGD/AGDLayout.h>
#include <AGD/GraphCopyLI.h>


//-------------------------------------------------------------------
//                          LayoutInterface
//-------------------------------------------------------------------

DRect LayoutInterface::area() const
{
	if (gp->empty()) return DRect();

	node v = gp->first_node();
	double xmin = get_x(v), ymin = get_y(v);
	double xmax = xmin, ymax = ymin;

	forall_nodes(v,*gp) {
		double x = get_x(v), y = get_y(v);
		double w = get_width(v), h = get_height(v);

		if (x - w/2 < xmin) xmin = x - w/2;
		if (x + w/2 > xmax) xmax = x + w/2;
		if (y - h/2 < ymin) ymin = y - h/2;
		if (y + h/2 > ymax) ymax = y + h/2;
	}

	edge e;
	forall_edges(e,*gp) {
		const DPolyline &bends = get_bend_points(e);

		DPoint p;
		forall(p,bends) {
			if (p.xcoord() < xmin) xmin = p.xcoord();
			if (p.xcoord() > xmax) xmax = p.xcoord();
			if (p.ycoord() < ymin) ymin = p.ycoord();
			if (p.ycoord() > ymax) ymax = p.ycoord();
		}
	}

	return DRect(xmin,xmax,ymin,ymax);
}

void LayoutInterface::make_straight_line ()
{
	DPolyline empty;
	DPoint center;

	edge e;
	forall_edges (e, *gp) {
		set_bend_points (e, empty);
		set_source_anchor (e, center);
		set_target_anchor (e, center);
	}
}

void LayoutInterface::set_all_anchors (DPoint p)
{
	edge e;
	forall_edges (e, *gp) {
		set_source_anchor (e, p);
		set_target_anchor (e, p);
	}
}


//-------------------------------------------------------------------
//                           LayoutBuffer
//-------------------------------------------------------------------

LayoutBuffer::LayoutBuffer (const graph& G) : LayoutInterface(G)
{
	_pos.init(G);
	_bends.init(G);
	_s_anchor.init(G);
	_t_anchor.init(G);
}


//-------------------------------------------------------------------
//                           GraphCopyLI
//-------------------------------------------------------------------

void GraphCopyLI::update()
{
	node v;
	edge e, e1;
	const graph &g = gc_p->original();

	forall_nodes (v, g) {
		a_p->set_position(v, get_position(gc_p->copy(v)));
	}

	forall_edges (e, g) {
		a_p->set_source_anchor(e, get_source_anchor(
			gc_p->chain(e).head()));
		a_p->set_target_anchor(e, get_target_anchor(
			gc_p->chain(e).tail()));

		DPolyline dpl, dpl1;
		forall (e1, gc_p->chain(e)) {
			dpl1 = get_bend_points(e1);
			dpl.conc(dpl1);
			dpl.append(get_position(target(e1)));
		}
		dpl.pop_back();
		if (gc_p->is_reversed(e))
			dpl.reverse_items();
		a_p->set_bend_points(e, dpl);
	}

	a_p->update();
}

//-------------------------------------------------------------------
//                           AGDLayout
//-------------------------------------------------------------------

AGDLayout::AGDLayout (const graph& G) : LayoutBuffer(G)
{
	_width.init (G,0);
	_height.init (G,0);
}
