
// #line 175 "OrthoShape.lw"
#include <AGD/OrthoShape.h>

#include <LEDA/stack.h>

const string epsilon = "";
enum state { nothing_read, read_10, read_1 };

char flip(char c) { 
  if (c != '0' && c != '1') error_handler(1, "flip: invalid character"); 
  if (c == '0') return '1';
  return '0';
}



// #line 193 "OrthoShape.lw"
OrthoShape::OrthoShape(PlaneGraphCopy& G, face f_0) {
  _G = &G; _f_0 = f_0; init();
}

OrthoShape::OrthoShape(graph& G, face f_0) {
  
  cout << "graph G in da construcotr of OrthoShape: " << endl;
  G.print();
  _G = new PlaneGraphCopy(G);
  _f_0 = _G->copy_of_face(f_0); init();
  cout << "_G in da constructor of OrthoShape: " << endl;
  _G->print(); newline; 
  
}

void OrthoShape::init() {
  _a.init(*_G, 0); _s.init(*_G, epsilon);
  _number_of_bends = 0; _zeroes.init(*_G, 0);
  _dir.init(*_G, unexplored);
}

void OrthoShape::set_string(const leda_edge e, const char c, int n) {
  if (c != '0' && c != '1') error_handler(1, "bitstring: no bits");
  for (int i = 0; i < n; i++) _s[e] += c;
  if (c == '0') { _zeroes[e] += n; _number_of_bends += n; }
}

bool OrthoShape::check(string& msg) const {

  if (!Is_Planar_Map(*_G)) { 
    msg = "OrthoShape::check: The graph is no planar map";
    return false;
  }

  edge e; forall_edges(e, *_G) {
    string s = _s[e], r = _s[_G->reversal(e)]; 
    for (int i = 0; i < s.length(); i++) if (s[i] != flip(r[i])) {
      msg = "OrthoShape::check: The bit strings are inconsistent";
      return false;
    }
  } 	    		

  face f; forall_faces(f, *_G) {
    int rho = 0;
    forall_face_edges(e, f) {
      //print_edge(e); cout << string("\t|0| = %d, |1| = %d, 2 - a/90 = %d\n", _zeroes[e], ones(e), 2 - _a[e]/90);
      rho += (_zeroes[e] - ones(e) + 2 - _a[e]/90);
    }
    //cout << "rho = " << rho << endl;
    if (rho != ((f == _f_0) ? -4 : 4)) {
      msg = "OrthoShape::check: Face shapes are not rectagonal";
      return false;
    }
  }

  node v; forall_nodes(v, *_G) {    
    int angle_sum = 0;
    forall_in_edges(e, v) angle_sum += _a[e];
    if (angle_sum != 360) {
      msg = "OrthoShape::check: Angle sums corrupted";
      return false;
    }
  }
  
  return true;

}

void OrthoShape::add_bend(const edge e) {
  cout << _zeroes[e] << " before ";
  _s[e] += '0'; _zeroes[e]++; _number_of_bends++;
  _s[_G->reversal(e)] += '1';
  cout << ", " << _zeroes[e] << " (after)" << endl;
}

void OrthoShape::normalize() {
  edge e; forall_edges(e, *_G) for (int i = _zeroes[e]; i > 0; i--) {
    int a_e = _a[e], a_r = _a[reversal(e)];
    edge e_s = _G->split(e);
    _a[e_s] = a_e; _a[e] = 90;
    _a[reversal(e)] = a_r;
    _a[reversal(e_s)] = 270;
    _zeroes[e_s] = _zeroes[reversal(e_s)] 
      = _zeroes[e] = _zeroes[reversal(e)] = 0; 
    _s[e_s] = _s[reversal(e_s)] = _s[e] = _s[reversal(e)] = epsilon;
    //cout << string("e_s: %d, e: %d, r_s: %d, r: %d\n", _dir[e_s], _dir[e], _dir[reversal(e_s)], _dir[reversal(e)]);
    --_number_of_bends;
  }
}

void OrthoShape::dissect() {
  if (!_number_of_bends == 0) 
    error_handler(1, "OrthoShape, dissect: Shape must be normalized");
  face f; forall_faces(f, *_G) {
    edge e_act = _G->first_face_edge(f); int s_f_size = 0;
    edge e; forall_face_edges(e, f) s_f_size += abs(_a[e]/90 - 2); 
    state z = nothing_read; stack<edge> S;

    while (s_f_size > 4) {		// we basically do 
      switch (_a[e_act]) {		// one walk around f
        case 90:  if (z == read_10) { 
// #line 319 "OrthoShape.lw"
edge e_stack = S.pop(), e_rect_beg = _G->face_cycle_succ(e_stack),
     e_rect_end = _G->face_cycle_succ(e_act),
     e_split = _G->split(e_rect_end),
     e_scaff = _G->split_face(e_rect_beg, e_split);

_a[e_stack] -= 90;
_a[e_split] = _a[e_rect_end]; _a[reversal(e_rect_end)] = _a[reversal(e_split)];
_a[e_rect_end] = 90; _a[reversal(e_split)] = 180;
_a[e_scaff] = 90; _a[reversal(e_scaff)] = 90; 

e_act = e_scaff; z = nothing_read; s_f_size -= 2;


// #line 293 "OrthoShape.lw"
                                                           ; }
                  if (z == read_1) z = read_10;
                  break;
        case 180: break;
        case 270:
        case 360: if ((f == _f_0) && (z == read_10)) { 
// #line 340 "OrthoShape.lw"
edge e_stack = S.pop(), e_rect_beg = _G->face_cycle_succ(e_stack),
     e_rect_end = _G->face_cycle_succ(e_act),
     e_new = _G->split_face(e_rect_beg, e_rect_end),
     e_split = _G->split(e_new);
    
_a[e_stack] -= 90;
_a[e_new] = 270;
_a[e_split] = _a[e_act] - 90;
_a[reversal(e_new)] = 90; _a[e_act] = 90;
_a[reversal(e_split)] = 90;
 
e_act = e_stack; z = nothing_read; s_f_size -= 2;



// #line 298 "OrthoShape.lw"
                                                                          ; } 
                  else { 
                    z = read_1; S.push(e_act); 
                    if (_a[e_act] == 360) S.push(e_act);
                  }
                  break;
        default: error_handler(1, "angle not in {90, 180, 270, 360}");
      }
  
      if (z == nothing_read && !S.empty()) {
        z = read_1; e_act = _G->face_cycle_succ(S.top()); 
      } else e_act = _G->face_cycle_succ(e_act);
    }
  }
  _zeroes.init(*_G, 0); 
}


// #line 359 "OrthoShape.lw"
void OrthoShape::orientate() { 
  _dir.init(*_G, unexplored);
  orientate(_G->first_edge(), N); 
}

void OrthoShape::orientate(edge e, direction d) {
  while (_dir[e] == unexplored) {
    _dir[e] = d; edge r = reversal(e);
    if (_dir[r] == unexplored) orientate(r, opp(d));
    switch (_a[e]) {
      case  90: d = next(d); break;
      case 270: d = prev(d); break;
      case 360: d = opp(d); break;
    }
    e = _G->face_cycle_succ(e);
  }
}

ostream& operator<< (ostream& O, const OrthoShape& H) {
  O << string("OrthoShape with %d nodes, %d edges and %d bends", 
    H.number_of_nodes(), H.number_of_edges(), H.number_of_bends()) << endl;
  face f; forall_faces(f, H) {
    O << ((f == H.get_const_f_0()) ? "external " : "internal ") 
      << "face" << endl;
    edge e; forall_face_edges(e, f) {
      O << "\t";
      H.print_edge(e); O << "\ts = " << H.get_string(e) << 
        "\ta = " << H.get_angle(e) << "\tdir = " << int(H.get_dir(e)) << endl;
    }
  }
  return O;
}

edge OrthoShape::split(edge e) {
  cout << "split of "; print_edge(e); newline;
  cout << "e before  "; print_edge(e); cout << _a[e] << endl;
  cout << "r before  "; print_edge(reversal(e)); cout << _a[reversal(e)] << endl;
  edge s = _G->split(e);
  _dir[s] = _dir[e]; _dir[reversal(e)] = _dir[reversal(s)];
  _a[s] = _a[e]; _a[reversal(e)] = _a[reversal(s)];
  _a[e] = _a[reversal(s)] = 180;
  cout << "e after "; print_edge(e); cout << _a[e] << endl;
  cout << "r after "; print_edge(reversal(e)); cout << _a[reversal(e)] << endl;
  cout << "s after "; print_edge(s); cout << _a[s] << endl;
  cout << "sr after"; print_edge(reversal(s)); cout << _a[reversal(s)] << endl;
  return s;
}
