//-------------------------------------------------------------------
//
//                        PlanarSubgraph.cpp
//
//
// author: Carsten Gutwenger, Thomas Ziegler (pre- and postprocessing)
//         Sebastian Leipert (actual algorithm)
// date:   novembre 1997
//-------------------------------------------------------------------


#include <AGD/PlanarSubgraph.h>

extern int verticecount;
extern int edgecount;
#include <AGD/impl/MaxPlanSub/adjlists.h>
extern adjlists* spanplan(adjlists*);  
                 //returns deledges, parameter is inputgraph


PlanarSubgraph::PlanarSubgraph() : SubgraphModule()
{
	add_post_rule (key::planar);
	add_post_rule (key::simple, 1, key::simple);
	add_post_rule (key::no_self_loops, 1, key::no_self_loops);
	add_post_rule (key::no_reversals, 1, key::no_reversals);
}


class _ps_edge {
public:
	_ps_edge() { }
	_ps_edge(int _s,int _t,edge _e)
	{ src = _s; tgt = _t; e = _e; }

	int src, tgt;
	edge e;
};

// I/O operators for LEDA (dummies only)
ostream& operator<< (ostream& out, const _ps_edge)
{ return out; }

istream& operator>> (istream& in, _ps_edge)
{ return in; }

class _ps_int_pair {
public:
	_ps_int_pair() { }
	_ps_int_pair(int _s, int _t)
	{ src = _s; tgt = _t; }
	
	bool operator==(const _ps_edge& e)
	{ return src == e.src && tgt == e.tgt; }

	bool operator!=(const _ps_edge& e)
	{ return src != e.src || tgt != e.tgt; }

	int src, tgt;
};

// I/O operators for LEDA (dummies only)
ostream& operator<< (ostream& out, const _ps_int_pair)
{ return out; }

istream& operator>> (istream& in, _ps_int_pair)
{ return in; }

// the following functions are used by bucket sort
static int src(const _ps_edge& e)
{ return e.src; }

static int tgt(const _ps_edge& e)
{ return e.tgt; }

static int src(const _ps_int_pair& p)
{ return p.src; }

static int tgt(const _ps_int_pair& p)
{ return p.tgt; }


// pre- and postprocessing have run time O(|V|+|E|)

void PlanarSubgraph::_call (const graph& G, list<edge>& UnplanarEdges)
{
  //all nodes get a new number (starting with 1)
	node v;
	edge e;
	int i,j;

	node_array<int> index(G);

	// assign indices from 1 to |V|
	i = 1;
	forall_nodes(v,G)
		index[v] = i++;

	verticecount = G.number_of_nodes(); 
	edgecount = G.number_of_edges();
	adjlists *graph = new adjlists(verticecount);

	// build list of all edges, s.t. index[source] < index[target]
	// don't care about self loops
	list<_ps_edge> edges;

	forall_edges(e,G) {
		i = index[source(e)];
		j = index[target(e)];
		if (i != j)
			edges.append(_ps_edge(Min(i,j),Max(i,j),e));
	}
	// sort edges, s.t. parallel edges come after each other
	edges.bucket_sort(1,verticecount,tgt);
	edges.bucket_sort(1,verticecount,src);

	// build graph for actual algorithm
	for (list_item it = edges.first(); it != nil; ) {
		graph->insert_edge(i = edges[it].src,j = edges[it].tgt);
		do {
			it = edges.succ(it);
		} while (it != nil && edges[it].src == i && edges[it].tgt == j);
	}

	// compute planar subgraph
	adjlists* del_graph=spanplan(graph);
	delete graph;

	// build the list of deleted edges
	list<_ps_int_pair> deleted_edges;

	for (i = 0; i < del_graph->get_size(); i++) {
		if (del_graph->node_degree(i) > 0) {
			del_graph->start_at_first_node(i);

			while (!del_graph->end_of_list(i)) {
				j = del_graph->read_next_node(i);
				if (i < j) deleted_edges.append(_ps_int_pair(i,j));
			}
		}
	}

	// sort list of deleted edges according to the same ordering
	// as the list edges
	deleted_edges.bucket_sort(1,verticecount,tgt);
	deleted_edges.bucket_sort(1,verticecount,src);

	// build the list of edges in G that must be deleted
	while (!deleted_edges.empty()) {
		_ps_int_pair p = deleted_edges.head();
		while (p != edges.head())
			edges.pop();
		while (!edges.empty() && p == edges.head())
			UnplanarEdges.append(edges.pop().e);
		deleted_edges.pop();
	}

  delete del_graph;
}

AGDModule *PlanarSubgraph::clone() const 
{
  return new PlanarSubgraph;
}
