//-------------------------------------------------------------------
//
//                     VisibilityMapper.cpp
//
//    author: Carsten Gutwenger
//    date:   octobre 1997
//-------------------------------------------------------------------


#include <AGD/VisibilityMapper.h>
#include <AGD/DSegment.h>


void VisibilityMapper::init ()
{
	const graph& G = graph_A();

	left.init (G);
	right.init (G);

	get_size_x_y ();
	set_coeff ();

	node v;
	pos.init (G);
	forall_nodes (v, G) {
		pos[v] = DPoint (ax*0.5*(2*x(v)+right[v]+left[v]) + bx,
			map_y(y(v)));
	}

}

void VisibilityMapper::cleanup ()
{
	left  = node_array<int>();
	right = node_array<int>();
	pos   = node_array<DPoint>();
}

void VisibilityMapper::get_size_x_y ()
{
	const graph& G = graph_A();
	int dx, l, r;
	edge e;
	node v;
	double w, h;

	size_y = 0.0;
	size_x = min_size_x();

	forall_nodes (v, G) {
		w = width  (v);
		h = height (v);

		l = 0, r = 0;
		forall_inout_edges (e, v) {
			const IPolyline& ipl = bends(e);

			if (ipl.length() > 2)
				error_handler(1,"VisibilityMapper: no valid visibility representation");

			if (ipl.length() >= 1) {
				dx = ((v == source(e)) ? ipl.head() :
					ipl.tail()).xcoord() - x(v);
				if (dx < l) l = dx;
				if (dx > r) r = dx;
			}
		}
		double s = (w - (r-l) * sep_x()) / (r-l+1);

		left [v] = l;
		right [v] = r;

		if (s > size_x) size_x = s;
		if (h > size_y) size_y = h;
	}
}

void VisibilityMapper::set_coeff ()
{
	IRect bb = gbbox();

	ax = sep_x() + size_x;
	bx = left_margin() + size_x/2 - bb.left() * (sep_x()+size_x);

	ay = -sep_y() - size_y;
	by = upper_margin() + size_y/2 + bb.top() * (sep_y() + size_y);

	the_bbox = DRect(0.0, map_x(bb.right()) + size_x/2 + left_margin(),
		0.0, map_y(bb.bottom()) + size_y/2 + upper_margin());
}

void VisibilityMapper::map_edge (edge e, DPolyline& points,
	DPoint& s_a, DPoint& t_a)
{
	DSegment ds;
	double dy;
	IPolyline ipl = bends(e);

	if (left[source(e)] != -right[source(e)]) {
		if (ipl.length() == 0 ||
			ipl.head().ycoord() != y(source(e))) {
			ipl.push (IPoint(x(source(e)),y(source(e))));
		}
	}
	if (left[target(e)] != -right[target(e)]) {
		if (ipl.length() == 0 ||
			ipl.tail().ycoord() != y(target(e))) {
			ipl.append (IPoint(x(target(e)),y(target(e))));
		}
	}

	IPoint ip;
	forall (ip, ipl)
		points.append (DPoint(map_x(ip.xcoord()),map_y(ip.ycoord())));

	if (points.length() >= 1) {
		IPoint ip1 = ipl.head(),
			ip2 = ipl.tail();

		if (ip1.ycoord() == y(source(e))) {
			dy = height(source(e))/2;
			if (y(source(e)) > y(target(e))) {
				points.get_point(1).inc_y (dy);
			} else {
				points.get_point(1).inc_y (-dy);
			}
		}
		if (ip2.ycoord() == y(target(e))) {
			dy = height(target(e))/2;
			if (y(target(e)) > y(source(e))) {
				points.get_rev_point(1).inc_y (dy);
			} else {
				points.get_rev_point(1).inc_y (-dy);
			}
		}

	}

}

DPoint VisibilityMapper::map_node (node v)
{
	int len = right[v] - left[v] + 1;

	width (v, Max (width(v),
		(len-1)*sep_x() + len*size_x));
	return pos[v];
}
