
#include <AGD/PlanAug.h>
#include <LEDA/graph_misc.h>
#include <AGD/LEDAMakeBiconnected.h>


const int AugmentModule::in_graph = 1;
const int AugmentModule::out_graph = 1;


//-------------------------------------------------------------------
//                          AugmentModule
//-------------------------------------------------------------------

bool AugmentModule::check (const graph& G, string& message)
{
	return check_graph (in_graph, G, message);
}

void AugmentModule::call (graph& G, list<edge>& L)
{
	_call (G, L);

	_num_added_edges = L.length();
}


void AugmentModule::call (graph& G)
{
	list<edge> L;

	call (G, L);
}

void AugmentModule::hide_self_loops (graph& G, list<edge>& L)
{
	edge e;

	forall_edges (e, G) {
		if (source(e) == target(e)) {
			G.hide_edge(e);
			L.append(e);
		}
	}
}


static node_array<int>* num_ptr;
  
static int source_num(const edge& e) { return (*num_ptr)[source(e)]; }
static int target_num(const edge& e) { return (*num_ptr)[target(e)]; }

void AugmentModule::hide_multiple_edges (graph& G, list<edge>& L)
{
	list<edge> el = G.all_edges();

	if (el.empty()) return;

	node_array<int> num(G);
	int  n = 0;
	node v;
	forall_nodes(v,G) num[v] = n++;
  
	num_ptr = &num;

	el.bucket_sort(0,n-1,&source_num);
	el.bucket_sort(0,n-1,&target_num);
  
	edge e0 = el.pop();

	edge e;
	forall(e,el) {
		if (source(e0) == source(e) && target(e0) == target(e)) {
			L.append (e);
			G.hide_edge (e);
		} else {
			e0 = e;
		}
	}
 }


//-------------------------------------------------------------------
//                            PlanAug
//-------------------------------------------------------------------

PlanAug::PlanAug() : AugmentModule()
{
	add_precondition (key::planar);

	add_post_rule (key::planar);
	add_post_rule (key::biconnected);
	add_post_rule (key::simple, in_graph, key::simple);
	add_post_rule (key::no_self_loops, in_graph, key::no_self_loops);

	streng_cond (true);
}

AGDModule *PlanAug::clone() const
{
	PlanAug *M = new PlanAug();

	M->streng_cond (streng_cond());

	return M;
}

void PlanAug::_call (graph& G, list<edge>& L)
{
	list<edge> L1;

	bool undirected = G.is_undirected();

	if (undirected) G.make_directed();

	hide_self_loops (G, L1);
	hide_multiple_edges (G, L1);

	L = onehalf (G, _streng_cond);

	edge e;
	forall(e, L1) G.restore_edge (e);

	if (undirected) G.make_undirected();
}


//-------------------------------------------------------------------
//                         LEDAMakeBiconnected
//-------------------------------------------------------------------

LEDAMakeBiconnected::LEDAMakeBiconnected() : AugmentModule()
{
	add_post_rule (key::planar, in_graph, key::planar);
	add_post_rule (key::simple, in_graph, key::simple);
	add_post_rule (key::no_self_loops, in_graph, key::no_self_loops);
	add_post_rule (key::biconnected);
}

void LEDAMakeBiconnected::_call (graph& G, list<edge>& L)
{
	Make_Biconnected (G, L);
}

AGDModule *LEDAMakeBiconnected::clone() const
{
	return new LEDAMakeBiconnected;
}
