//-------------------------------------------------------------------
//
//                          bctree.cpp
//
//    author: Sergey Fialko
//    date:   february 1997
//-------------------------------------------------------------------


#include <AGD/impl/bctree.h>

//*****************************************************************************
/*****************************************************************************
void non_planar_heaps::init (list<cutpoint> lcp)  {
 cutpoint cp;
 forall (cp, lcp)
       	non_planars [cp] = new set<cutpoint>();
	
}
****************************************************************************
void non_planar_heaps::add_cutpoint (cutpoint new_cp)   {
 if (!non_planars.defined (new_cp))
 	non_planars [new_cp] = new set<cutpoint>();

 cpoints.insert (new_cp);
}*/
//****************************************************************************
bool non_planar_heaps::are_non_planar (cutpoint cp1, cutpoint cp2)  {

 if ( non_planars.defined (cp1) && non_planars[cp1].member (cp2)) return true;
 if ( non_planars.defined (cp2) && non_planars[cp2].member (cp1)) return true;
 return false;
}
//****************************************************************************
void non_planar_heaps::add_info (cutpoint main_cp, cutpointset n_planars)  {
   cutpoint cp;

   forall (cp, n_planars) {
   		non_planars[cp].insert (main_cp);
		non_planars[main_cp].insert (cp);
   }
}
//****************************************************************************
void non_planar_heaps::add_np_pair (cutpoint cp1, cutpoint cp2)  {
  non_planars[cp1].insert (cp2);
  non_planars[cp2].insert (cp1);
}
/**************************************************************
		CLASS BC-node
***************************************************************/
int bc_node_struct::continue_DFS (int i, bc_node fathr) {
 int num = dfs_number = ++i;
 bc_node bcn;
 father = fathr;

 forall (bcn, adj_nodes)
   if (!bcn->dfs_number) num = bcn->continue_DFS (num, this);

 return num;
}

//************************************************************
bc_node bc_node_struct::sohn ()  {
 bc_node bcn;
 forall (bcn, adj_nodes)
   if ( bcn->dfs_number > dfs_number)  return bcn;
 return nil;  //*this is a pendant or stand-alone block
}
//************************************************************
inline void bc_node_struct::del_bc_node ( bc_node bcn) {
	list_item li = adj_nodes.search (bcn);
	if ( li != nil ) adj_nodes.del_item (li);
}

//***************************************************************
int compare_dfs (bc_node const &bcn1, bc_node const &bcn2) 
              { return compare (dfs_number (bcn1), dfs_number(bcn2));}
//**************************************************************
inline int bl_size_ord (block const& bl1, block const& bl2 ) {
  if ( number_of_nodes (bl1) >  number_of_nodes (bl2))  return 1;
  return ( number_of_nodes (bl1) ==  number_of_nodes (bl2)) ? 0 : -1;
}
//**************************************************************
inline int bcn_size_ord (bc_node const& bc1, bc_node const& bc2 ) { 
  if ( number_of_nodes (bc1) >  number_of_nodes (bc2))  return 1;
  return ( number_of_nodes (bc1) ==  number_of_nodes (bc2)) ? 0 : -1;
}

/****************************************************************
    CLASS Block-and-Cutpoint Tree = bctree
*************************************************************/
bctree::bctree (const graph* pG, bool construct, bool DFS)
{
 G = pG;
 node_owner.init (*pG, nil);
 num_connected_comps = 0;
 if (construct)
	compute_bcTree();
 if (DFS)
	DFS_num ();
}
//**************************************************************
bctree::~bctree ()
{
  cutpoint cp;
  forall ( cp, cutpoints )
	delete cp;

  block bl;
  forall ( bl, blocks )
	delete bl;
}
//**************************************************************
void bctree::init (graph* pG, bool DFS) {
 G = pG;
 node_owner.init (*G, nil);
 
 while ( !cutpoints.empty ())
    delete cutpoints.pop ();

 while ( !blocks.empty ())
    delete blocks.pop ();

 num_connected_comps = 0;
 roots_.clear ();

 compute_bcTree();
 if (DFS) DFS_num ();
}
//***************************************************************
void bctree::update (bool DFS)  {
 while ( !cutpoints.empty ())
    delete cutpoints.pop ();

 while ( !blocks.empty ())
    delete blocks.pop ();

 node_owner.init (*G, nil);
 num_connected_comps = 0;
 compute_bcTree();

 roots_.clear ();
 if (DFS)  DFS_num ();
}
//**************************************************************
list<block> bctree::all_blocks_with_degree (int deg )  {
	list<block> result;
	block bl;
	forall (bl, blocks)
		if ( degree (bl) == deg )  result.push (bl);

	return result;
}
//**************************************************************
int bctree::number_blocks_with_degree (int deg )  {
	int result = 0;
	block bl;
	forall (bl, blocks)
		if ( degree(bl) == deg )  result++;

	return result;
}
//***************************************************************
list<bc_node> bctree::all_bcnodes ()  {
  list<bc_node> result;
  cutpoint cp;
  forall (cp, cutpoints) result.push (cp);
  block bl;
  forall (bl, blocks) result.push (bl);

  return result;
}
//**************************************************************
bc_node bctree::granddad (bc_node bcn) {
  bc_node vater = bcn->father;
  return ( vater == nil )  ?  nil  :  vater->father;
}
//***************************************************************
cutpoint bctree::new_cutpoint (node v) {
  cutpoint cp = is_cutpoint (v);
  if ( cp != nil ) return cp;

  cp = new cutpoint_struct (v);
  cutpoints.push (cp);
  node_owner[v] = cp;
  return cp;
}

//***************************************************************
cutpoint bctree::new_cutpoint (node v, block bl)
{
  cutpoint pcp = is_cutpoint (v);
  if ( pcp != nil ) {
    if ( !are_adj (pcp, bl) ) make_nodes_adj (pcp, bl);
    return pcp;
  }

  pcp = new_cutpoint (v);
  make_nodes_adj (pcp, bl);
  return pcp;
}

//***************************************************************
bool bctree::is_valid (bc_node nod)  {
if ( cutpoints.search ((cutpoint)nod) != nil || blocks.search ((block)nod) != nil )
	return true;
return false;
}
//****************************************************************
cutpoint bctree::is_cutpoint (node v) {
  bc_node owner = node_owner[v];
  if (owner != nil && owner->is_cutpoint())
     return (cutpoint)owner;

  return nil;
}
//*****************************************************************
bool bctree::is_adj_cutpoint (node v, block bl) {
  bc_node bcn = node_owner [v];
  if ( bcn == nil || !bcn->is_cutpoint ())  return false;
  return are_adj ( bcn, bl );
}
//****************************************************************
inline bool bctree::are_adj (bc_node bcn1, bc_node bcn2) {
   return ((bcn1->adj_nodes).search (bcn2) == nil ) ? false : true;
}
//*****************************************************************
block bctree::same_block (cutpoint c1, cutpoint c2) 
{
  bc_node bl;
  
  forall (bl, adj_nodes (c1))
     if ( are_adj (bl, c2))
        return (block)bl;

  return nil;
}
//*****************************************************************
node bctree::head_node ( block bl ) {
	if ( degree (bl))  return (first_adj_bcnode (bl))->first_node ();
	return nil;
}
//****************************************************************
inline void bctree::make_nodes_adj (bc_node bcn1, bc_node bcn2 ) {
	bcn1->add_bc_node (bcn2);
	bcn2->add_bc_node (bcn1);
}
//****************************************************************
void bctree::del_node (node v, block bl) {
 list_item li = (bl->nodes).search (v);
 if ( li != nil ) {
	 node_owner [v] = nil;
	 (bl->nodes).del_item (li);
 }
}
//*****************************************************************
block bctree::new_block (stack<node>& Stack, node v) {
 node nod;
 cutpoint cpoint;
 block Block = new_block ();
 do {
   nod = Stack.pop();
   if ((cpoint = is_cutpoint(nod)) != nil && !are_adj (cpoint, Block))
   		 make_nodes_adj (Block, cpoint);
   else  add_node (nod, Block);
} while ( nod != v );

 return Block;
}
//*****************************************************************
void bctree::DFS_num (bc_node rut)   {
 int Max_number = 0;
 cutpoint cp;
 block bl;
 bc_node bcn;
 list<bc_node> bcnodes = all_bcnodes ();
 roots_.clear ();

 forall (bcn, bcnodes)
	 bcn->dfs_number = 0;

 if ( rut != nil ) {
    root_ = rut;
    roots_.push (root_);
    Max_number = root_->continue_DFS (0, nil);
  }
  else    root_ = nil;

  if  ( Max_number < number_of_bcnodes () )  {
	cutpoints.bucket_sort (1, number_of_blocks (), &degree_ord);

	forall_rev ( cp, cutpoints )
	    if ( !dfs_number (cp) ) {
		 if ( root_ == nil ) root_ = cp;
		 roots_.append (cp);
		 Max_number = cp->continue_DFS (Max_number, nil );
  	    }

	 if ( Max_number < number_of_bcnodes () ) // Es sind noch Stand-Alone Blcke geblieben
	    forall ( bl, blocks )
		if ( !dfs_number (bl)) {
			bl->dfs_number = ++Max_number;
			roots_.append (bl);
			bl->father = nil ;
		}
	 if ( root_ == nil )  root_ = blocks.head ();
  }
}

//*********************************************************************
bc_node bctree::common_ancestor (bc_node bcn1, bc_node bcn2) {
  list<bc_node> parents1, parents2;

  do
   parents1.push (bcn1);
  while ( (bcn1 = father (bcn1)) != nil );

  do
   parents2.push (bcn2);
  while ( (bcn2 = father(bcn2)) != nil );

  bc_node result = nil;
  while ( parents1.head () == parents2.head ()) {
     parents1.pop ();
     result = parents2.pop ();
  }

  return result;
}
//*********************************************************************
list<bc_node> bctree::path ( bc_node bcn1, bc_node bcn2) {
  list<bc_node> parents1, parents2;

//Build the path from bcn1 to root
  do
   parents1.push (bcn1);
  while ( (bcn1 = father(bcn1)) != nil );

//Build the path from bcn2 to root
  do
   parents2.push (bcn2);
  while ( (bcn2 = father(bcn2)) != nil );

//Remove heads in parents1 and parents2 if they are equal
//lca is the last node of the eqaul nodes
  bc_node lca = nil;
  while ( !parents1.empty () && !parents2.empty () && parents1.head () == parents2.head ())
  {
     parents1.pop ();
     lca = parents2.pop ();
  }

  list<bc_node> result;
  if ( lca == nil )
    return result;

  parents1.conc (parents2);
  parents1.push ( lca );

  return parents1;
}
//****************************************************************
// root hat depth = 1
int bctree::depth (bc_node bcn)
{
  int result = 1;
  while ( father (bcn) != nil ) {
	bcn = father (bcn);
	result++;
  }
  return result;
}
//****************************************************************
void bctree::del_bc_node (bc_node bcn)  {
   bc_node adj;
   forall ( adj, bcn->adj_nodes)
   		adj->del_bc_node (bcn);

   list_item li;
   if ( is_cutpoint (bcn)) {
   		li = cutpoints.search ((cutpoint)bcn);
		if ( li != nil )
			cutpoints.del_item (li);
		delete (cutpoint)bcn;
   }
   else {
	     li = blocks.search((block)bcn);
		 if ( li != nil )
		 	blocks.del_item (li);
		 delete (block)bcn;
   }
}

//****************************************************************
// This function searches for an edge e, which is adjcent to v,
//checked[e]=false and there are no already checked edges parallel to
node bctree::find_unchecked (node v, edge_array<bool>& checked, bool only_check)
{
  edge e;
  node w = nil;
  forall_inout_edges ( e, v )
    if ( !checked [e] ) {
	if ( only_check )
        	return opposite (v, e);
      	if ( w == nil ) {
        	checked[e] = true;
		w = opposite (v, e);
    	}
    	else   // Check for parallel edges.
		if ( opposite (v,e) == w )
			checked [e] = true;
    }
 return w;
}

//*****************************************************************
//
void bctree::compute_bcTree ()
{
 node v;
 list<node> all_nodes = G->all_nodes ();

 forall ( v, all_nodes )
	if ( node_owner [v] == nil ) {
		num_connected_comps++;
		compute_SubBCTree (v);
	}
}
//*****************************************************************
// This function computes bc-tree for subgraph of G ( for whole G, if G is connected ),
// and returns the number of processed nodes ( G->number_of_nodes(), if *G is connected ).
//*****************************************************************
int bctree::compute_SubBCTree (node start)
{
 int i=1;
 node v, w;

 if ( start == nil )	v = G->first_node ();
 else {
	if ( !G->degree (start))  {
	   new_block (start);
	   return i;
	}
       v = start;
 }

 node_array<int> DFS_numb (*G,0);
 node_array<node> parent (*G,nil);
 node_array<int> low_point (*G, G->number_of_nodes()+1);
 edge_array<bool> checked (*G,false);
 stack<node> Stack;

 DFS_numb[v] = 1;
 Stack.push (v);

 do {
   while ( (w = find_unchecked (v, checked)) != nil)
      if ( DFS_numb [w] )
       low_point [v] = Min (DFS_numb [w], low_point [v]);
      else {
         parent [w] = v;
         DFS_numb [w] = ++i;
         low_point [w] = i;
         Stack.push (w);
         v = w;
       }

    if ( parent[v] != start )
      if ( low_point [v] < DFS_numb [parent[v]] )
       low_point [parent[v]] = Min(low_point [v],low_point [parent[v]]);
      else
         new_cutpoint (parent[v], new_block (Stack, v));
    else {
      if ( find_unchecked (start, checked, true) != nil )
         new_cutpoint (start);
      Stack.push (start);
      new_block (Stack, v);
    }
  v = parent [v];

 } while ( (parent[v] != nil) || (find_unchecked (v, checked, true) != nil) );

 return i;
}

//************************************************************
void bctree::print_node (bc_node bcn)
{
  if (bcn->is_cutpoint ()) {
    cout << "Cutpoint, DFS-N=" << bcn->dfs_number << ", ";
    cout << degree (bcn) << " Blocks adj, vertex: ";
    G->print_node (bcn->first_node());
    cout << "\n";
  }
  else {
    node v;
    list<node> nodes = bcn->all_nodes ();
    cout << "Block, DFS-N=" << bcn->dfs_number << ", ";
    cout << degree (bcn) << " Cutpoints adj, Knoten: ";
    forall ( v, nodes )
       G->print_node (v);
    cout << ", Face:";
//    forall ( v, ((block)bcn)->outside_nodes)  G->print_node (v);
    cout << "\n";
  }
}

//******************************************************************
// Diese Funktion
void bctree::coalesce_path (bc_node start, bc_node stop, list<block> &path_blocks )
{
 while ( start != stop ) {
    if ( !is_cutpoint (start) )
    	path_blocks.push ((block)start);
    start = father (start);
 }
}
//******************************************************************
block bctree::union_nodes (bc_node bcn1, bc_node bcn2)
{
  list<bc_node> pfad = path (bcn1, bcn2);
  bc_node bcn, cp, lca;
  if ( pfad.empty ())  return nil; //bcn1 und bcn2 sind nicht connected
  else  lca = pfad.head ();	   //lca = Gemeinsamer Vorfahren

  set<bc_node> adj_cpoints; // Dasselbe fr Cutpoints, ausserdem sind dabei auch CP's, die
  		       	    // adj zu den Blcken aus der oberen Liste sind.
  node v;

  bcn = pfad.inf (pfad.max (&bcn_size_ord));
  block newb;

  if ( is_cutpoint (bcn)) newb = new_block ();
  else  { // Ich mache den grssten Block im path zum newb
  	newb = (block)bcn;
	forall ( cp, newb->adj_nodes)  {
	           adj_cpoints.insert (cp);
	           cp->del_bc_node (newb);
	}
	(newb->adj_nodes).clear ();
	pfad.del_item (pfad.search (newb));
  }          
  	
  if ( is_cutpoint (lca))  {
      adj_cpoints.insert (pfad.pop ());
      newb->dfs_number = Min ( dfs_number(newb), dfs_number(pfad.inf(pfad.min (&compare_dfs))));
      newb->father = lca;
  }
  else  {
      newb->dfs_number = dfs_number (lca);
      newb->father = father (lca);
  }

  forall ( bcn, pfad )
	if ( !is_cutpoint (bcn) ) {
		// Addiere alle Knoten zum neuen Block
		forall ( v, ((block)bcn)->nodes )      add_node (v, newb);
		forall ( cp, bcn->adj_nodes)           adj_cpoints.insert (cp);
		del_bc_node (bcn);
     	}

   forall ( bcn, adj_cpoints )
     if ( !degree (bcn)) {
	add_node ( first_node (bcn), newb); //Jetzt haben einige "Ketten"-CP's (beide adj
	//Blocks liegen im Kreis ) zu normalen Knoten degradiert.
	del_bc_node (bcn);
     }
    else  {
         make_nodes_adj (bcn, newb); // oder eben nicht...
         if ( bcn != father(newb))    bcn->father = newb;
    }

  // Root updating
  if ( lca == root_ )
    if ( !is_valid (lca) || !is_cutpoint (lca)) {
  	root_ = newb;
	newb->father = nil;
    }

  return newb;
}
/******************************************************************
block bctree::connect_to_ancestor ( block pendant, block ancestor ) {
  bc_node bcn;
  set<cutpoint> adj_cpoints;

  set<block> path_blocks;

  bcn = pendant;

  do {
      if ( !is_cutpoint (bcn))   path_blocks.insert ((block)bcn);
      bcn = father (bcn);
  } while ( bcn != ancestor );

  block bl, newb = new block ();
  node v;
  cutpoint cp;
  
  forall (bl, path_blocks) {
       forall ( v, bl->nodes )      add_node (v, newb);
       forall ( cp, bl->adj_nodes)  adj_cpoints.insert ((cutpoint)cp);
       del_bc_node (bl);
  }
}*/
//******************************************************************
block bctree::union_pendants (cutpoint father, list<block>& sons)
{
  if ( sons.size () == 1 ) return sons.head ();

  bc_node cp;
  list<block> adj_blocks;
  node v;
  block blck;

  forall ( blck, sons )  
  	coalesce_path (blck, father, adj_blocks);

  block newb = adj_blocks.inf (adj_blocks.max (&bl_size_ord));
  (newb->adj_nodes).clear ();
  adj_blocks.del_item (adj_blocks.search (newb));
  
  if ( degree (father) > sons.size ()) {
        if ( newb->father == father )
	    father->del_bc_node (newb);

  	make_nodes_adj (father, newb);
  	newb->father = father;
  }
  else {
        add_node ( first_node (father), newb);
  	del_bc_node (father);
  	newb->father = nil;
  	root_ = newb;
  }

  set <bc_node> adj_cpoints;

  forall ( blck, adj_blocks ) {
      forall ( v, blck->nodes)
	  add_node (v, newb);

      forall ( cp, blck->adj_nodes)
	  adj_cpoints.insert (cp);

      del_bc_node (blck);
  }
  
  adj_cpoints.del (father);
  forall ( cp,adj_cpoints ) {
       add_node ( first_node (cp), newb);
       del_bc_node (cp);
  }
   
   return newb;
}
//*******************************************************************************
void bctree::connect_components (edge e)  {
  node begin = G->source (e), end = G->target (e);
  bc_node source = node_owner[begin], target = node_owner[end];
  block newb = new_block ();

  if ( is_cutpoint (source))
  	  make_nodes_adj (source, newb);
  else
	  if ( degree (source) || number_of_nodes (source) != 1 ) {
	  	del_node (begin, (block)source);
	  	cutpoint cp = new_cutpoint (begin);
	  	make_nodes_adj (source, cp);
	    make_nodes_adj (newb, cp);
	  }
	  else { // source is Isolated-Block with one node
			add_node ( first_node (source), newb );
			del_bc_node ( source );
	  }

  if ( is_cutpoint (target))
  	  make_nodes_adj (target, newb);
  else
  	if ( degree (target) || number_of_nodes (target) != 1 ) {
	  	del_node (begin, (block)target);
	  	cutpoint cp = new_cutpoint (end);
	  	make_nodes_adj (target, cp);
	    make_nodes_adj (newb, cp);
	  }
	  else { // target is Isolated-Block with one node
			add_node ( first_node (target), newb );
			del_bc_node ( target );
	  }

  num_connected_comps--;
}
//*******************************************************************************
void bctree::LEDA_Graph ( GRAPH <bc_node, int>& G, d_array<bc_node, node>& to_copy )  {
	G.clear ();

	cutpoint cp;
	forall ( cp, cutpoints )
		to_copy[cp] = G.new_node (cp);

	block bl;
	bc_node bcn;
	forall ( bl, blocks ) {
		to_copy[bl] = G.new_node (bl);
		forall ( bcn, bl->adj_nodes )
			G.new_edge ( to_copy[bl], to_copy[bcn], 0 );
	}
}
//*******************************************************************************
node bctree::adj_to_cutpoint_node ( block bl, cutpoint cp ) {
 if ( !degree (bl) )  return nil;

 node n, vert = (cp) ? vertex (cp) : head_node (bl);
 edge e;

 forall_inout_edges ( e, vert )
 	if ( owner (n = opposite (vert, e)) == bl ) return n;

 return nil; 
}
