//-------------------------------------------------------------------
//
//                         geometry.cpp
//
//    author: Carsten Gutwenger
//    date:   march 1997
//-------------------------------------------------------------------


#include <AGD/IPolyline.h>
#include <AGD/ISegment.h>
#include <AGD/DPolyline.h>
#include <AGD/DSegment.h>
#include <ctype.h>


//-------------------------------------------------------------------
//                             IPoint
//-------------------------------------------------------------------

ostream& operator<< (ostream& out, const IPoint& ip)
{
	return out << "(" << ip.xcoord() << "," << ip.ycoord() << ")";
}

// code adapted from class point for compatibility reasons!
istream& operator>> (istream& in, IPoint& ip) 
{ // syntax: {(} x {,} y {)}

  int x,y; 
  char c;

  do in.get(c); while (in && isspace(c));

  if (!in) return in;

  if (c != '(') in.putback(c);

  in >> x;

  do in.get(c); while (isspace(c));
  if (c != ',') in.putback(c);

  in >> y; 

  do in.get(c); while (c == ' ');
  if (c != ')') in.putback(c);

  ip = IPoint(x,y); 
  return in; 

 } 


//-------------------------------------------------------------------
//                             DPoint
//-------------------------------------------------------------------

ostream& operator<< (ostream& out, const DPoint& dp)
{
	return out << "(" << dp.xcoord() << "," << dp.ycoord() << ")";
}

// code adapted from class point for compatibility reasons!
istream& operator>> (istream& in, DPoint& dp) 
{ // syntax: {(} x {,} y {)}

  double x,y; 
  char c;

  do in.get(c); while (in && isspace(c));

  if (!in) return in;

  if (c != '(') in.putback(c);

  in >> x;

  do in.get(c); while (isspace(c));
  if (c != ',') in.putback(c);

  in >> y; 

  do in.get(c); while (c == ' ');
  if (c != ')') in.putback(c);

  dp = DPoint(x,y); 
  return in; 

 } 


//-------------------------------------------------------------------
//                            IPolyline
//-------------------------------------------------------------------

IPoint& IPolyline::get_point (int i)
{
	list_item it;

	for (it = first(); i > 1; it = succ(it), i--) ;

	return operator[](it);
}

IPoint& IPolyline::get_rev_point (int i)
{
	list_item it;

	for (it = last(); i > 1; it = pred(it), i--) ;

	return operator[](it);
}

ISegment IPolyline::get_segment (int i) const
{
	list_item it;
	
	if (i >= length() || i < 0)
		error_handler(1,"IPolyline: invalid segment index");

	for (it = first(); i > 1; it = succ(it), i--) ;

	return ISegment (operator[](it), operator[](succ(it)));
}

ISegment IPolyline::get_rev_segment (int i) const
{
	list_item it;
	
	if (i >= length() || i < 0)
		error_handler(1,"IPolyline: invalid segment index");

	for (it = last(); i > 1; it = pred(it), i--) ;

	return ISegment (operator[](it), operator[](pred(it)));
}


//-------------------------------------------------------------------
//                            DPolyline
//-------------------------------------------------------------------

DPoint& DPolyline::get_point (int i)
{
	list_item it;

	for (it = first(); i > 1; it = succ(it), i--) ;

	return operator[](it);
}

DPoint& DPolyline::get_rev_point (int i)
{
	list_item it;

	for (it = last(); i > 1; it = pred(it), i--) ;

	return operator[](it);
}

DSegment DPolyline::get_segment (int i) const
{
	list_item it;
	
	if (i >= length() || i < 0)
		error_handler(1,"DPolyline: invalid segment index");

	for (it = first(); i > 1; it = succ(it), i--) ;

	return DSegment (operator[](it), operator[](succ(it)));
}

DSegment DPolyline::get_rev_segment (int i) const
{
	list_item it;
	
	if (i >= length() || i < 0)
		error_handler(1,"DPolyline: invalid segment index");

	for (it = last(); i > 1; it = pred(it), i--) ;

	return DSegment (operator[](it), operator[](pred(it)));
}

