//-------------------------------------------------------------------
//
//                           oh.cpp
//
//    author: Sergey Fialko
//    date:   august 1997
//-------------------------------------------------------------------


#include <LEDA/sortseq.h>
#include <LEDA/d_array.h>
#include <math.h>
#include <AGD/impl/bctree.h>

graph *G;
//In diesem d_array werden die Soehne jeden Schnittknoten gespeichert.
d_array <cutpoint, blocklist*> sons_of_cutpoint (nil);

//Die Funktion liefert die Anzahl der Soehne zurueck
inline int n_sons (cutpoint const& cp) { return sons_of_cutpoint[cp]->size (); }

//Jeder Cutpoint hat einen Set von Cutpoints zu denen er nicht planar ist, die
// Listen werden nach und nach gefuellt - das alles ist np_heap.
non_planar_heaps np_heap;

//Falls es im Graph einen Schnittpunkt gibt mit dem Grad in BCtree > als
// (Anzahl von Pendants/2) dann wird MASSIVE = true sein
bool MASSIVE;

//*****************************************************************************
// Diese Funktion liefert true zurueck, falls die Knoten v und n adjacent sind.

bool are_adj ( node n, node v )  {
 edge e;
 forall_inout_edges ( e, n )
 	if ( opposite ( n, e ) == v )
 		return true;
 return false;
}

//***************************************************************
// Eine Funktion zum Sortieren einer Liste von Bloecken nach
// der Blockgroesse (=Anzahl der Knoten).
inline int block_size_ord (block const& bl )
        { return number_of_nodes (bl); }

//***************************************************************
// Diese Funktion berechnet fuer den Knoten n die Menge der Faces
// zu denen n gehoert (LEDA liefert ja eine Liste)

void set_of_faces ( node n, set<face>& faces )  {
    edge e;
    forall_inout_edges(e, n)
    	faces.insert (G->adj_face(e));
}

//*****************************************************************************
// Die Funktion liefert wahr falls die Kante (v, w) planar ist.
// Bevor man die Planaritaet des Graphen G+(v,w) testet, wird ueberprueft
// ob die b und w evtl. adjacent sind oder einen Face gemeinsam
// haben - dann isr (v,w) planare Kante.

bool PLANAR (node v, node w)
{
  if ( v == w || are_adj (v,w))  return true;
  else {
     set<face> first, scnd;
     set_of_faces ( v, first );
     set_of_faces ( w, scnd );
//Bestimme den Schnitt beider Mengen
     first &= scnd;

     if ( !first.empty ())
        return true;
     
     edge e = G->new_edge (v,w);
     bool planar = PLANAR (*G);
     G->del_edge (e);
     return planar;
  }
}
//**********************************************************************
// Dasselbe wie oben nur fuer den Knoten v wurde schon die Menge von
// seinen Faces berechnet.
bool PLANAR (node v, set<face>& v_faces, node w)
{
  if ( v == w || are_adj (v,w))  return true;
  else {
     set<face> w_faces, common;
     set_of_faces ( w, w_faces );
     common = w_faces & v_faces;
     if ( !common.empty ())
       return true;
     
     edge e = G->new_edge (v,w);
     bool planar = PLANAR (*G);
     G->del_edge (e);
     return planar;
  }
}
//***************************************************************************
// Die Funktion sucht fuer einen Scnittknoten cp einen hoechsten Vorfahren-
// Schnittknoten anc im bctree, so dass der Pfad zwischen dem cp und anc nur aus
// bc_nodes mit dem Grad 2 besteht (ausser vielleicht anc) und eine
// Kante (cp, anc) planar ist. 
cutpoint search_for_highest_planar ( cutpoint cp) {
  bc_node bcn = cp;
  cutpoint last_ancestor;
  
  while ( bcn != nil )  {
  	if ( degree (bcn) != 2 )
  		if ( is_cutpoint (bcn)) return (cutpoint)bcn;
  		else return last_ancestor;
  	
  	if ( is_cutpoint (bcn))   last_ancestor = (cutpoint)bcn;
   	else if ( number_of_nodes (bcn) > 2 &&
   		 !PLANAR ( first_node (first_adj (bcn)), first_node (last_adj (bcn))))
		return last_ancestor;
	bcn = father (bcn);		
  }
  return last_ancestor;
}
/****************************************************************************
/ Diese Funktion findet fr einen Pendant ( einen Block mit degree = 1 ) einen
/ "wahren" Vater, das heisst einen Vorfahren mit degree > 2, wobei alle andere
/ bc_nodes auf dem Pfad zwischen dem Block und gefundenem Vorfahren degree = 2
/ haben muessen (eine Kette), und eine Kante zwischen dem Block und dem
/ gefundenem Vorfahren muss planar sein. Wenn gefundener bc_node ein Block ist,
/ wird sein Cutpoint-Nachfolger auf dem Pfad zum Block genommen */
//*****************************************************************************
cutpoint Reduce_Path (block blok)
{
  cutpoint cp = search_for_highest_planar ( (cutpoint)father(blok));
  if ( !sons_of_cutpoint.defined (cp))
  	sons_of_cutpoint [cp] = new blocklist;
  sons_of_cutpoint[cp]->push (blok); 
  return cp;
}

//***************************************************************************
// Diese Funktion macht einen evtl. nicht zusammenhaengenden Graphen zusammen-
//haengend. Der Unterschied zum LEDA-Programm besteht darin, dass hier versucht
// wird Pendants miteinander zu verknuepfen. Insgesamt werden n - 1 Kanten
// addiert (n = #Zusammenhangskomponenten), jede Komponente bekommt hoechstens
// 2 Kanten, jeder Pendant - hoechstens eine. Wenn jede Zusammenhangs-
// komponente des Graphen auch 2-zusammenhaengend ist, dann wird der Graph 
// 2-fach zusammenhaengend gemacht. 
//***************************************************************************/
int MAKE_CONNECTED (graph& Gr, bctree& BC, list <edge> &added )   {
 if ( BC.number_of_components () == 1 )  return 0;

 blocklist isolated = BC.all_blocks_with_degree (0);
 blocklist pendants = BC.all_blocks_with_degree (1);
 block bl;
 int  nIsolated = isolated.size ();

//Sortiere Bloecke nach der Anzahl der Knoten
 pendants.bucket_sort (1, Gr.number_of_nodes (), &block_size_ord);
 
 node v, n, last_node;

 bool ONLY_ISOLATED = ( pendants.empty ()) ? true : false;
// ONLY_ISOLATED==true -> der Graph kann leicht 2-zusammenhaengend gemacht werden.
// Bilde eine Kette ueber alle isolierten Bloecke des Graphen. 
// Schiliesse die Kette mit einer Kante vom letzten zum ersten Block in 
// der Kette ab. last_node ist ein Knoten im ersten Block der Kette.
// Jede Komponente im Graphen bekommt zwei neue inzidente Kante, die
// Kanten muessen mit verschiedenen Knoten inzident sein (falls die Anzahl
// der Knoten in Komponente >1 ist).
 edge e;


 if ( ONLY_ISOLATED ) last_node = first_node (isolated.pop ());
 else last_node = BC.adj_to_cutpoint_node ( pendants.pop ());

 if ( ONLY_ISOLATED ) { 
   if ( indeg (last_node) + outdeg (last_node) == 0 )//ist isolierter Knoten
     v = last_node;
   else {
     if ( outdeg (last_node))
       e = Gr.first_adj_edge (last_node);
     else
       e = Gr.first_in_edge (last_node); 
     v = opposite (last_node, e);
   }

   n = first_node (isolated.tail ());
   if ( indeg (n) + outdeg (n) > 0 ) {
      e = ( Gr.first_adj_edge (n)) ? Gr.first_adj_edge (n) : Gr.first_in_edge (n);
      node tmp = opposite (n, e);
      n = tmp;
   }
   added.push ( Gr.new_edge (n, v));
 }//Ich habe jetzt letzte, abschliessende Kante hinzugefuegt. 

 list<bc_node> roots = BC.roots ();//Die Wurzel-bc-nodes der ZHK's

 //Verknuepfe die isolierten Bloecke miteinander
 while ( !isolated.empty () ) {
   bl = isolated.pop ();
   roots.del_item (roots.search (bl));

   n =  first_node(bl);

   if ( number_of_nodes (bl) == 1 ) e = nil;
   else e =(Gr.first_adj_edge (n)) ? Gr.first_adj_edge (n) : Gr.first_in_edge (n);

   added.push ( Gr.new_edge ( last_node, n));
   last_node = (e) ? opposite (n, e) : first_node(bl);

 }
 if ( ONLY_ISOLATED ) 
   return added.size ();

 if ( BC.number_of_components () - nIsolated == 1 )
   return added.size ();
   
 sortseq <int, bc_node> roots_nums;
 bc_node bcn;
//Mit Hilfe von roots_nums werde ich Pendants nach den ZHK zu denen sie
//gehoeren sortieren. Jede ZHK darf hoechstens 2 neue Kanten bekommen,
//die mit Pendants inzident sein muessen. 

 forall ( bcn, roots )
   if (degree (bcn))
     roots_nums.insert (dfs_number (bcn), bcn);

//In dieser Map hat jede Wurzel die Liste der Pendants, die zu ihrer
//ZHK gehoeren. Ein Pendant p gehoert zu der ZHK, derer Wurzel maximale
//DFS-Nummer hat, die kleiner als die Nummer von p ist (locate_pred).

 map <bc_node, blocklist> trees;
 
 forall ( bl, pendants ) {
   bcn = roots_nums.inf ( roots_nums.locate_pred (dfs_number (bl)));
   trees[bcn].push (bl);
 }
 
 block last_pendant = nil;

 while ( !roots.empty () ) {//Verknuepfe Komps
    bcn = roots.pop ();

    if ( trees[bcn].size () == 1 && last_pendant && roots.size () > 0 ) {
         roots.append (bcn);
	 bcn = roots.pop ();
    }
    
    bl = trees[bcn].pop ();
    if ( last_pendant ) 
       added.push ( Gr.new_edge ( BC.adj_to_cutpoint_node (last_pendant),
				  BC.adj_to_cutpoint_node (bl)));
    
    if ( !trees[bcn].empty ())
        last_pendant = trees[bcn].pop ();
    else 
        last_pendant = bl;
 }

 return added.size ();
}
//******************************************************************************
//Diese Funktion liefert true, wenn der Pfad vom cp1 zum cp2 mindestens
//zwei bc-nodes  mit dem Grad >2 oder mindestens einen Block mit dem Grad 
// >3 enthaelt. Dabei werden die Grade von cp1 und cp2 um  n_sons (cp2)
//reduziert.
bool connect_condition ( cutpoint cp1, cutpoint cp2, bctree& BC )  {
   list<bc_node> path = BC.path (cp1, cp2);
   
   if ( MASSIVE && path.head () == BC.root ())
      return true;

   int deg1 = 0, deg2 = 0;
   
   if ( degree (cp1) > n_sons (cp2) + 1)
     deg1 =  degree (cp1) -  n_sons (cp2) + 1;
   
   path.del_item (path.search (cp1));

   if ( degree (cp2) > n_sons (cp2) + 1)
     deg2 =  degree (cp2) -  n_sons (cp2) + 1;

   path.del_item (path.search (cp2));

   if ( deg1 && deg2 ) return true;

   path.bucket_sort ( 1, BC.number_of_bcnodes (), &degree );
   
   bc_node bcn1 = path.Pop ();
   bc_node bcn2 = ( path.empty ()) ? nil : path.Pop ();
  
   if ( deg1 == 0 ) deg1 = degree(bcn1);
   else if ( deg2 == 0 ) deg2 = degree(bcn1);

   if ( bcn2 && deg2 == 0 ) deg2 = degree(bcn2);

   if ( deg2 > 2 && deg1 > 2 ) return true;
   if ( !is_cutpoint (bcn1) && degree (bcn1) > 3 ) return true;
   if ( bcn2 &&  !is_cutpoint (bcn2) && degree (bcn2) > 3 ) return true;
   	
   return false;
}
//*******************************************************************************
list<edge> onehalf (graph &GO, bool STRENG_COND )  {

 G = &GO;
// if ( !PLANAR (*G)) {
//	cout << "Onehalf: Input Graph is not Planat. Program exit\n";
//	exit (1);
// }

 block blok;
 cutpoint cp, first;
 blocklist pendants;
 edge e;
 list<edge> result, added_to_bidirect;
 list<cutpoint> fathers;

 bctree BC (G, true, true);

 if ( MAKE_CONNECTED (*G, BC, result))
      BC.update (true);

 Make_Bidirected (*G, added_to_bidirect);
 G->make_planar_map ();


 forall (blok, pendants = BC.all_blocks_with_degree (1))
   Reduce_Path (blok);

 MASSIVE = ( degree (BC.root ()) - 1 > ceil ((float)pendants.size ()/2)) ? true : false;

//Jetzt erstelle ich eine Liste von Schnittpunkten die Soehne haben
 forall_defined ( cp, sons_of_cutpoint)
    fathers.push (cp);

 while ( fathers.size () > 0 ) {
// Suche jetzt in der Liste fathers einen Cutpoint der die hchste Anzahl von Pendants hat
   fathers.bucket_sort (0, pendants.size (), &n_sons);
   //   check_all (BC, fathers);
   first = fathers.Pop (); //Unter dem first habe ich jetzt einen Cp mit Maximaler #Sons

   set<face> first_faces, cp_faces, common_fa;
   set_of_faces (vertex (first), first_faces);

   cutpoint best_cp = nil, good_cp = nil;

// Suche jetzt einen anderen Schnittknoten v der zum first planar ist
// hoechste n_sons(v) hat.
   forall ( cp, fathers ) {
	if ( !STRENG_COND && best_cp && n_sons (best_cp) > n_sons (cp))  break;
	if ( !np_heap.are_non_planar (first, cp)) {
		bool condition = connect_condition (cp, first, BC );
   		if ( condition || (!good_cp && STRENG_COND) || !(best_cp || STRENG_COND))
   			if ( PLANAR (vertex(first), first_faces, vertex (cp))) {
 				if ( condition ) {
 					best_cp = cp;
 					break;
 				}
 				else if (STRENG_COND) 	good_cp = cp;
 				     else  	    	best_cp = cp;
			}
			else np_heap.add_np_pair (cp, first);
	}
  }
 
   //  np_heap.add_info (first, found_non_planars);

  if ( !best_cp ) best_cp = good_cp;

  if ( !best_cp )  { // Der first ist zu keinem Cutpoint PLANAR oder es gibt keine mehr
		     // Verknuepfe Soehne vom first miteinander
    // 	list<block>* sons = sons_of_cutpoint [first];
       	block bl2, bl1 = sons_of_cutpoint[first]->head ();
       	
       	forall (bl2, *sons_of_cutpoint[first]) {
       	    if ( bl2 == sons_of_cutpoint[first]->head ()) continue;
       	    e = G->new_edge ( BC.adj_to_cutpoint_node (bl1),
			      BC.adj_to_cutpoint_node (bl2));

       	    result.push (e);
       	    added_to_bidirect.push (G->new_edge (target (e), source (e)));
       	    bl1 = bl2;
       	}

       	blok = BC.union_pendants ( first, *sons_of_cutpoint [first]);
       	delete sons_of_cutpoint [first];
       	sons_of_cutpoint.undefine (first);

     	if ( BC.number_of_blocks () == 1 )  break; //Done - Graph is Biconnected
	if ( BC.root () == first )  BC.DFS_num ( fathers.head ());

	// Suche hoechsten Vorfahren-Block ( = ancestor) der zum blok Planar ist
	cp = search_for_highest_planar (first);
	block ancestor = (block)father (cp);

    	e = G->new_edge (BC.adj_to_cutpoint_node (ancestor, cp),
			 BC.adj_to_cutpoint_node (blok));
    	
    	result.push (e);
    	added_to_bidirect.push (G->new_edge (target (e), source (e)));

    	blok = BC.union_nodes ( ancestor, blok );
	//    	G->make_planar_map ();
  }
   else {  // Es wurde einer gefunden
 /* Jetzt verknuepfe ich die Pendants vom first und best_cp miteinander und
 update BCtree = nach dem Verknpfen bekomme ich ein neues Block der aus allen
 BC-Nodes zwischen Pendants vom first und best_cp und lca vom first und best_cp
  besteht. Dabei versuche ich fr alle Paare von Pendants einen gemeinsamen Face
 zu finden. Wenn es mir fr alle Paare gelingt, dann brauche ich, nach dem Verknpfen,
 planar_map nicht zu  aktualisieren. */

     block pend1, pend2 ;
     blocklist first_sons;
     bool rebuild_map = false;
//falls ich fuer ein Paar kein gemeinsames Face finde, dann ist rebuild_map = true
          
     forall ( pend2, *sons_of_cutpoint[best_cp] )  {
	pend1 = sons_of_cutpoint[first]->pop ();
	first_sons.push (pend1);
	
	if ( !rebuild_map )  {
	     cutpoint fath2 = (cutpoint)father(pend2);
	     set<face> fath2_faces;

	     //Berechne die Menge der Faces von fath2
	     set_of_faces (vertex(fath2), fath2_faces);
		
	     //Finde einen gemeinsamen Face fuer fath2 und first
	     fath2_faces &= first_faces;
	
	     edge e1 = nil, e2 = nil;
	
	     if (!fath2_faces.empty ()) {//Es gibt einen mehrere--? gemeinsamen Face
			face fa;
			forall (fa, fath2_faces) {
				forall_face_edges (e, fa) {
				  if ( BC.owner (source (e)) == pend1 )	e1 = e;
				  if ( BC.owner (source (e)) == pend2 )	e2 = e;
				}
				if ( e1 && e2 ) break;
				else 		e1 = e2 = nil;
			}
		}
		if ( e1 && e2 )  {
			e = G->split_face (e1, e2);
			result.push (e);
			added_to_bidirect.push ( G->reversal (e));
		}
		else 
			rebuild_map = true;
	}
		
	if ( rebuild_map )  {
		e = G->new_edge ( BC.adj_to_cutpoint_node (pend1),
				  BC.adj_to_cutpoint_node (pend2));
		result.push (e);
		added_to_bidirect.push (G->new_edge (target (e), source (e)));
	}
     }
     
     blok = BC.union_nodes (BC.union_pendants (first, first_sons),
     			    BC.union_pendants (best_cp, *sons_of_cutpoint[best_cp]));

     fathers.del_item (fathers.search (best_cp));
     delete sons_of_cutpoint[best_cp];
     sons_of_cutpoint.undefine (best_cp);
// Falls first jetzt den Grad 2 hat, dann koennen wir die Reduce_Chain-Prozedur f"ur den
// Sohn von first noch einmal anwenden, weil dieser Pendant jetzt einen anderen Vater haben kann. 
     if ( n_sons (first)) {
       if ( degree (first) == 2 ) {
	 cutpoint new_father = search_for_highest_planar (first);
	 if ( new_father != first) {
	   if ( !sons_of_cutpoint.defined (new_father)) {
	     sons_of_cutpoint [new_father] = new blocklist;
             fathers.append (new_father);
	   }
	   sons_of_cutpoint[new_father]->push (sons_of_cutpoint[first]->front());
	   if ( n_sons (first) == 2 )
	       sons_of_cutpoint[new_father]->push (sons_of_cutpoint[first]->back());

	   delete sons_of_cutpoint[first];
	   sons_of_cutpoint.undefine (first);
	 }
	 else
	   fathers.append ( first );
       }
       else
     	   fathers.append ( first );
     }
     else {
       delete sons_of_cutpoint[first];
       sons_of_cutpoint.undefine (first);
     }

     if ( rebuild_map )		G->make_planar_map ();
//     if ( !BC.is_valid (BC.root ()))  BC.DFS_num ();
  }
  /*  Updating : Neu nach dem ein neuer Block erzeugt wurde, kann auch er Pendant
  // werden und in diesem Fall soll er in weitere Processing miteinbezogen werden
  // (in pendants eingefuegt werden, und die */
  if ( degree (blok) == 1 ) {
    if ( !father (blok))  BC.DFS_num (fathers.head());
    //    create_pend++;
    first = Reduce_Path ( blok);

    if ( !fathers.search (first)) {
	fathers.push (first);
	//	np_heap.add_cutpoint (fathers.tail ());
    }
  }
}

 forall_defined (cp, sons_of_cutpoint ) {
 	delete sons_of_cutpoint[cp];
	sons_of_cutpoint.undefine (cp);
 }
 
 forall ( e, added_to_bidirect ) 	G->del_edge (e);	

 return result;
}
/* Planaritt wird getestet:
   1. Beim Reduce Paths ( node-node )
   2. Wenn zu einem Cutpoint ein anderer Planarer gesucht wird ( node-node )
   4. Wenn fuer  einen  pendant  highest_planar  gesucht  wird ( node-node ) */
