/************************************************************************

         Filename   :   spanplan.cc

         Version    :   09.1994

         Author     :   Sebastian Leipert

         Language   :   C++


 File containing the procedure main and procedures for input and output
 and for computing the biconnected components of a graph.

************************************************************************/ 



#include <AGD/impl/MaxPlanSub/proto.h>

     
  int      verticecount;           // number of nodes in graph
  int      edgecount;              // number of edges in graph
  int      total_num_del_edges;    // total number of deleted edges
  int      node_count;             // counts the number of nodes
                                   // in the actual pq-tree 
  pq_node  *root;                  // root of the actual pq-tree

  int      _printDuringReduction;
  int      _printPQTreeIterative;
  int      _iterationNumber;
  char     _debugInformation[INFOSIZE];
	

/************************************************************************
                           main
************************************************************************/ 

adjlists* spanplan(adjlists* graph)
{       
  total_num_del_edges = 0;
  node_count = 0;
  root = NULL;

  _printDuringReduction = FALSE;
  _printPQTreeIterative = FALSE;
  _iterationNumber = 0;

  adjlists *adjlptr=new adjlists(verticecount);
  adjlptr=dfs(&graph);                  // start the program

  return adjlptr;
} 

/************************************************************************
                          dfs
************************************************************************/ 

adjlists* dfs(adjlists **graph)

  // Computes the connected components of the graph. If a connected
  // component is found, the procedure dfs_for_bcc is called in order 
  // to find the biconnected components.

{
  int     *visited; 
  int     dfbi,i;
  int     *label;
  int     *dfbilabel;
  int     *father;
  int     blockcount;   // counts the number of blocks
  int     neighbor;

  dfbi = 0;
  blockcount = 1;
  
  adjlists deleted_edges(verticecount);  // stores the removed edges

  visited = (int *) calloc ((verticecount+1),sizeof(int));
  label = (int *) calloc ((verticecount+1),sizeof(int));
  father = (int *) calloc ((verticecount+1),sizeof(int));
  dfbilabel = (int *) calloc ((verticecount+1),sizeof(int));

  stack_2 edgestack(2*edgecount+1,2*edgecount+1);
                              // stores the edges of a biconnected component 

  for (i=1; i <= verticecount; i++) {
    if (visited[i] == FALSE) {
                                        // Connected component is found.
        dfs_for_bcc(i,visited,label,dfbilabel,&dfbi,father,
                    &blockcount,&edgestack,*graph,&deleted_edges);
    }
  }

                                        // Print out the removed edges
                                        // and remove them from the graph
                                        // to get a planar subgraph.
    
  delete[] visited;
  delete[] label;
  delete[] father;
  delete[] dfbilabel;
  
  adjlists* del_graph=new adjlists(verticecount);

  for (i=1; i <= verticecount; i++)   
    if (deleted_edges.node_degree(i) > 0)
    {
      deleted_edges.start_at_first_node(i);
      while (!deleted_edges.end_of_list(i)) {
        neighbor = deleted_edges.read_next_node(i);
        if (neighbor > i) {
          del_graph->insert_edge(i,neighbor);
        } // if 
      } // while
    } // if
 
  return del_graph;
} // dfs



/************************************************************************
                          dfs_for_bcc
************************************************************************/

void dfs_for_bcc(
                 int node,
                 int *visited,
                 int *label,
                 int *dfbilabel,
                 int *dfbi,
                 int *father,
                 int *blockcount,
                 stack_2 *edgestack,
                 adjlists *graph,
                 adjlists *deleted_edges
                )

   // Computes for each connected component all biconnected components.
   // If a biconnected component is found, the procedure bic_comp_found
   // is called for further progression.
{


   int    neighbor;


   visited[node] = TRUE;
   (*dfbi)++;
   dfbilabel[node] = *dfbi;
   label[node] = *dfbi;
   while (!(*graph).end_of_list(node))
   {
      neighbor = (*graph).read_next_node(node);
      if (!(*graph).old_edge_marked(node))
      {
         edgestack->push(node,neighbor);
         (*graph).mark_old_edge(node);
      } // if
      if (!visited[neighbor])
      {
         father[neighbor] = node;
         dfs_for_bcc(neighbor,visited,label,dfbilabel,dfbi,father,
                    blockcount,edgestack,graph,deleted_edges);
         if (label[neighbor] >= dfbilabel[node])
                                          // Biconnected component is found.
         {
            bic_comp_found(edgestack, *blockcount,node,neighbor,deleted_edges);
            (*blockcount)++;
         } // if
         if (label[node] > label[neighbor])
           label[node] = label[neighbor];
      } // if
      else if (neighbor != father[node])
        if (label[node] > dfbilabel[neighbor])
          label[node] = dfbilabel[neighbor];
   } // while
} // dfs_for_bcc
         


/************************************************************************
                          bic_comp_found
************************************************************************/ 

void bic_comp_found(
                    stack_2 *edgestack,
                    int,
                    int node1,
                    int node2,
                    adjlists *deleted_edges
                   )

   // Gathers all edges of the found biconnected component into a new
   // object of the class adjlists named block. Then calls the procedure
   // st-number for this block in oder to compute an st-numbering.
   // Then calls the procedure planar in order to planarize the biconnected
   // component named block.

{

   int      i;
   int      found;
   int      finode,senode;
   int      t;               // st-number of the sink = number of vertices
                             // in the block
   int      *st_no;          // array storing the st-numbers of the vertices
   int      *no_st;          // array storing to every st-number the
                             // corresponding vertex-number
   int      neighbor;


   found = FALSE;
   adjlists block(verticecount);  // stores the biconnected component
   
   st_no = (int *) calloc ((verticecount+1),sizeof(int));
   no_st = (int *) calloc ((verticecount+1),sizeof(int));
   while ((!edgestack->stack_empty()) && (!found))
                              // Store the biconnected component in block.
   {
      edgestack->pop(&finode,&senode);
      if ((finode == node1 && senode == node2) ||
          (finode == node2 && senode == node1))
         found = TRUE;
      block.insert_edge(finode,senode);
   }// while

                              // Compute st-numbering for the block.
   t = st_number(&block,&st_no,&no_st);

   adjlists del_edges(t);     // Stores the removed edges of the block.

                              // Planarize the block.
   planar(&block,t,&del_edges);

                              // Add the removed edges of block to
                              // the removed edges of the graph.
                              // Thats where we need the array no_st for,
                              // since we need the original numbers of the
                              // incident vertices, but del_edges contains
                              // st-numbered vertices.
   for (i=1; i <= t; i++)
   {
      if (del_edges.node_degree(i) > 0)
      {
         del_edges.start_at_first_node(i);
         while (!del_edges.end_of_list(i))
         {
            neighbor = del_edges.read_next_node(i);
            if (no_st[neighbor] > no_st[i])
               deleted_edges->insert_edge(no_st[neighbor],no_st[i]);
         }// while
      }// if
   }// for
   
   delete[] st_no;
   delete[] no_st;
}// bic_comp_found





/************************************************************************
                     printNode
************************************************************************/ 



void printNode(char *filename,int number,
               pq_node* parent,pq_node* son)

{
   ofstream output(filename,ios::app);


   output << "e " << parent->debugTreeNumber
          << ' ' <<number << endl;

   output << "n " << number << ' ' << "\\i"
          << (son->typ == Q_NODE ? " Q-NODE" :
             (son->typ == P_NODE ? " P-NODE" : " LEAF") )
          << " TotalCount: " << son->total_count
          << ", Status: "
          << (son->label == FULL ? " FULL" :
             (son->label == EMPTY ? " EMPTY" :
             (son->label == PARTIAL ? " PARTIAL" :
             (son->label == PERTINENT ? " PERTINENT" : 
	     (son->label == TO_BE_DELETED ? "TO_BE_DELETED" :
             (son->label == KILLED ? "KILLED" :
	      "ERROR"))))))
          << "   Key: " << son->key
          << "  Leafnumber: " << son->leafnum
          << "\\i" << endl;

   output << "c " << number << ' '
          << (son->typ == Q_NODE ? Q_NODE_CAT			     :
             (son->typ == P_NODE ?  P_NODE_CAT			     :
             (son->label == FULL ? LEAF_FULL_CAT		     : 
             (son->label == TO_BE_DELETED ? TO_BE_DELETED_CAT	     :
             (son->label == KILLED  ? KILLED_CAT		     :
              LEAF_CAT)))))
          << endl;
  
   son->debugTreeNumber = number;
}







/************************************************************************
                     print_out_pq_tree
************************************************************************/ 
              

void print_out_pq_tree(char *name,char *ending,int i)
{


   char      filename[INFOSIZE];
   FILE      *debugInfo;


   strcpy(filename,name);
   strcat(filename,ending);

   ofstream output(filename,ios::out);
   output << "#TYPE: COMPLETE TREE" << endl
          << "#TIME: NOT" << endl
          << "#BOUNDS: NONE" << endl
          << "#INFORMATION: STANDARD" << endl
          << "#NODE_NUMBER: NONE" << endl;




   int   _count = 1;
   root->debugTreeNumber = _count;

   
   output << "n " << _count << ' ' << "\\iROOT: "
          << (root->typ == Q_NODE ? " Q-NODE" : "P_NODE")
          << ", TotalCount: " << root->total_count
          << ", Status: "
          << (root->label == FULL ? " FULL" :
             (root->label == EMPTY ? " EMPTY" :
             (root->label == PARTIAL ? " PARTIAL" :
             (root->label == PERTINENT ? " PERTINENT" : 
	     (root->label == TO_BE_DELETED ? "TO_BE_DELETED" :
             (root->label == KILLED ? "KILLED" :
	      "ERROR"))))))
          << "\\i" << endl;
          

   output << "c " << _count++ << ' '
          << (root->typ == Q_NODE ? Q_NODE_CAT : P_NODE_CAT) << endl;

  pq_node    *check_node = NULL;
  pq_node    *first_son = NULL;
  pq_node    *next_son = NULL;
  pq_node    *new_son = NULL;
  pq_node    *last_son = NULL;
  
  

  debugInfo = fopen(_debugInformation,"a");

  fprintf(debugInfo,"\n\n");
  fprintf(debugInfo,"Schleifendurchlauf Nummer %d :\n",i);
  queue<pq_node*> *helpqueue = new queue<pq_node*>;

  fprintf(debugInfo,"ROOT :\n");
  if (root->typ == Q_NODE)
     fprintf(debugInfo,"Q-NODE ");
  else if (root->typ == P_NODE)
     fprintf(debugInfo,"P-NODE ");

  fprintf(debugInfo," : key: %d leafnum: %d total_count: %d\n",root->key,
           root->leafnum,root->total_count);
  
  if (root->typ == P_NODE)
  {  if (root->child_sib_list != NULL) 
     {
        first_son = root->child_sib_list;

        printNode(filename,_count++,root,first_son);

        helpqueue->enqueue(first_son);
        if (first_son->sib_next != NULL)
           next_son = first_son->sib_next;
        while (next_son != first_son)
        {
           printNode(filename,_count++,root,next_son);

           helpqueue->enqueue(next_son);
           next_son = next_son->sib_next;
        }
     }
  }
  else if (root->typ == Q_NODE)
  {  root->endmost_children->start_at_bottom();
     first_son = root->endmost_children->read_next();
     last_son = root->endmost_children->read_next();

     printNode(filename,_count++,root,first_son);
     printNode(filename,_count++,root,last_son);
     
     helpqueue->enqueue(last_son);
     last_son->imm_siblings->start_at_bottom();
     next_son = last_son->imm_siblings->read_next();
     if (next_son != first_son)
        printNode(filename,_count++,root,next_son);

     helpqueue->enqueue(next_son);
     while (next_son->imm_siblings->count() > 1)
     {
        next_son->imm_siblings->start_at_bottom();
        new_son = next_son->imm_siblings->read_next();
        if (new_son == last_son)
           new_son = next_son->imm_siblings->read_next();

        if (new_son != first_son)
           printNode(filename,_count++,root,new_son);

        helpqueue->enqueue(new_son);
        last_son = next_son;
        next_son = new_son;
     }
  }


  while (!helpqueue->queue_empty())
  {  check_node = helpqueue->dequeue();
     if (check_node->typ == LEAF)
        fprintf(debugInfo,"LEAF   ");
     else if (check_node->typ == P_NODE)
     {  fprintf(debugInfo,"P_NODE ");
        if (check_node->child_sib_list != NULL) 
        {
           first_son = check_node->child_sib_list;
           printNode(filename,_count++,check_node,first_son);
           helpqueue->enqueue(first_son);
           if (first_son->sib_next != NULL)
              next_son = first_son->sib_next;
           while (next_son != first_son)
           {
              printNode(filename,_count++,check_node,next_son);
              helpqueue->enqueue(next_son);
              next_son = next_son->sib_next;
           }
        }
     }
     else if (check_node->typ == Q_NODE)
     {  fprintf(debugInfo,"Q_NODE ");
        check_node->endmost_children->start_at_bottom();
        first_son = check_node->endmost_children->read_next();
        last_son = check_node->endmost_children->read_next();

        printNode(filename,_count++,check_node,first_son);
        printNode(filename,_count++,check_node,last_son);

        helpqueue->enqueue(last_son);
        last_son->imm_siblings->start_at_bottom();
        next_son = last_son->imm_siblings->read_next();
        if (next_son != first_son)
           printNode(filename,_count++,check_node,next_son);
           
           
        helpqueue->enqueue(next_son);
        while (next_son->imm_siblings->count() > 1)
        {
           next_son->imm_siblings->start_at_bottom();
           new_son = next_son->imm_siblings->read_next();
           if (new_son == last_son)
              new_son = next_son->imm_siblings->read_next();

           if (new_son != first_son)
              printNode(filename,_count++,check_node,new_son);
           helpqueue->enqueue(new_son);
           last_son = next_son;
           next_son = new_son;
        }
     }
     fprintf(debugInfo," : key: %d leafnum: %d total_count: %d debug_num: %d",
            check_node->key,check_node->leafnum,
            check_node->total_count,check_node->debugTreeNumber);
     fflush(stdout);
     fprintf(debugInfo,"  parent : %d \n",check_node->parent->total_count);
  }
  fflush(stdout);

  delete helpqueue;

  fclose(debugInfo);

}












