#include <AGD/impl/cons.h>
//****************************************************************************
//*****************************************************************************
PLANAR_CONSTRAINT::PLANAR_CONSTRAINT (ABA_MASTER *m, const ABA_ARRAY<int> &sup,
				      const ABA_ARRAY<double> &coeff, int num)
		     : ABA_SROWCON ( m, 0, ABA_CSENSE::Less, sup.size (),
                       sup, coeff, sup.size () - 1, false, false, false), 
		       number_ (num)
{
 long int prod = 1;
 int i;
 for ( i=0; i < sup.size (); i++)
     prod *= sup[i];

 Key = (prod*12345679)%1113331;//eine gute primzahl zu finden
}
//*****************************************************************************
bool PLANAR_CONSTRAINT::equal (ABA_CONVAR* other)
{
   PLANAR_CONSTRAINT* con = (PLANAR_CONSTRAINT*)other;

   if ( row ()->rhs () != con->row()->rhs ())
	return false;
      
   if ( row ()->nnz() != con->row ()->nnz())
   	return false;
	
   int i;
   for ( i = 0; i < row ()->nnz(); i++ )
   // PLANAR_CONSTRAINTs have only 1.0 (or zero) coeff -> I have only to check supports. It can 
   // be made faster because the array of supports (indexes) is sorted 
       if ( row ()->support (i) != con->row ()->support (i))
       		return false;
   
   return true;
}   
//***************************************************************************
void PLANAR_CONSTRAINT::print ()
{
    int i;
    cout << row ()->nnz () << " - NNZ\n";
    for ( i = 0; i < row ()->nnz (); i++ )
    	cout << row()->support(i) << " ";
    
    cout << "\n";
}
//****************************************************************************
// We overwrite this funktion because we know that all coefficients are 1's
// and the non-violation can be checked faster. We need only to sum values
// of Variables (without multiplications)
//
bool PLANAR_CONSTRAINT::violated (ABA_ACTIVE<ABA_VARIABLE, ABA_CONSTRAINT> *vars,
                          	  double *x, double *sl)
{
  double sum = 0;
  int i;
  
  if (sl)   {
  	for ( i = 0; i < row ()->nnz (); i++)
  		sum += x[row()->support(i)];

        if (  master_->equal (sum, row()->rhs ()))
  	    *sl = 0;
        else
	    *sl =  row()->rhs () - sum;

  	if ( *sl >= 0 )
  		return false;
  	else
  		return true;
  }
  // the slack can be not computed
  double slack = 0;
  for ( i = 0; i < row ()->nnz (); i++)  {
	slack += 1.0 - x[row()->support(i)];
 // Return earlier if the slack is already greater than 1.0 
	if ( slack - master_->machineEps () > 1.0)
		return false;
  }
  return true;
}
//*****************************************************************************
double PLANAR_CONSTRAINT::slack(ABA_ACTIVE<ABA_VARIABLE, ABA_CONSTRAINT> *variables, double *x)
{
  double sum = 0;
  int i;
  
  for ( i = 0; i < row ()->nnz (); i++)
  	sum += x[row()->support(i)];

  if ( master_->equal (sum, row()->rhs ()))
       return 0;
  
  return row()->rhs () - sum;
}

//***************************************************************************
// this funktion searches for a variable in this constraint that has minimal
// value in the vector x. Return value: the number of found variable
// If all variables have the value 1.0 then the number of first variable
// will be returned.

int PLANAR_CONSTRAINT::find_min (double *values)
{
 int i, min_number;
 double min_value = 1.0;
 for ( i = 0; i < row()->nnz (); i++) {
    int num = row()->support(i);
    if ( values[num] < min_value)    {
       min_value = values[num];
       min_number = num;
    }
 }
 if ( min_value == 1.0 )
    return row()->support (0);
 
 return min_number;
}
//************************************************************************
// This function returns true iff each variable in *this constraint
// is also present in the list "varset". "varset" must be sorted upwards.

bool PLANAR_CONSTRAINT::is_subset_of (list<int>& varset)
{
  int position = 0, num = row()->support(0);
  int i;  

  forall (i, varset) {
     if ( i > num )
         return false;

     if ( i == num) {
        position++;//search for next support
		if ( position == row()->nnz())//All supports have been checked
			return true;
		num = row()->support(position);
     }
  }
  return false;//max elem. in varset < num
}
//************************************************************************
// selected must be sorted
// Checks the status of this constraint for the vector x.
// 
PLANAR_CONSTRAINT::VIOLATE_STATUS PLANAR_CONSTRAINT::violated_or_equal 
(double *x, list<int>& selected)
{
  int i = 0;
  double slack = 0;
  do   {
     slack += 1.0 - x[row()->support (i)];
     i++;
  }  while (i < row()->nnz() && slack - master_->machineEps () <= 1.0);

  if ( slack - master_->machineEps () > 1.0 )
     return NOT_VIOLATED;

  if ( master_->equal (slack, 1.0))
     if ( is_subset_of (selected))
         return EQUAL;
     else 
         return NOT_VIOLATED;

  return VIOLATED;
}
//************************************************************************
// Binary search for the number var in this->support
//
int PLANAR_CONSTRAINT::coeff (int var) {
  int low = 0, high = row()->nnz (), middle;
  
  while (low < high)  {
	middle = (low+high)/2;
	if ( var <= row()->support (middle))
		high = middle;
	else
		low = middle + 1;
  }

  if (var == row()->support (low))
	return 1;
  else
	return 0;
}
