#include <AGD/impl/planarisator.h>
#include <math.h>
#include <iostream.h>

//***************************************************************************
// finds a variable with maximal Importance (=maximum in array)
// Returns the position of found variable in array;

int VarImportance::find_max () {
  int i, maxFound = 0, maxPosition = -1;
  for(i = 0; i <= numRefs_.high (); i++)
	if (numRefs_[i] > maxFound)
		maxPosition = i;
  return maxPosition;
}
//******************************************************************************

//*******************************************************************************
PLANARISATOR::PLANARISATOR (graph &Gr, int LowerBound, double eps, double machEps)
       : ABA_MASTER("PLANARISATOR", true, false, ABA_OPTSENSE::Max, eps, machEps, 1.0e30),
       G(&Gr), LowerB (LowerBound), numLastCons(-1), nKuratowski (0),
       LogStatisticsToFile(false), separatedFromPool(0), destroyedFromPool(0),
       StatisticsFileName (this)
{
 getParameter  ("LogStatisticsToFile", LogStatisticsToFile);
 searchInPool = new ABA_CPUTIMER (this);

 if ( LogStatisticsToFile )
     if ( getParameter ("PlanatorStatisticsFileName", StatisticsFileName))
          StatisticsFileName = "//KM//usr//fialko//planarisator.statistics.log";

 if ( getParameter  ("MoreSeparate", MoreSeparate))
     MoreSeparate = false;

 if ( getParameter  ("BranchMoreReferences", BranchMoreReferences))
     BranchMoreReferences = true;
}
//*******************************************************************************
ABA_SUB* PLANARISATOR::firstSub () 
{ return new PLANAR_SUB (this); }
//******************************************************************************
void PLANARISATOR::set_weights (edge_array<int>& w) {
   edge e;
   weights_.init (*G, 0);
   forall_edges (e, *G)
       weights_[e] = w[e];
}

//*******************************************************************************
void PLANARISATOR::initializeOptimization ()  {  	
//Maximize, Upper Bound is #ofEdges, lower bound is (#ofEdges - result_of_Approx) or 0 
// (if no heuristic were called).

  edge2evar.init (*G, 0);
  ABA_BUFFER<ABA_VARIABLE*> var_buf (this, G->number_of_edges ());
  int i, num = 0;
  edge e;
  KURATOWSKI_EDGE* ke;

  forall_edges (e, *G) {
        ke = new KURATOWSKI_EDGE ( this, source (e), target (e), nil, num, weights_[e]);
//	print_evar (ke, 0);
	var_buf.push (ke);
	num++;
  }

  G->del_all_edges ();
  for ( i = 0; i < var_buf.number (); i++)
     restore_edge ((KURATOWSKI_EDGE*)var_buf[i]);
  
  ABA_BUFFER<ABA_CONSTRAINT*> EulerCon (this, 1);
  if (G->number_of_edges ()/2 > 3*G->number_of_nodes () - 6 )  {
  	ABA_ARRAY <int> support (this, G->number_of_edges ()/2, 1);
        ABA_ARRAY <double> coeff (this, G->number_of_edges ()/2, 1.0);
	
	for (num = 0; num < G->number_of_edges ()/2; num++)
		support[num] = num;
		
	EulerCon.push (new ABA_SROWCON (this, 0, ABA_CSENSE::Less, 
	               G->number_of_edges ()/2, support, coeff, 
		       3*G->number_of_nodes () - 6, false, false, false));
  }
  
//  dualBound (G->number_of_edges ()/2);
  LowerB == 0 ? primalBound (0) : primalBound (var_buf.number () - LowerB);

  initializePools (EulerCon, var_buf, var_buf.number (), 0);

  CutPool = new ABA_STANDARDPOOL<ABA_CONSTRAINT, ABA_VARIABLE> (this, G->number_of_edges ()/2, true );
  objInteger (true);
}

//*******************************************************************************
void PLANARISATOR::update_solution (list<int>& solution)  {
  int i;
  if ( !best_found_solution.empty() && solution.size () > best_found_solution.size ())
    	return;
// Delete old solution
  best_found_solution.clear ();

  forall (i, solution)
  	best_found_solution.push (i);
}
//*******************************************************************************
// The solution is a list of pairs of nodes. 
list<node> PLANARISATOR::solution () {
 int i;
 list<node> result;
 
 forall (i, best_found_solution) {
   KURATOWSKI_EDGE* ke =  variable (i);
   result.push (ke->target ());
   result.push (ke->source ());
 }
 return result;
}
//********************************************************************************
edge PLANARISATOR::restore_edge (KURATOWSKI_EDGE *ke)  {
   edge e = G->new_edge (ke->source (), ke->target ());
   ke->set_edge (e);
   edge2evar[e] = ke;
   e = G->new_edge (ke->target (), ke->source ());
   edge2evar[e] = ke;
   ke->set_reversal (e);
   G->set_reversal (ke->Edge (), e);
   return e;
}

//*******************************************************************************
void PLANARISATOR::remove_edge (KURATOWSKI_EDGE *ke)  {
	G->del_edge (ke->Edge ());
	G->del_edge (ke->reversal ());
	ke->del_edge (); //
}
//*******************************************************************************
void PLANARISATOR::terminateOptimization() {
  int i;
  edge e;

  for (i = 0; i < varPool ()->number (); i++)
     G->del_edge (variable (i)->reversal ());
}
//*******************************************************************************
//*******************************************************************************
// Output some statistics
//
void PLANARISATOR::output () 
{
 if ( !LogStatisticsToFile )
     return;

#undef string
  ofstream f (StatisticsFileName.string (), ios::app);
#define string leda_string
	
 if ( !f )  return ;
 
 f.setf (ios::right);
 f.width (3);
 f << G->number_of_nodes () << "  " << G->number_of_edges ()/2 << "  ";
 f.width (4);
 f.precision (3);
 float D = ((float)G->number_of_edges ()/2)/((float)G->number_of_nodes ());
 f << D << "  ";
 f.width (2);
 
 if ( LowerB )
       f << LowerB << "   ";
 else
       f << "--   ";

 if ( found ())
       f << best_found_solution.size () << "  ";
 else
        f << "--" << "  ";
 f.width (0);

 long tTime = totalTime ()->centiSeconds ();
 f << *totalTime () << "   ";
 f.precision (4);
 f << (float)(100*lpTime()->centiSeconds()/tTime);
// f.width (3);
 f << "%  " << (float)(100*separationTime()->centiSeconds ()/tTime) << "%  ";
 f << 100*searchInPool->centiSeconds ()/tTime;
// f.width (0);
 f << "%  " << nVars () <<"\t" << CutPool->number ();
 f <<  "\t" <<  nKuratowski << "\t" << separatedFromPool << "\t" << destroyedFromPool;
 f << "\t" << nLp() << "\t" << nSub() << "\t" << enumerationStrategy() << "  ";
 f << MoreSeparate << "  " << BranchMoreReferences <<"\n";
 f.close();
}
//********************************************************************
void PLANARISATOR::print_evar (KURATOWSKI_EDGE *ke, double val ) {
  if ( outLevel () == Full ) {
    G->print_node (ke->source());
    cout << " - ";
    G->print_node (ke->target ());
    cout << "\t" << ke->obj ();
    if ( val != 0 )
      cout << " = " << val;
      
    cout << "\n";
  }
}
//********************************************************************
// Create a new constraint for KURATOWSKI_EDGEs corresponding to the edges in list edges.
// The list numbers will be filled with the KURATOWSKI_EDGE's numbers and then sorted.
PLANAR_CONSTRAINT* PLANARISATOR::createCon (list<edge>& edges)
{
  ABA_ARRAY<int> support (this, edges.size ());
  ABA_ARRAY<double> coeff (this, edges.size (), 1.0);// All coeffs are 1's

  list<int> numbers;
  edge e;
  forall (e, edges)
     numbers.push ( evar_for (e)->number ());

  numbers.sort ();

  int i = 0;
  while (!numbers.empty ())
	support[i++] = numbers.pop ();

  numLastCons++;

  return new PLANAR_CONSTRAINT (this, support, coeff, numLastCons);
}
//********************************************************************
int PLANARISATOR::equalSubCompare(ABA_SUB *s1, ABA_SUB *s2)  {
 if ( !s1->branchRule () || !s2->branchRule ())   return 0;

// The problem with set to 0 variable will be prefered
 if (s1->branchRule()->branchOnSetVar() && s2->branchRule()->branchOnSetVar())  {
   if (((ABA_SETBRANCHRULE*)s1->branchRule())->setToUpperBound()) return -1;
   if (((ABA_SETBRANCHRULE*)s2->branchRule())->setToUpperBound()) return 1;
   return 0;
 }
 else return 0;
}
