#include <AGD/impl/plaugment.h>
#include <stdio.h>

class PLAU_SUB;

//***************************************************************
void set_faces ( node n, set<face>* faces, graph& G )  {
    edge e;
    forall_inout_edges(e, n)
    	faces->insert (G.adj_face(e));
}
//*****************************************************************************
//*= true, if n and v are adjazent
bool are_adj (node n, node v);
//*****************************************************************************
int compare (const evar &evar1, const evar &evar2)
   { return compare ((long) evar1, (long) evar2);}
//*****************************************************************************
int number (PLAU_EDGEVAR* const &evar)
{ return evar->number (); }
//*******************************************************************************
PLAU_MASTER::PLAU_MASTER (graph& Gr, int LowerBound, double eps, double machEps)
	: ABA_MASTER("PLAUGMENT", true, false, ABA_OPTSENSE::Max, eps, machEps, 1.0e30),
	  originalG(&Gr), counter(0), nPlanar(0),
	  MaxSolutionLevel (-1), OptSolutionLevel (-1), 
	  AveLPLevel(0), LowerB (-LowerBound), LogStatisticsToFile (false),
	  StatisticsFileName (this), branch(false), wrongSet(false)
{
  getParameter ("LogStatisticsToFile", LogStatisticsToFile);

  if (getParameter ("MaxOpenSubs", MaxOpenSubs))
     MaxOpenSubs = 1000;
  if (getParameter ("UseNewBranchingStrategy", UseNewBranchingStrategy))
     UseNewBranchingStrategy = true;

  separTime = new ABA_CPUTIMER (this);

  if ( LogStatisticsToFile )
     if ( getParameter ("PlauStatisticsFileName", StatisticsFileName))
          StatisticsFileName = "//KM//usr//fialko//plaugment.statistics.log";
}
//*******************************************************************************
PLAU_MASTER::~PLAU_MASTER ()  {}
//*******************************************************************************
ABA_SUB* PLAU_MASTER::firstSub () { return new PLAU_SUB (this);}
//*******************************************************************************
void PLAU_MASTER::initializeOptimization ()  {	
// Copy orinalG to G (without Self-Loops and parallel edges)
  make_copy ();

  BC_.init (&G, true);//true = DFS-Numbering

 //Initialize upper Bound = General Augmentation Optimum
  int nPends = (int)ceil((float)(BC_.number_blocks_with_degree(1))/2);
  if (degree (BC_.root ()) > nPends)   {
       balanced_ = false;
       dualBound ( degree (BC_.root ()));
  }
  else {
       balanced_ = true;
       dualBound (nPends);
  }

  UpperBound = (int) dualBound ();

  if (LowerB != 0) //
  	primalBound (LowerB);//Maximize, Upper Bound is GA-Optimum, lower bound is result of Approx.

  ABA_BUFFER <ABA_VARIABLE*> vars ( this, (int)pow (BC_.number_of_blocks (), 2)/2);
  computeSparseGraph (vars);//Select variables

// Compute statical Biconnectivity constrains
  compute_bicons (BC_.root ());

// In bicons are generated biconnectivity constraints
  ABA_BUFFER<ABA_CONSTRAINT*> all_cons (this, bicons.size ());
  while ( !bicons.empty ())
  	all_cons.push (bicons.pop ());

  initializePools ( all_cons, vars, vars.number (), 0, false);

  Cuts = new ABA_NONDUPLPOOL<ABA_CONSTRAINT, ABA_VARIABLE> (this,  vars.number ()/2, true);
  objInteger (true);
  cplexOutputLevel (0);
  origStrat = enumerationStrategy();
//  enumerationStrategy (DiveAndBest);
}
//******************************************************************************
void PLAU_MASTER::terminateOptimization()
{
  bc_node bcn;
  forall_defined (bcn, block_edges)  {
	delete block_edges[bcn];
	block_edges.undefine (bcn);
  }
}
//*******************************************************************************
bool PLAU_MASTER::update_solution (list<int> &solution ) {
  if ( !best_found_solution.empty() && solution.size () > best_found_solution.size ())
    	return false;

// Delete old solution from originalG
  while ( !best_found_solution.empty ())
  	originalG->del_edge (best_found_solution.pop ());
	
//Add the new solution to originalG
  int i;
  forall (i, solution)
     best_found_solution.push (originalG->new_edge ( G.inf (evar(i)->source()),
			       G.inf (evar(i)->target())));
  return true;
}  

//********************************************************************************
edge PLAU_MASTER::restore_edge (PLAU_EDGEVAR *evar) {
	 edge e = G.new_edge ( evar->source (), evar->target (), evar );
	 evar->set_edge (e);
	 return e;
}
//*******************************************************************************
void PLAU_MASTER::remove_edge (PLAU_EDGEVAR *evar) {
	G.del_edge ( evar->Edge ());
	evar->del_edge ();
}
//********************************************************************************
edgevarset* PLAU_MASTER::edges_for_block (bc_node bl)  {
  return ( block_edges.defined (bl)) ? block_edges[bl] : nil;
}
//*******************************************************************************
void PLAU_MASTER::output ()  {
  if ( !LogStatisticsToFile ) 
      return;

#undef string
  ofstream f (StatisticsFileName.string (), ios::app);
#define string leda_string
	
  if ( !f )  return ;
  	
  f << G.number_of_nodes () << "   " << G.number_of_edges () << "   ";
  float D = ((float)G.number_of_edges ())/((float)G.number_of_nodes ());
  f.precision (3);
  f << D << "\t";
  f << UpperBound << "\t";
  if ( LowerB )
  	f << -LowerB << "  ";
  else
  	f << "--  ";

  	if ( found ()) 
  		f << best_found_solution.size () << "\t";
  	else
  		f << "--\t";
  f.precision (4);
  long tTime = totalTime ()->centiSeconds ();
  f << *totalTime () << "   " << 100*lpTime()->centiSeconds()/tTime;
  f << "%   " << 100*separTime->centiSeconds()/tTime << "%   ";
  f << nVars() <<"\t";

  int nFix = 0, i;
  for (i = 0; i < nVars (); i++)
      if ( variable (i)->fsVarStat ()->fixed () )
         nFix++;

  f << nFix << "\t";

  int nDupl = 0, nColl = 0;
  Cuts->statistics (nDupl, nColl);

  f << conPool()->number () << "\t" << Cuts->number () << "\t" << nColl;
  f <<  "\t" << nPlanar << "\t" << nLp() << "\t" << nSub() << "\t";
  f << counter << "\t" << MaxSolutionLevel << "\t" << OptSolutionLevel << "\t";
  f << (float)AveLPLevel/(float)nSubSelected() << "\t" << nSubSelected () << "\t";
  f <<  ABA_MASTER::enumerationStrategy() << "  ";
  if (!balanced_)
     f << "Nicht_Balanciert!  ";
  if (nDupl)
     f << "nDupl=" << nDupl;

  f << "\n";

 // Format of statistics file: still to do
 // V  E  E/V(Density) LowerBound(or --) Solution(or -- if no better solution was found)
 // TotalCpuTime %LPTime %SeparTime NumberOfGeneratedVars NumFixedVars NumberOfGeneratedBicons  
 // #PlanarCuts #CollisionsInCutPool  nPlanarTest #SuccSeparatingsFromPool 
 // ...
}
//********************************************************************************
void PLAU_MASTER::computeSparseGraph (ABA_BUFFER<ABA_VARIABLE*>& vars) {
  list<edge> added_to_bidirect = Make_Bidirected (G);
  G.make_planar_map ();
  
  node n;
  block Bl;
  list <block> pendants, BadBlocks;
  node_array <facesset*> faces_for_node (G, nil);
  
  forall ( Bl, BC_.all_blocks ())
	if ( degree (Bl) == 1 )  {
		pendants.push (Bl);
	    block_edges [Bl] = new edgevarset;
    }
   	else {	//
//Wenn er wenig ( < 5 ) Knoten hat oder nur aus Schnittpunkten besteht -
//dann kann man diesen block nicht bearbeiten.
	if ( number_of_nodes (Bl) + degree (Bl) < 5 || !number_of_nodes(Bl))
	     		 continue;

// Wenn nicht, versuche ich einen Face zu finden, 
//der alle zum Block adj Cutpoints einschliesst.
// Wenn es so einen Face gibt dann kann ich diesen Block auch nicht bearbeiten.
    list<node> cpoints;
    bc_node bcn;
    forall ( bcn, BC_.adj_nodes (Bl))
       cpoints.push (first_node (bcn));

// Wenn es nur 2 adj Schnittpunkte gibt und die sind mit einer Kante verknpft
    if ( cpoints.size () == 2 && are_adj(cpoints.head(), cpoints.tail ()))
     	    	 continue;

	facesset common_face;
   	forall ( n, cpoints )  {
        	if ( faces_for_node[n] == nil ) 
			{
     			faces_for_node[n] = new facesset;
     			set_faces ( n, faces_for_node[n], G );
        	}
        	if ( n == cpoints.head ())
			    common_face += *faces_for_node [n];
     		else
		     	common_face &= *faces_for_node[n];
		     	
		if ( common_face.empty ()) 
			  break;
        }
     
       if ( !common_face.empty ()) //Common_face umfasst alle zum Bl adj cutpoints.
	      continue;
       BadBlocks.push (Bl);
       adj_to_BadBlocks.push ((cutpoint)father (Bl));
    
       block_edges [Bl] = new edgevarset;
  }

  forall (Bl, pendants)
  	good_edges [Bl]=pendants.size () - 1;
  
  block pend1, pend2;
  PLAU_EDGEVAR* evar;
  
  int i = 0;
//Selecting edges between BadBlocks and pendants
  while ( !pendants.empty())  {
	pend1 = pendants.pop ();	
	node pend1_node = BC_.adj_to_cutpoint_node (pend1);
	//fath is the only cutpoint adj to pendant pend1
	node fath = BC_.head_node (pend1);
	if ( faces_for_node[fath] == nil )  
	{ //Initialize faces of fath
     		faces_for_node[fath] = new facesset;
     		set_faces ( fath, faces_for_node[fath], G );
        }
// First select edges between pendants and BadBlocks
	forall ( pend2, BadBlocks ) 
	{
		list<bc_node> pfad = BC_.path (pend1, pend2);
		bc_node ancestor = (pfad.head () == pend2) ? pfad.inf (pfad.get_item (1)) : father(pend2);

//ancestor is a cutpoint, adj to pend2 and belongs to shortest path to pend1
		node adj2ancestor = BC_.adj_to_cutpoint_node (pend2, (cutpoint)ancestor);
		if ( !adj2ancestor ) continue;
		
		bool planar = (pfad.size() == 3) ? true : false;
		n = first_node(ancestor);
		if ( !planar && are_adj (n, fath))
			planar = true;
			
		if ( !planar )  {
			if ( faces_for_node[n] == nil ) {
     				faces_for_node[n] = new facesset;
     				set_faces ( n, faces_for_node[n], G );
        		}
        		set<face> common = *faces_for_node[n] & *faces_for_node[fath];
        		if ( !common.empty())	planar = true;
        	}
		evar = new PLAU_EDGEVAR ( this, pend1_node, adj2ancestor, i, planar, MIXED_BLOCKS);
		i++;
		vars.push (evar);
		block_edges[pend1]->insert (evar);
		block_edges[pend2]->insert (evar);
	} // forall ( pend2, BadBlocks )..

// Now select the edges between Pendants
	forall ( pend2, pendants ) {
		bool plan = ( father (pend1) == father (pend2)) ? true : false;
		n = BC_.head_node (pend2);
		if ( !plan && are_adj (fath, n))
			plan = true;
		if ( !plan ) {
			if ( faces_for_node[n] == nil)  {
				faces_for_node[n] = new facesset;
     				set_faces ( n, faces_for_node[n], G );
        		}
        		set<face> common = *faces_for_node[n] & *faces_for_node[fath];
        		if ( !common.empty())  	plan = true;
        	}
		evar = new PLAU_EDGEVAR (this, pend1_node, BC_.adj_to_cutpoint_node (pend2), i, plan);
		i++;
		vars.push (evar);
		block_edges[pend1]->insert (evar);
		block_edges[pend2]->insert (evar);
	}
  }
// select edges between two BadBlocks (pend1 and pend2)
  while ( !BadBlocks.empty ()) {
  	pend1 = BadBlocks.pop ();
  	forall ( pend2, BadBlocks )  {
		list<bc_node> pfad = BC_.path (pend1, pend2);
		bc_node begin, end; // 2 end-cutpoints of path pend1 - pend2
		
		begin = father (pend1);
		end = father (pend2);
		
		if ( pfad.head() == pend1 ) 
			begin = pfad.inf(pfad.get_item (1));
		if ( pfad.head() == pend2 ) 
			end = pfad.inf(pfad.get_item (1));
		
		node adj2begin = BC_.adj_to_cutpoint_node (pend1, (cutpoint)begin);
		node adj2end = BC_.adj_to_cutpoint_node (pend2, (cutpoint)end);
		if ( !adj2begin || !adj2end ) continue;
		
		bool planar = ( pfad.size () == 3 ) ? true : false;
		if ( !planar && are_adj (first_node(begin), first_node(end)))
			planar = true;
			
		if ( !planar) {
			if ( faces_for_node[first_node(begin)] == nil)  {
				faces_for_node[first_node(begin)] = new facesset;
     				set_faces ( first_node(begin), faces_for_node[first_node(begin)], G );
        		}
        		if ( faces_for_node[first_node(end)] == nil)  {
				faces_for_node[first_node(end)] = new facesset;
     				set_faces ( first_node(end), faces_for_node[first_node(end)], G );
        		}
        		set<face> common = *faces_for_node[first_node(end)] & *faces_for_node[first_node(begin)];
        		if ( !common.empty())   planar = true;
        	}
		evar = new PLAU_EDGEVAR ( this, adj2begin, adj2end, i, planar, NON_PENDANTS);
		i++;
		vars.push (evar);
		block_edges[pend1]->insert (evar);
		block_edges[pend2]->insert (evar);
	}
  }			

  edge e;	    
  forall (e, added_to_bidirect) 	
  	G.del_edge (e);
	
  forall_nodes ( n, G )
  	if ( faces_for_node[n] )
  		delete faces_for_node[n];
}

//*******************************************************************
set<PLAU_EDGEVAR*>* PLAU_MASTER::compute_bicons (bc_node bcn)  {
  if ( degree (bcn) == 1 ) { //We are in a pendant
  	bicons.push (createCon (block_edges [bcn]));
  	set<PLAU_EDGEVAR*> *result = new set<PLAU_EDGEVAR*> (*block_edges[bcn]);
  	return result;
  }

  bc_node sohn;
  d_array <int, edgevarset*> v_chains_edges;
  int counter = 1;
      		
  forall ( sohn, BC_.adj_nodes (bcn))  
  {
  	if ( father (bcn) == sohn ) continue;	
  	v_chains_edges[counter] = compute_bicons (sohn);
  	counter++;
  }

  if ( !is_cutpoint (bcn))  {
  	if ( degree (bcn) == 2 && !edges_for_block(bcn))
  		return v_chains_edges[1];
  	int i;
  	for ( i = 2; i < degree (bcn); i++ )  {
  		*v_chains_edges[1] %= *v_chains_edges [i];
  		delete v_chains_edges [i];
  		v_chains_edges.undefine (i);
  	}
  		
  	if ( edges_for_block (bcn))
  		*v_chains_edges[1] %= *block_edges[bcn];
  	bicons.push (createCon ( v_chains_edges[1]));
  	return v_chains_edges[1];
  }
// bcn is a cutpoint
  if ( degree (bcn) == 2 )
  	return v_chains_edges[1];
  	
  if ( degree (bcn) == 3 ) {
  	set<PLAU_EDGEVAR*> *result = new set<PLAU_EDGEVAR*> (*v_chains_edges[1]);
  	*result %= *v_chains_edges[2];
  	delete v_chains_edges[1];
  	delete v_chains_edges[2];
  	v_chains_edges.undefine (1);
  	v_chains_edges.undefine (2);
  	bicons.push (createCon (result));
  	return result;
  }
  
  int i;
  set<PLAU_EDGEVAR*> *last = nil;
  if ( BC_.root () != bcn )  {
	last = new edgevarset (*v_chains_edges[1]);
  	for (i = 2; i < degree (bcn); i++ )
  		*last %= *v_chains_edges[i];
  	v_chains_edges [degree(bcn)] = last;
  	bicons.push (createCon ( last));
  }

  list<int> partition;
  partition.push (1);

  edgevarset for_partition = *v_chains_edges [1];
  compute_all_partitions ( partition, for_partition, v_chains_edges, degree(bcn));

  forall_defined ( i, v_chains_edges ) {
  	if ( v_chains_edges[i] != last )  delete v_chains_edges[i];
  	v_chains_edges.undefine (i);
  }
  	
  return last;
 }
//******************************************************************
void PLAU_MASTER::compute_all_partitions (list<int> partit, edgevarset for_partit,
		       		 d_array <int, edgevarset*>& sets, int degree )
{
   if ( partit.size () > 1 )
	bicons.push (createCon ( &for_partit));
   
   if ( degree -  partit.size () == 2 )
   	return;

   int i = partit.tail ();
   for ( ++i; i <= degree; i++ )  {
   	edgevarset next = for_partit % *sets[i];
   	partit.append (i);
   	compute_all_partitions ( partit, next, sets, degree);
	partit.Pop ();
   }
}   		
//*******************************************************************
// Diese Funktion macht copy vom originalG ins G (dabei werden parallel-
// Kanten ( auch in Gegenrichtung) und selfLoops ausgeschlossen. Alle
// kopierten Kanten werden auch ins Buffer vars hinzugefuegt
//*******************************************************************
void PLAU_MASTER::make_copy ()
{
   edge e;
   node n, v;

   node_array <node> orig2copy (*originalG, nil);

   forall_rev_nodes (n, *originalG)
     orig2copy [n] = G.new_node (n);

   node_matrix<bool> is_connected (G, false);

   forall_edges (e, *originalG) {
	n = orig2copy [originalG->source (e)];
	v = orig2copy [originalG->target (e)];
	if ( !is_connected (v, n) && n != v ) {
		is_connected (n, v) = is_connected (v, n) = true;
		G.new_edge (n, v, (PLAU_EDGEVAR*)0);
	}
   }
}
//*******************************************************************
// BiConn - Constraint = the sum of values of edges must be greater than 1.
// The edges belonging to constr. may be not sorted accordingly to their numbers,
// because a BiConn-Constraint will be never compared with anothe constraint.

ABA_CONSTRAINT* PLAU_MASTER::createCon ( set<PLAU_EDGEVAR*>* edges)
{
     ABA_ARRAY <int> support (this, edges->size ());
     ABA_ARRAY <double> coeff (this, edges->size (), 1.0);
   
     PLAU_EDGEVAR* evar;
     int i = 0;
        
     forall ( evar, *edges )  
        support[i++] = evar->number ();
   
     return new ABA_SROWCON (this, 0, ABA_CSENSE::Greater, edges->size (), support, coeff, 1.0, 
            false, false, false);
}
//*******************************************************************
// Planarity-Constraint = sum of values of an edgeset should be LESS 
// than (their number - 1)
// 
PLANAR_CONSTRAINT* PLAU_MASTER::createCon ( list<int> &edges)
{
     ABA_ARRAY<int> support (this, edges.size ());
     ABA_ARRAY<double> coeff (this, edges.size (), 1.0);
   
     int i = 0, num;
     
     edges.sort ();
     forall ( num, edges )
        support[i++] = num;
   
     return new PLANAR_CONSTRAINT (this, support, coeff);
}
//********************************************************************
void PLAU_MASTER::print_evar (PLAU_EDGEVAR* evar, double val )  {
    evar->print ();
    if ( val != 0 )
      out () << " = " << val;
    out () << "\n";
}
//*******************************************************************
PLAU_EDGEVAR* PLAU_MASTER::evar (int i)
{
  return (PLAU_EDGEVAR*)varPool()->slot(i)->conVar();
}
//*******************************************************************
bool PLAU_MASTER::planar_edge (int i)
{
   if (evar (i)->is_planar ())
	return true;
	
   PLAU_EDGEVAR* e = evar (i);
   restore_edge (e);
   nPlanar++;
   
   if ( !PLANAR (G)) {
   	remove_edge (e);
	bc_node s = BC_.owner (e->source ()), t =  BC_.owner (e->target ());
	edges_for_block(s)->del (e);
	edges_for_block(t)->del (e);
	if ( e->kind () == OUTSIDE_PENDANTS ) {
		good_edges[s]--;
		good_edges[t]--;
	}
   	return false;
   }
   else {
   	e->set_planar ();
   	remove_edge (e);
   	return true;
   }
}
//******************************************************************
int PLAU_MASTER::equalSubCompare(ABA_SUB *s1, ABA_SUB *s2)
{
  if (((PLAU_SUB*)s1)->NumOfSet2One () > ((PLAU_SUB*)s2)->NumOfSet2One())
  	return 1;
  if (((PLAU_SUB*)s1)->NumOfSet2One () < ((PLAU_SUB*)s2)->NumOfSet2One())
  	return -1;
  return 0;
}
