#include <LEDA/sortseq.h>
#include <LEDA/node_pq.h>
#include <AGD/impl/plaugment.h>

//****************************************************************************
PLAU_SUB::PLAU_SUB (ABA_MASTER *master)
        :ABA_SUB (master, 50.0, 1000.0, 50.0, true), NumOfSet2One_(0), BranchingVar (-1)
{}
//*****************************************************************************
PLAU_SUB::PLAU_SUB (ABA_MASTER *master, ABA_SUB *father, ABA_BRANCHRULE *brule)
	 :ABA_SUB (master, father, brule ), BranchingVar (-1)
{
   if ( ((ABA_SETBRANCHRULE*)brule)->setToUpperBound() )
   	NumOfSet2One_ = ((PLAU_SUB*)father)->NumOfSet2One() + 1;
   else
   	NumOfSet2One_ = ((PLAU_SUB*)father)->NumOfSet2One();
}
//*****************************************************************************
bool PLAU_SUB::feasible ()
{
  bool feasible;

  if ( integerFeasible ()) 
    feasible = true;
  else
    feasible = false;

  list<int> ones, rest, Set;
  int i;

// Select in ones all variables having largest (about 1.0) values: 
// sum_{e \in ones} x(e) <= ones.size () - 1
// rest has all other variables with non-zero values. Besides Set stores
//numbers of set variables
  filter_solution (ones, rest, Set);
  
  plau_master()->separTime->start ();
  int nPlanarCuts = separatePlanarCuts (ones, rest);
  plau_master()->separTime->stop ();

  if ((!nPlanarCuts && !feasible) || (nPlanarCuts == 1 && plau_master()->branch)) {
 // This sub will be branched now
 // Move all variables having fractional values to the list rest. Search for a branching
 // Variable within the list rest.
   
   	  forall (i, ones) 
	    if ( !master_->equal (xVal(i), 1.0)) 
	      rest.push (i);

	   if ( plau_master ()->UseNewBranchingStrategy)
	      BranchingVar = newBranchingStrategy  (rest, Set);
	   else
	      BranchingVar = oldBranchingStrategy  (rest);
           return false;
  }

  if (!nPlanarCuts && feasible) //Solution is feasible
  {
        plau_master ()->update_solution (ones);
        if (level () > plau_master ()->MaxSolutionLevel)
	    plau_master ()->MaxSolutionLevel = level ();//for tuning and debug
        plau_master ()->OptSolutionLevel = level ();
        return true;
  }

  if ( nPlanarCuts ) //some constraints have been separated
    return false;
}
//*************************************************************************
// In the list vars are only vars having values greater then 0.0
// I try to branch on an OUTSIDE_PENDANTS-variable with the value closest to 0.5.
// This variable should be also planar. If it is known that the var is not planar
// Need I this strategy?
int PLAU_SUB::oldBranchingStrategy (list<int>& vars)
{
// best_Pendats is the smallest difference to 0.5 for all OUTSIDE_PENDANTS-edges
// best_Mixed is the same for all remaining edges
   float fract, best_Pendats = MAXINT, best_Mixed = MAXINT;

// best_PendantsEdge stores the number of PLAU_EDGEVAR with  best_Pendats
// best_MixedEdge is the same for best_Mixed
   int best_PendantsEdge, best_MixedEdge;
   int i, var = MAXINT;

   forall ( i, vars )  {
// If Var #i is already set or has value 1.0 then do nothing
      if ( fsVarStat (i)->fixedOrSet () || master_->equal (xVal (i), 1.0))  {
      	cerr << "Eine selektierte Variable ist fixiert, gesetzt oder gleich 1.0\n";
      	exit (Fatal);
      }
// Else calculate the fractional part of its value: I try to set 
// an PLAU_EDGEVAR with value close to 0.5
      fract = fabs (0.5 - fracPart(xVal (i)));

// OUTSIDE_PENDANTS-edges are prefered to all other kinds of edges
      PLAU_EDGEVAR* e = edge_var (i);
      if ( e->kind () == OUTSIDE_PENDANTS ) {
           if ( fract < 0.2 ) // 
	     return i;
	   else if ( fract < best_Pendats ) 
           {
	           best_Pendats = fract;
		   best_PendantsEdge = i;
	   }
      }
      else if ( fract <  best_Mixed ) {
	       best_Mixed = fract;
	       best_MixedEdge = i;
           }
   }
   if ( best_Pendats != MAXINT )
	return best_PendantsEdge;

   if ( best_Mixed != MAXINT )
    	return best_MixedEdge;

   return -1;
}
//********************************************************************************
// In this structure I will store how many edges with fractional values are adjacent
// to a block. The list edges stores the numbers of edges itself.
struct b_edges   
{
    int n_free;
    list<int> edges;
};
//*********************************************************************************
// The new branching strategy first finds all blocks that are source or target of
// an edge with fractional solution in last solution (all such variables are in the
// list vars). 
// Then I select from blocks found in first step one with minimal amount of "free" edges.
// A free edges is an edge that is either fixed nor set. Finally I branch on 
// a variable that is adjacent to the block found in second step and has closest value
// to 0.5.
int PLAU_SUB::newBranchingStrategy (list<int>& vars, list<int>& Set)
{
// For each pendant p I'll select edges that have fractional values in last LP
// (they must be in the list vars) and are adjacent to p. Those edges will be
// stored in b_struct edges. Besides I'll compute for p the number of adjacent
// edges that are not set or fixed. This number is n_free. Each pendant having
// at least one adjacent edge from vars has an entry in blocks with pointer to
// corresponding structure b_edges
  d_array<bc_node, b_edges*> blocks;

  int i;
// I'm interested mainly for PENDATS edges. All non-Pendants edges are stored in 
// the list mixed_edges.
  list<int> mixed_edges;

  forall (i, vars)
  	if ( edge_var (i)->kind () == OUTSIDE_PENDANTS) {
  		bc_node sour = s_owner (i), tar = t_owner (i);
  		if (!blocks.defined (sour)) {
  		  blocks[sour] = new b_edges ();
		  blocks[sour]->n_free = plau_master()->ngood_edges (sour);
  		  blocks[sour]->edges.push (i);
  		}
  		else
  		  blocks[sour]->edges.push (i);

  		if (!blocks.defined (tar)) {
  		  blocks[tar] = new b_edges ();
		  blocks[tar]->n_free = plau_master()->ngood_edges (tar);
  		  blocks[tar]->edges.push (i);
  		}
  		else
  		  blocks[tar]->edges.push (i);
  	}
  	else    mixed_edges.push (i);

  bc_node b;

// Now I reduce the numbers n_free of all relevant pendants by the number
// of edges that are set in this subproblem  	
  forall ( i, Set)
  	if ( edge_var (i)->kind () == OUTSIDE_PENDANTS)  {
  		bc_node sour = s_owner (i), tar = t_owner (i);
  		if (blocks.defined (sour))
	  		blocks[sour]->n_free--;

  		if (blocks.defined (tar)) 
	  		blocks[tar]->n_free--;
  	}
// Search for a pendant with minimal n_free  
  int min = MAXINT;
  bc_node min_b = nil;
  b_edges *ed;

  forall_defined (b, blocks) {
  	ed = blocks[b];
  	if ( min > ed->n_free ) {
  		min = ed->n_free;
  		min_b = b;
  	}
  }
  
  list<int> edges;
  if (min_b) 
     edges = blocks[min_b]->edges;
  else
     edges = mixed_edges;
  double fract, best = MAXINT;
  int best_edge;

// Select an edge from list edges of found pendant min_b that
// has value closest to 0.5  
  forall (i, edges )  {
  	fract = fabs (0.5 - fracPart(xVal (i)));
    	if (fract < best) {
    		best = fract;
    		best_edge = i;
    	}
  }
 
  forall (ed, blocks)
  	delete ed;
  
  return best_edge;
 }
//*********************************************************************************
// Overwrite ABACUS virtual funktion. 
//
int PLAU_SUB::selectBranchingVariable (int& var) {
   if (BranchingVar < 0)
     return 1;
   var = BranchingVar;
   return 0;
}
//*********************************************************************************
// Args: in selected are numbers of PLAU_EDGEVARs with greatest values in last LP-solution
//       in scnd_choice are all remaining PLAU_EDGEVAR's with non-zero values
//
int PLAU_SUB::separatePlanarCuts (list<int>& selected, list<int>& scnd_choice) {
  edge e;
  list<edge> nonPlanar;
  int i;

// First check whether every edge in selected is planar.
  forall (i, selected)
  	if ( plau_master()->planar_edge (i))
  		continue;
  	else {
// Push the numbers of non planar variables into plau_master()->ToFixVars
	  plau_master()->ToFixVars.push (i);

// Exception situation:
// a variable is non planar but it also has been chosen as branching variable
// in one of earlier subs and set to upper bound. This sub must be fathomed. I can't
// make this immediately, but I cancel the separation, set the bit wrongSet.
// The return value will be 0, this indicates that no constraints have been separated.
	  if ( fsVarStat (i)->status () == ABA_FSVARSTAT::SetToUpperBound ) {
	    plau_master ()->wrongSet = true;
	    return 0;
	  }
	}
	
//Finish the separation, if non planar variables were found
  if ( !plau_master()->ToFixVars.empty ())
    return plau_master()->ToFixVars.size ();

// If all variables with largest values are planar, then check whether the other
// variables with non-zero are planar too.

  forall (i, scnd_choice ) {
  	if ( plau_master()->planar_edge (i))
  		continue;
  	else  {
	  plau_master()->ToFixVars.push (i);
	  if ( fsVarStat (i)->status () == ABA_FSVARSTAT::SetToUpperBound )
	    return 0;
	}
  }

// Finish separation if non planar variables were found
  if ( !plau_master()->ToFixVars.empty ())
    return 1;

// Separate from Pool plau_master()->Cuts. Add separated constraints to this sub.
// 1. Sort the list selected for faster comparing.
  selected.merge_sort ();

// 2. Search also for constrains that are not violated but the slack for the
//    solution vector is equal 0 (lhs = rhs). Another condition
//    for selection such constrains is they must contain only variables
//    from list selected. Store such constraints in not_violated
  list<PLANAR_CONSTRAINT*> equals;
  
  int fromPool = SeparationFromCutsPool (selected, equals);

  if (fromPool) // There are some violated constraints
  {
      plau_master ()->counter += fromPool;//counter counts successful separations from Pool
      return fromPool;// ??? Ueberpruefe, ob die constraints addiert werden
  }
  if (!equals.empty ()) //I can't find any really violated constraint, but
                        // I have some constraints in equals
  // I'll now separate within selected edges, but before I have to destroy all
  // Kuratowski subdivisons represented by constraints in equals. Steps:
  // 1. Find in each constraint stored in equals a variable which has
  //    the minimal value in last LP OR select any variable if all variables 
  //    have the value 1.0.
  // 2. Remove this variable from selected and insert it in scnd_choice.
  {
      plau_master ()->counter += equals.size ();
      PLANAR_CONSTRAINT* con;
      forall (con, equals)   {
	int min = con->find_min (xVal_);
	list_item li = selected.search (min);
	if ( li != nil )
		scnd_choice.push (selected.del (li));
      }
   }

// Separate from scratch 
  ABA_BUFFER <ABA_CONSTRAINT*> buff (master_, master_->maxConAdd());
  bool violated = true;

  int result = quickSeparate (selected, buff, violated); //separate from scratch.
  if ( result)  {
	addCons (buff, plau_master ()->Cuts);
	if (!violated) 
// Only one constraint is separated and it is not violated - this subproblem 
// must be branched
	   plau_master ()->branch = true;
  }
   return result;
}
/*****************************************************************************/
/*****************************************************************************
 This function puts all variables with values about 1.0 in the list ones
and all other non-zero variables in the list rest. The sum of values of variables
put in selected must be greater or equal than their number - 1. Set contains
the numbers of all Set Variables
*/
void PLAU_SUB::filter_solution ( list<int>& ones, list<int>& rest, list<int>& Set)
{
  p_queue<double, int> vars;
//Sort variables having non zero values accordigly to their values
  int i, num;
  for ( i = 0; i < nVar (); i++ ) {
    	if ( !master_->equal ( 0.0, xVal (i)))
		vars.insert (-(xVal (i)), i);
	if ( IsLocalSet (i))
		Set.push (i);
  }

  double sum = 0;
  pq_item maxi;
  i = 0;

// Now put variables with greatest values as long as their sum is not less then their 
// number - 1.
  do {
     maxi = vars.find_min ();
     sum -= vars.prio (maxi);
     
     ones.push (vars.inf (maxi));
     vars.del_item (maxi);
     i++;
  }  while ( sum > i - 1 + master_->eps () && !vars.empty ());

  if ( sum + master_->eps () < i - 1 )
	//Move the last added variable from ones to rest
	rest.push ( ones.pop ());
  
  while ( !vars.empty ()) {
  	rest.push (vars.inf ( vars.find_min ()));
  	vars.del_min ();
  } 
}
  
//******************************************************************************** 
void PLAU_SUB::activate () {
// The numbers of variables that are non planar and have to be fixed are detected
// during the separation phase and are stored in the list plau_master()->ToFixVars.
  while  (!plau_master()->ToFixVars.empty ())  {
    bool newVal;
    int i = plau_master()->ToFixVars.pop ();
    ABA_SUB::fix ( i, new ABA_FSVARSTAT (master_, ABA_FSVARSTAT::FixedToLowerBound), newVal);
  }
  
  if ( plau_master()->origStrat == ABA_MASTER::DiveAndBest || 
       plau_master()->origStrat == ABA_MASTER::BreadthFirst )  {
     // Usually BestFirst generates too many open problems and runs often "out of memory".
     // Therefore I switsch sometimes to DepthFirst to reduce the number of open Problems.
     // If the number is moderate I switch back to BestFirst.
	  if ( master_->openSub ()->number () > plau_master ()->MaxOpenSubs )
  		master_->enumerationStrategy (ABA_MASTER::DepthFirst);
	  if ( master_->openSub ()->number () < 100 )
		master_->enumerationStrategy (plau_master ()->origStrat);
  }
}
//*************************************************************************
void PLAU_SUB::fixByLogImp(ABA_BUFFER<int> &variables, ABA_BUFFER<ABA_FSVARSTAT*> &status)
{
  while ( !plau_master ()->wrongSet && !plau_master()->ToFixVars.empty ())   {
      variables.push (plau_master()->ToFixVars.pop ());
      status.push(new ABA_FSVARSTAT(master_, ABA_FSVARSTAT::FixedToLowerBound));
  }
}
//*************************************************************************
// 
int PLAU_SUB::quickSeparate ( list<int>& edges, ABA_BUFFER <ABA_CONSTRAINT*> &buff,
                              bool& violated )
{
  edge e;
  int i;

  forall (i, edges)
  	restore_edge (i);

  plau_master ()->nPlanar++;
  while ( !PLANAR (*Graph ()))  
  {
	plau_master ()->nPlanar++;
	list<int> minKur, safes;
// Remove one edge. Test planarity. If the Graph is not planar, then push
// removed edge to safes. Otherwise the removed edge destroy one(last) Kuratowski
// subdivision and therefore belongs to this subdivison. Add this edge to minKur and restore
// it in Graph.
// So I check each edge in the list edges and then generate a new constraint for minKur,
  	forall (i, edges) {
  		remove_edge (i);
  		plau_master ()->nPlanar++;
  		
  		if (!PLANAR (*Graph ()))
  			safes.push (i);
  		else  {
  			restore_edge (i);
  			minKur.push (i);
  		}
  	}
// The found constraint is in MinKur
	PLANAR_CONSTRAINT* con = plau_master ()->createCon (minKur);
	int min = con->find_min (xVal_);
	remove_edge (min);
	minKur.remove (min);
	buff.push ((ABA_CONSTRAINT*)con);

	forall (i, safes) 
  		restore_edge (i);
        safes.conc (minKur);
  	edges = safes;
  }
  
  forall (i, edges)
  	remove_edge (i);
 //Now check whether at least one of separated constrains is really violated
  if ( buff.number () == 1 && buff[0]->slack (actVar(), xVal_) == 0 )
  	violated = false;

  return buff.number ();
}
//*********************************************************************************
 bool PLAU_SUB::exceptionBranch() 
 {
    if ( plau_master ()->branch ) {
      plau_master ()->branch = false;
      return true;
    }
    return false;
 }
//********************************************************************
bool PLAU_SUB::exceptionFathom()  {
  if ( !plau_master ()->wrongSet ) return false;
  plau_master()->wrongSet = false;
  return true;
}
//********************************************************************
// Separation from Pool:
// 1. Find all violated constraints
// 2. Find constraints that have slack 0 and contains only vars from list selected.
//    Push these constraints into list equals.
int PLAU_SUB::SeparationFromCutsPool (list<int>& selected, list<PLANAR_CONSTRAINT*>& equals)
{
  int i, debug = 0, num = addConBuffer_->number ();
  equals.clear ();
  ABA_NONDUPLPOOL<ABA_CONSTRAINT, ABA_VARIABLE> *pool = plau_master ()->Cuts;

  for (i = 0; i < pool->size (); i++)  {
      PLANAR_CONSTRAINT *con = (PLANAR_CONSTRAINT*)(pool->slot(i)->conVar());
      if (con )
      {
          debug++;
          PLANAR_CONSTRAINT::VIOLATE_STATUS stat = con->violated_or_equal (xVal_, selected);
          if (!stat)
	     continue;
	  if ( stat == PLANAR_CONSTRAINT::VIOLATED )
             addConBuffer_->insert (pool->slot(i), true);
	  else // stat = PLANAR_CONSTRAINT::EQUAL 
	     equals.push (con);
      }
  }
  if ( debug != pool->number ()) {
     cerr << "Fehler in der SeparationFromCutsPool\n";
     exit (Fatal);
  }
  return addConBuffer_->number () - num;
}
//***********************************************************************************
void PLAU_SUB::fathom (bool reoptimize) {
 ABA_SUB::fathom (reoptimize);
 if (nIter_ = 0) // this problem is fathomed without cutting planes generation 
                 //(i.e opt.solution in other problem has been found)
     return;
 plau_master ()->AveLPLevel += level ();
}
