//-------------------------------------------------------------------
//
//                         agd_demo.c
//
// This demo program shows, how to use layout algorithms of the
// AGD-Library with LEDA's GraphWin.
//
//    authors: David Alberts, Carsten Gutwenger, and some more ...
//    date:   july 1997
//-------------------------------------------------------------------

#include "GraphWinAGD.h"
/*
#include <LEDA/graphwin.h>

#include <AGD/FPPLayout.h>
#include <AGD/ConvexLayout.h>
#include <AGD/ConvexDrawLayout.h>
#include <AGD/MixedModelLayout.h>
#include <AGD/PlanarStraightLayout.h>
#include <AGD/PlanarDrawLayout.h>
#include <AGD/SchnyderLayout.h>
#include <AGD/SpringLayout.h>
#include <AGD/TutteLayout.h>
#include <AGD/VisibilityRepresentation.h>
#include <AGD/GraphWinInterface.h>
#include <AGD/generators.h>
#include <AGD/VisibilityMapper.h>
#include <AGD/TreeLayout.h>
#include <AGD/SugiyamaLayout.h>

#include <AGD/PlanAug.h>
#include <AGD/MaxPlanSub.h>
#include <AGD/LEDAMakeBiconnected.h>


void random_bic_graph (graph& G, int num_nodes)
{
	random_tric_graph (G, num_nodes, 2*num_nodes);
	int i, n = num_nodes/2;

	for (i = 1; i <= n; i++) {
		edge e = G.choose_edge();
		G.del_edge(e);
	}

	Make_Biconnected (G);
}

void make_simple (graph& G)
{
	Delete_Loops (G);
	Make_Simple (G);

	edge_array<edge> rev(G);
	Is_Bidirected (G, rev);
	list<edge> del_edges;
	edge e;

	forall_edges (e, G) {
		if (rev[e] != nil) {
			del_edges.append(e);
			rev[rev[e]] = nil;
		}
	}

	forall (e, del_edges) G.del_edge(e);
}

void random_graph_2 (graph& G, int num_nodes, int num_edges)
{
	random_graph (G, num_nodes, num_edges);

	make_simple(G);
}

void random_graph_3 (graph& G, int num_nodes, int num_edges)
{
	random_planar_graph (G, num_nodes, num_edges);

	int i, m = num_edges/10;

	for (i = 1; i <= m; i++)
		G.new_edge(G.choose_node(),G.choose_node());

	make_simple(G);
}


//
// the class AGDDemo encapsulates the algorithm calls
//

class AGDDemo {

public:

	//
	// the available layout algorithms
	//

	static FPPLayout *fpp;
	static ConvexLayout *convex;
	static ConvexDrawLayout *convex_draw;
	static PlanarStraightLayout *straight;
	static PlanarDrawLayout *planar_draw;
	static MixedModelLayout *mixed;
	static SchnyderLayout *schnyder;
	static SpringLayout *spring;
	static TutteLayout *tutte;
	static VisibilityRepresentation *visib_rep;
	static SugiyamaLayout *sugiyama;
	static TreeLayout *tree_layout;

	static PlanAug *plan_aug;
	static LEDAMakeBiconnected *make_bic;
	static MaxPlanSub *plan_sub;

	static void init_algorithms();
	static void cleanup();

	//
	// call function for each algorithm
	//

	static void callFppLayout (GraphWin& GW) {
		callGridLayout (GW, *fpp);
	}
	static void callConvexLayout (GraphWin& GW) {
		callGridLayout (GW, *convex);
	}
	static void callConvexDrawLayout (GraphWin& GW) {
		callGridLayout (GW, *convex_draw);
	}
	static void callPlanarStraightLayout (GraphWin& GW) {
		callGridLayout (GW, *straight);
	}
	static void callPlanarDrawLayout (GraphWin& GW) {
		callGridLayout (GW, *planar_draw);
	}
	static void callMixedModelLayout (GraphWin& GW) {
		callGridLayout (GW, *mixed);
	}
	static void callSchnyderLayout (GraphWin& GW) {
		callGridLayout (GW, *schnyder);
	}
	static void callTutteLayout (GraphWin& GW) {
		tutte->bbox (DRect(GW.get_xmin(), GW.get_xmax(),
			-GW.get_ymax(), -GW.get_ymin()));
		callLayout (GW, *tutte);
	}
	static void callSpringLayout (GraphWin& GW) {
		callLayout (GW, *spring);
	}
	static void callVisibilityRepresentation (GraphWin& GW) {
		callGridLayout (GW, *visib_rep);
	}
	static void callSugiyamaLayout (GraphWin &GW);
	static void callTreeLayout(GraphWin& GW) {
		callLayout(GW, *tree_layout);
	}


	//
	// call function for graph generators
	//

	static void callRandomPlanarTricGraph (GraphWin& GW) {
		gen_random_graph (GW, random_tric_graph,
			"Create Random Triconnected Planar Graph");
	}
	static void callRandomPlanarBicGraph (GraphWin& GW) {
		gen_random_graph (GW, random_bic_graph,
			"Create Random Biconnected Planar Graph");
	}
	static void callRandomGraph (GraphWin& GW) {
		gen_random_graph (GW, random_graph_2,
			"Create Random Graph");
	}
	static void callRandomGraph2 (GraphWin& GW) {
		gen_random_graph (GW, random_graph_3,
			"Create Random Graph (2)");
	}

	//
	// call function for augmentation algorithms
	//

	static void callPlanAug (GraphWin& GW) {
		callAugment (GW, *plan_aug);
	}
	static void callMakeBic (GraphWin& GW) {
		callAugment (GW, *make_bic);
	}

	//
	// call function for subgraph modules
	//

	static void callPlanSub (GraphWin& GW) {
		callSubgraph (GW, *plan_sub);
	}

	//
	// call functions for options dialog boxes
	//

	static void optionsConvex (GraphWin& GW);
	static void optionsConvexDraw (GraphWin& GW);
	static void optionsStraight (GraphWin& GW);
	static void optionsPlanarDraw (GraphWin& GW);
	static void optionsMixedModel (GraphWin& GW);
	static void optionsSpring (GraphWin& GW);
	static void optionsTreeLayout(GraphWin& GW);
	static void optionsSugiyama( GraphWin &GW );

	//
	// call functions for step by step animation
	//

	static void animateSpring (GraphWin& GW) {
		animate (GW, *spring);
	}
	static void animateSugiyama (GraphWin& GW) {
		animate (GW, *sugiyama);
	}

private:
	static bool sugiyama_infos;
	static anim_status;

	static void embed_random(const graph& G, node_array<point>& dest);
	static void stretch(const graph& G, node_array<double>& destx,
        node_array<double>& desty, double x0, double y0, double x1, double y1);
	static void stretch(const graph& G, node_array<point>& dest,
        double x0, double y0, double x1, double y1);
	static void gen_random_graph(GraphWin& gw, void (*f)(graph&,int,int), string s);
	static void gen_random_graph(GraphWin& gw, void (*f)(graph&,int), string s);

	static void fit_in_window (GraphWin& GW, LayoutInterface& A);
	static void anim_action (int n);
	static void wait (panel& P, float delay, int QUIT);

	// call a layout algorithm
	static bool callLayout (GraphWin& GW, LayoutModule& L);
	static bool callGridLayout (GraphWin& GW, GridLayoutModule& GL);

	// call a augmentation algorithm
	static bool callAugment (GraphWin& GW, AugmentModule& A);

	// call a subgraph module
	static bool callSubgraph (GraphWin& GW, SubgraphModule& S);

	// perform an animation sequence
	static void animate(GraphWin& GW, LayoutModule& L);
};


void AGDDemo::callSugiyamaLayout( GraphWin &GW )
{
	if (callGridLayout (GW, *sugiyama) && sugiyama_infos) {
		panel results ("Results");

		results.text_item ("Used heuristic: ");
		results.text_item (sugiyama->name_of_heuristic (
			sugiyama->used_heuristic()));
		results.text_item ("");
		results.text_item ("Number of crossings: ");
		results.text_item ((sugiyama->number_of_crossings() ==
			sugiyama->NOT_COMPUTED) ?
			string ("NOT COMPUTED") :
			string ("%d", sugiyama->number_of_crossings()));
		results.text_item ("");
		results.text_item ("Number of runs: ");
		results.text_item ((sugiyama->number_of_runs() ==
			sugiyama->NOT_COMPUTED) ?
			string ("NOT COMPUTED") :
			string ("%d", sugiyama->number_of_runs()));
		results.button ("OK");
		results.open (GW.get_window());
	}
}

void AGDDemo::fit_in_window (GraphWin& GW, LayoutInterface& A)
{
	const double eps = 0.000001;

	double xmin = GW.get_xmin();
	double xmax = GW.get_xmax();
	double ymin = GW.get_ymin();
	double ymax = GW.get_ymax();

	DRect bbox;

	A.get_layout_size (bbox);

	if (bbox.left() >= xmin-eps)
		if (bbox.right() <= xmax+eps)
			if (-bbox.bottom() >= ymin-eps)
				if (-bbox.top() <= ymax+eps)
					return;

	double r = (xmax-xmin) / (ymax-ymin);

	double h = bbox.bottom() - bbox.top();

	double xm = Max (bbox.right(), h*r+bbox.left());

	GW.win_init (bbox.left(), xm, -bbox.bottom());
}


//
// call a layout algorithm L
//

bool AGDDemo::callLayout (GraphWin& GW, LayoutModule& L) {
	string msg;
	GraphWinInterface A (GW); // an interface for our GraphWin

	// can L draw the graph displayed in GW ?
	if (L.check(GW.get_graph(), msg)) {
		L.call (GW.get_graph(), A); // yes, we can call L
		GW.fill_window();
		return true;
	} else {
		GW.acknowledge (msg); // no, so we display the error message
	}

	return false;
}

bool AGDDemo::callGridLayout (GraphWin& GW, GridLayoutModule& L)
{
	if (callLayout(GW, L)) {
		IRect ir = L.get_grid_area();
		string msg("grid size: %d * %d", ir.width(), ir.height());
		GW.message (msg);

		return true;
	}
	return false;
}

bool AGDDemo::callAugment (GraphWin& GW, AugmentModule& A) {
	string msg;

	// can A be called for the graph displayed in GW ?
	if (A.check(GW.get_graph(), msg)) {

		list<edge> e_list;
		edge e;

		if (A.call (GW.get_graph(), e_list)) { // yes, we can call A

			GW.update_graph();

			GW.deselect_all_edges();
			forall (e, e_list)
				GW.select(e);

			string msg("%d edge(s) added.", A.num_added_edges());
			GW.message (msg);

			return true;
		}
	} else {
		GW.acknowledge (msg); // no, so we display the error message
	}

	return false;
}


bool AGDDemo::callSubgraph (GraphWin& GW, SubgraphModule& S)
{
	graph &G = GW.get_graph();
	string msg;

	// can S be called for the graph displayed in GW ?
	if (S.check(G, msg)) {

		if (S.call (G)) { // yes, we can call S
			GW.update_graph();

			string msg("%d edge(s) deleted.", S.num_del_edges());
			GW.message (msg);

			return true;
		}
	} else {
		GW.acknowledge (msg); // no, so we display the error message
	}

	return false;
}


void AGDDemo::optionsConvex (GraphWin& GW)
{
	bool   size_opt   = convex->size_opt();
	bool   max_outer  = convex->max_outer();
	double base_ratio = convex->base_ratio();

	panel P ("Options for algorithm Convex");

	P.bool_item ("size_opt:",   size_opt);
	P.bool_item ("max_outer:",  max_outer);
	P.real_item ("base_ratio:", base_ratio);

	P.button ("Ok");
	int cancel = P.button ("Cancel");

	if (P.open (GW.get_window()) != cancel) {
		convex->size_opt   (size_opt);
		convex->max_outer  (max_outer);
		convex->base_ratio (base_ratio);
	}
}

void AGDDemo::optionsConvexDraw (GraphWin& GW)
{
	bool   size_opt   = convex_draw->size_opt();
	bool   side_opt   = convex_draw->side_opt();
	bool   max_outer  = convex_draw->max_outer();
	double base_ratio = convex_draw->base_ratio();

	panel P ("Options for algorithm ConvexDraw");

	P.bool_item ("size_opt:",   size_opt);
	P.bool_item ("side_opt:",   side_opt);
	P.bool_item ("max_outer:",  max_outer);
	P.real_item ("base_ratio:", base_ratio);

	P.button ("Ok");
	int cancel = P.button ("Cancel");

	if (P.open (GW.get_window()) != cancel) {
		convex_draw->size_opt   (size_opt);
		convex_draw->side_opt   (side_opt);
		convex_draw->max_outer  (max_outer);
		convex_draw->base_ratio (base_ratio);
	}
}

void AGDDemo::optionsSugiyama( GraphWin &GW ) // MiK
{
  list<string> heuname;

  for ( int i = 0; i < sugiyama->number_of_heuristics(); i++ )
    heuname.append
      ( sugiyama->
	name_of_heuristic( (SugiyamaHeuristics::heuristic_type) i ) );
  
  int
    heur   = sugiyama->used_heuristic(),
    runnum = sugiyama->preset_number_of_runs();

  panel P( "Options for algorithm " + sugiyama->name() );

  P.choice_item( "heuristic:", heur, heuname );
  P.int_item( "preset number of runs:", runnum );
  P.text_item( string( "(%d = auto-analyse mode)",
			     sugiyama->AUTO_ANALYSE ) );
  P.bool_item ("show info", sugiyama_infos);

  const int
    OK     = P.button( "Ok" ),
    INFO   = P.button( "Info" ),
    CANCEL = P.button ("Cancel");
  int choice;

  P.display( window::center, window::center );
  while ( ( choice = P.read() ) == INFO ) {
    panel PI( "Misc information about Sugiyama" );
    PI.text_item( "Algorithm: " );
    PI.text_item( sugiyama->long_name() );
    PI.text_item( "" ); // why? I don't know, but it's needed anyway...strange
    PI.text_item( "Author: " );
    PI.text_item( sugiyama->author() );
    PI.text_item( "" );
    PI.text_item( "Implementation: " );
    PI.text_item( sugiyama->impl_author() );
    PI.text_item( "" );
    PI.text_item( "Last changed: " );
    PI.text_item( sugiyama->impl_date() );
    PI.text_item( "" );
    PI.text_item( "Version: " );
    PI.text_item( sugiyama->impl_version() );
    PI.button( "OK" );
    PI.open( window::center, window::center );
  }
  if ( choice == OK ) {
    sugiyama->used_heuristic( (SugiyamaHeuristics::heuristic_type) heur );
    sugiyama->preset_number_of_runs( runnum );
  }
}



void AGDDemo::optionsStraight (GraphWin& GW)
{
	bool   size_opt   = straight->size_opt();
	bool   max_outer  = straight->max_outer();
	double base_ratio = straight->base_ratio();

	panel P ("Options for algorithm PlanarStraight");

	P.bool_item ("size_opt:",   size_opt);
	P.bool_item ("max_outer:",  max_outer);
	P.real_item ("base_ratio:", base_ratio);

	P.button ("Ok");
	int cancel = P.button ("Cancel");

	if (P.open (GW.get_window()) != cancel) {
		straight->size_opt   (size_opt);
		straight->max_outer  (max_outer);
		straight->base_ratio (base_ratio);
	}
}

void AGDDemo::optionsPlanarDraw (GraphWin& GW)
{
	bool   size_opt   = planar_draw->size_opt();
	bool   side_opt   = planar_draw->side_opt();
	bool   max_outer  = planar_draw->max_outer();
	double base_ratio = planar_draw->base_ratio();

	panel P ("Options for algorithm PlanarDraw");

	P.bool_item ("size_opt:",   size_opt);
	P.bool_item ("side_opt:",   side_opt);
	P.bool_item ("max_outer:",  max_outer);
	P.real_item ("base_ratio:", base_ratio);

	P.button ("Ok");
	int cancel = P.button ("Cancel");

	if (P.open (GW.get_window()) != cancel) {
		planar_draw->size_opt   (size_opt);
		planar_draw->side_opt   (side_opt);
		planar_draw->max_outer  (max_outer);
		planar_draw->base_ratio (base_ratio);
	}
}

void AGDDemo::optionsMixedModel (GraphWin& GW)
{
	int co_type            = (int)mixed->co_type();
	bool prefer_nodes      = mixed->prefer_nodes();
	bool max_outer         = mixed->max_outer();
	double base_ratio      = mixed->base_ratio();
	bool do_postprocessing = mixed->do_postprocessing();

	panel P ("Options for algorithm Mixed-Model");

	P.choice_item ("co_type:", co_type, "triconnected", "biconnected");
	P.bool_item ("prefer_nodes:",  prefer_nodes);
	P.bool_item ("max_outer:",     max_outer);
	P.real_item ("base_ratio:", base_ratio);
	P.bool_item ("do_postprocessing:", do_postprocessing);

	P.button ("Ok");
	int cancel = P.button ("Cancel");

	if (P.open (GW.get_window()) != cancel) {
		mixed->co_type           ((CanonicalOrderType)(co_type));
		mixed->prefer_nodes      (prefer_nodes);
		mixed->max_outer         (max_outer);
		mixed->base_ratio        (base_ratio);
		mixed->do_postprocessing (do_postprocessing);
	}
}


void AGDDemo::optionsSpring (GraphWin& GW)
{
	bool   noise = spring->noise();
	int    iter  = spring->iterations();

	panel P ("Options for algorithm Spring Embedder");

	P.bool_item ("noise:", noise);
	P.int_item  ("iterations:", iter);

	P.button ("Ok");
	int cancel = P.button ("Cancel");

	if (P.open (GW.get_window()) != cancel) {
		spring->noise (noise);
		spring->iterations (iter);
	}
}


void AGDDemo::optionsTreeLayout (GraphWin& GW)
{

	double sib_separation     = tree_layout->sib_separation();
	double level_separation   = tree_layout->level_separation();
	double subtree_separation = tree_layout->subtree_separation();

	panel P ("Options for algorithm TreeLayout");

	P.double_item ("sib_separation:", sib_separation);
	P.double_item ("level_separation:", level_separation);
	P.double_item ("subtree_separation:", subtree_separation);

	P.button ("Ok");
	int cancel = P.button ("Cancel");

	if (P.open (GW.get_window()) != cancel) {
		tree_layout->sib_separation(sib_separation);
		tree_layout->level_separation(level_separation);
		tree_layout->subtree_separation(subtree_separation);
	}
}


//
// perform a simple animation sequence with layout algorithm L
//

void AGDDemo::wait (panel& P, float delay, int QUIT)
{
	float T1, T2;
	int val;
	double x, y;

	used_time (T1);
	do {
		T2 = T1;
		P.get_event(val,x,y);
	} while (used_time(T2) < delay && anim_status != QUIT);
}

void AGDDemo::anim_action (int n)
{
	anim_status = n;
}

void AGDDemo::animate(GraphWin& GW, LayoutModule& L)
{
	graph& G = GW.get_graph();
	GraphWinInterface gwi(GW);

	string msg;
	if (L.check(G,msg) == false) {
		GW.acknowledge(msg);
		return;
	}

	// the graph may not get changed during animation!
	GW.disable_calls();

	// first call start_animation, if false is returned,
	// animation is not possible
	if (L.start_animation(G,gwi) == true) {

		// create a control panel for animation
		panel P("Animation Control Panel");

		P.buttons_per_line (5);
		const int
			SINGLE = P.button ("SINGLE", &AGDDemo::anim_action),
			PLAY   = P.button ("PLAY", &AGDDemo::anim_action),
			FAST   = P.button ("FAST", &AGDDemo::anim_action),
			STOP   = P.button ("STOP", &AGDDemo::anim_action),
			QUIT   = P.button ("QUIT", &AGDDemo::anim_action);
		const float delay = 0.5f, delay_fast = 0.15f;

		P.display();

		// perform animation until the user presses the Quit button,
		// or no more steps are possible
		bool cont = true;
		anim_status = STOP;
		int val;
		double x, y;
		while (cont && anim_status != QUIT) {
			while (P.get_event(val,x,y) != no_event) ;

			if (anim_status == SINGLE) {
				anim_status = STOP;
				cont = L.next_step();
				fit_in_window (GW, gwi);
			} else if (anim_status == PLAY || anim_status == FAST) {
				cont = L.next_step();
				fit_in_window (GW, gwi);
				wait (P, (anim_status == PLAY) ? delay : delay_fast, QUIT);
			}
		}

		// call stop animation only if animation is still possible
		if (cont) {
		  L.stop_animation();
		  fit_in_window( GW, gwi ); // MiK
		}
	}

	// now, we can enable calls again
	GW.enable_calls();
}
*/   
void start()
{
	GraphWinAGD GW;
	GW.open();
}

int main()
{
	//GraphWinAGD GW;

	// set some  GraphWin options to appropriate values
	//GW.set_constant_width (false);
	//GW.set_node_shape (rectangle_node);

	// initialize algorithms
	//AGDDemo::init_algorithms();

	// create menues for AGD
	/*int layout_menu     = gw_add_menu (GW, "AGD Layout");
	int animation_menu  = gw_add_menu (GW, "Animation");
	int generators_menu = gw_add_menu (GW, "AGD Generators");
	int options_menu    = gw_add_menu (GW, "AGD Options");
	int graph_menu      = gw_add_menu (GW, "AGD Graph");
	int augment_menu    = gw_add_menu (GW, "Augmentation", graph_menu);
	int subgraph_menu   = gw_add_menu (GW, "Subgraph", graph_menu);*/

	// create menu entries for layout algorithms
	/*gw_add_simple_call (GW, AGDDemo::callFppLayout, AGDDemo::fpp->name(), layout_menu);
	gw_add_simple_call (GW, AGDDemo::callSchnyderLayout, AGDDemo::schnyder->name(),
		layout_menu);
	gw_add_simple_call (GW, AGDDemo::callConvexLayout, AGDDemo::convex->name(),
		layout_menu);
	gw_add_simple_call (GW, AGDDemo::callConvexDrawLayout, AGDDemo::convex_draw->name(),
		layout_menu);
	gw_add_simple_call (GW, AGDDemo::callPlanarStraightLayout, AGDDemo::straight->name(),
		layout_menu);
	gw_add_simple_call (GW, AGDDemo::callPlanarDrawLayout, AGDDemo::planar_draw->name(),
		layout_menu);
	gw_add_simple_call (GW, AGDDemo::callVisibilityRepresentation, AGDDemo::visib_rep->name(),
		layout_menu);
	gw_add_simple_call (GW, AGDDemo::callMixedModelLayout, AGDDemo::mixed->name(),
		layout_menu);
	gw_add_simple_call (GW, AGDDemo::callTreeLayout, AGDDemo::tree_layout->name(),
		layout_menu);
	gw_add_simple_call (GW, AGDDemo::callSpringLayout, AGDDemo::spring->name(),
		layout_menu);
	gw_add_simple_call (GW, AGDDemo::callSugiyamaLayout, AGDDemo::sugiyama->name(),
		layout_menu);
	gw_add_simple_call (GW, AGDDemo::callTutteLayout, AGDDemo::tutte->name(),
		layout_menu);*/

	// create menu entries for graph generators
	/*gw_add_simple_call (GW, AGDDemo::callRandomPlanarTricGraph, "planar & triconnected",
		generators_menu);
	gw_add_simple_call (GW, AGDDemo::callRandomPlanarBicGraph, "planar & biconnected",
		generators_menu);
	gw_add_simple_call (GW, AGDDemo::callRandomGraph, "random", generators_menu);
	gw_add_simple_call (GW, AGDDemo::callRandomGraph2, "random (2)", generators_menu);*/

	// create menu entries for graph algorithms
	/*gw_add_simple_call (GW, AGDDemo::callPlanAug, "planar biconnected", augment_menu);
	gw_add_simple_call (GW, AGDDemo::callMakeBic, "biconnected", augment_menu);
	gw_add_simple_call (GW, AGDDemo::callPlanSub, "maximal planar", subgraph_menu);*/

	// create menu entries for algorithm options
	/*gw_add_simple_call (GW, AGDDemo::optionsConvex,     "Convex",         options_menu);
	gw_add_simple_call (GW, AGDDemo::optionsConvexDraw, "ConvexDraw",     options_menu);
	gw_add_simple_call (GW, AGDDemo::optionsStraight,   "PlanarStraight", options_menu);
	gw_add_simple_call (GW, AGDDemo::optionsPlanarDraw, "PlanarDraw",     options_menu);
	gw_add_simple_call (GW, AGDDemo::optionsMixedModel, "Mixed-Model",    options_menu);
	gw_add_simple_call (GW, AGDDemo::optionsSpring,     "Spring Embedder",options_menu);
	gw_add_simple_call (GW, AGDDemo::optionsTreeLayout, "Tree Layout",    options_menu);
	gw_add_simple_call( GW, AGDDemo::optionsSugiyama,   "Sugiyama",       options_menu );*/

	// create menu entries for algorithm animation
	/*gw_add_simple_call (GW, AGDDemo::animateSpring, "Spring-Embedder", animation_menu);
	gw_add_simple_call (GW, AGDDemo::animateSugiyama, "Sugiyama", animation_menu);*/

	// open the graph window with the new menues
	//GW.open();
start();
	// we are finished, delete algorithms created with new in AGDDemo
	//AGDDemo::cleanup();

	return 0;
}

/*
// initialization of AGDDemo's static variables
FPPLayout            *AGDDemo::fpp;
ConvexLayout         *AGDDemo::convex;
ConvexDrawLayout     *AGDDemo::convex_draw;
PlanarStraightLayout *AGDDemo::straight;
PlanarDrawLayout     *AGDDemo::planar_draw;
MixedModelLayout     *AGDDemo::mixed;
SchnyderLayout       *AGDDemo::schnyder;
SpringLayout         *AGDDemo::spring;
TutteLayout          *AGDDemo::tutte;
SugiyamaLayout       *AGDDemo::sugiyama;
TreeLayout           *AGDDemo::tree_layout;
VisibilityRepresentation *AGDDemo::visib_rep;

PlanAug              *AGDDemo::plan_aug;
LEDAMakeBiconnected  *AGDDemo::make_bic;
MaxPlanSub           *AGDDemo::plan_sub;

bool AGDDemo::sugiyama_infos;
int  AGDDemo::anim_status;

void AGDDemo::init_algorithms() {
	// initialize pointer to algorithm classes
	fpp         = new FPPLayout;
	convex      = new ConvexLayout;
	convex_draw = new ConvexDrawLayout;
	straight    = new PlanarStraightLayout;
	planar_draw = new PlanarDrawLayout;
	mixed       = new MixedModelLayout;
	schnyder    = new SchnyderLayout;
	spring      = new SpringLayout;
	tutte       = new TutteLayout;
	visib_rep   = new VisibilityRepresentation;
	tree_layout = new TreeLayout;
	sugiyama    = new SugiyamaLayout;

	plan_aug    = new PlanAug;
	make_bic    = new LEDAMakeBiconnected;
	plan_sub    = new MaxPlanSub;

	// set an appropriate grid coordinate mapper for grid algorithms
	MaxAllMapper mapper (5,5,5);
	VisibilityMapper vis_mapper (5,5,5,5,5);

	fpp        ->set_coordinate_mapper (mapper);
	convex     ->set_coordinate_mapper (mapper);
	convex_draw->set_coordinate_mapper (mapper);
	straight   ->set_coordinate_mapper (mapper);
	planar_draw->set_coordinate_mapper (mapper);
	mixed      ->set_coordinate_mapper (mapper);
	schnyder   ->set_coordinate_mapper (mapper);	
	sugiyama   ->set_coordinate_mapper( mapper );
	visib_rep  ->set_coordinate_mapper (vis_mapper);

	sugiyama_infos = false;
}

// delete memory allocated in init_algorithms
void AGDDemo::cleanup ()
{
	delete fpp;
	delete convex;
	delete convex_draw;
	delete straight;
	delete planar_draw;
	delete mixed;
	delete schnyder;
	delete spring;
	delete tutte;
	delete visib_rep;
	delete tree_layout;
	delete sugiyama;

	delete plan_aug;
	delete make_bic;
	delete plan_sub;
}

//////////////////////////////////////////////////////////////////////
// The following functions are copied from the GraphWin sources.
// Unfortunatly, there seems to be no simple concept to add new
// graph generators. Moreover, the functions used by GraphWin's
// generators are defined static and cannot be used by other modules!

void AGDDemo::embed_random(const graph& G, node_array<point>& dest) { 
	random_source ran;
	node v;
	double x,y;
	forall_nodes(v,G) {
		ran >> x >> y;
		dest[v]=point(x,y);
	}
}

void AGDDemo::stretch(const graph& G, 
				  node_array<double>& destx,
                  node_array<double>& desty,
                  double x0, double y0, double x1, double y1) {
	double xd=x1-x0;
	double yd=y1-y0;

	double f=20;

	x0+=xd/f;
	y0+=yd/f;

	  xd*=1.0-2.0/f;
	yd*=1.0-2.0/f;

	node v;
	forall_nodes(v,G) { 
		destx[v]=x0+destx[v]*xd;
		desty[v]=y0+desty[v]*yd;
	}
}

void AGDDemo::stretch(const graph& G, node_array<point>& dest,
                  double x0, double y0, double x1, double y1) {
 
	// stretches unit-square coordinates given in dest to window (x0,y0)(x1,y1)

	node_array<double> x(G),y(G);

	node v;
	forall_nodes(v,G) {
		x[v]=dest[v].xcoord();
		y[v]=dest[v].ycoord();
	}

	stretch(G,x,y,x0,y0,x1,y1);

	forall_nodes(v,G) dest[v]=point(x[v],y[v]);
}

void AGDDemo::gen_random_graph(GraphWin& gw, void (*f)(graph&,int,int), string s) {
	panel P;
	int num_nodes = gw.get_gen_nodes(),
		num_edges = gw.get_gen_edges();

	P.text_item (s);
	P.int_item ("nodes", num_nodes, 0, 200);
	P.int_item ("edges", num_edges, 0, 400);
	P.button ("ok", 1);
	P.button ("cancel", 0);

	if (P.open(gw.get_window()) == 0) return;

	gw.set_gen_nodes (num_nodes);
	gw.set_gen_edges (num_edges);

	bool old_flush=gw.set_flush(false); 
	gw.clear_graph();
	graph& G=gw.get_graph();
	f (G, num_nodes, num_edges);
	gw.update_graph();
    node_array<point> dest(G);
	embed_random(G,dest);
	stretch(G,dest,gw.get_xmin(),gw.get_ymin(),gw.get_xmax(),gw.get_ymax());
	gw.set_position(dest);
	gw.embed_edges();
	gw.set_flush(old_flush);
	gw.redraw();
}

void AGDDemo::gen_random_graph(GraphWin& gw, void (*f)(graph&,int), string s) {
	panel P;
	int num_nodes = gw.get_gen_nodes();

	P.text_item (s);
	P.int_item ("nodes", num_nodes, 0, 200);
	P.button ("ok", 1);
	P.button ("cancel", 0);

	if (P.open(gw.get_window()) == 0) return;

	gw.set_gen_nodes (num_nodes);

	bool old_flush=gw.set_flush(false); 
	gw.clear_graph();
	graph& G=gw.get_graph();
	f (G, num_nodes);
	gw.update_graph();
    node_array<point> dest(G);
	embed_random(G,dest);
	stretch(G,dest,gw.get_xmin(),gw.get_ymin(),gw.get_xmax(),gw.get_ymax());
	gw.set_position(dest);
	gw.embed_edges();
	gw.set_flush(old_flush);
	gw.redraw();
}
*/