//-------------------------------------------------------------------
//                         PROTOTYPE I
//
//                         agd_test.cpp
//
//    author: Carsten Gutwenger
//    date:   february 1997
//-------------------------------------------------------------------


#include <AGD/FPPLayout.h>
#include <AGD/ConvexLayout.h>
#include <AGD/ConvexDrawLayout.h>
#include <AGD/PlanarStraightLayout.h>
#include <AGD/MixedModelLayout.h>
#include <AGD/AGDLayout.h>
#include <AGD/generators.h>

#include <LEDA/graph_gen.h>
#include <LEDA/array2.h>
#include <stdio.h>

#include <fstream.h>

void my_error_handler (int no, const char *msg)
{
	cerr << "Fehler " << no << ": " << msg << "\n";

	if (no > 0) abort();
}

class Test {
public:
	static void test (GridLayoutModule *algo, void (*Options)() = 0);
	static void init();

	static void OptionsMixedModel();

	static FPPLayout            *fpp;
	static ConvexLayout         *convex;
	static ConvexDrawLayout     *convex_draw;
	static PlanarStraightLayout *straight;
	static MixedModelLayout     *mixed;

private:
	static void test_with_generator (GridLayoutModule& algo,
		bool directed,
		const char *name,
		void (*get_param)(),
		void (*call)(graph&));

	static void test_grid_with_generator (GridLayoutModule& algo,
		bool directed,
		const char *name,
		void (*get_param)(),
		void (*call)(graph&));

	static void test_grid_graph (GridLayoutModule& algo);

	static bool do_grid_test (graph& G, GridLayoutModule& algo,
		node_map<int>& xpos, node_map<int>& ypos,
		edge_map<IPolyline >& edge_points);

	static void get_num_nodes();
	static void get_num_nodes_edges();

	static void call_random_planar (graph& G);
	static void call_random_planar_tric (graph& G);

	static int n, m;
};


int main()
{
	set_error_handler (my_error_handler);

	Test::init();

	char input;

	do {
		cout << "\n\n************************\n";
		cout << "HAUPTMENUE\n\n";
		cout << "<1> Fraysseix\n";
		cout << "<2> Convex\n";
		cout << "<3> ConvexDraw\n";
		cout << "<4> PlanarStraight\n";
		cout << "<5> Mixed-Model\n";
		cout << "\n<0> Beenden\n";

		cout.flush();

		cout << "\nEingabe: ";
		cin  >> input;
		cout << endl;

		switch (input) {
		case '1':
			Test::test (Test::fpp);
			break;
		case '2':
			Test::test (Test::convex);
			break;
		case '3':
			Test::test (Test::convex_draw);
			break;
		case '4':
			Test::test (Test::straight);
			break;
		case '5':
			Test::test (Test::mixed, Test::OptionsMixedModel);
			break;
		}

	} while (input != '0');

	return 0;
}


int Test::n, Test::m;
FPPLayout             *Test::fpp;
ConvexLayout          *Test::convex;
ConvexDrawLayout      *Test::convex_draw;
PlanarStraightLayout *Test::straight;
MixedModelLayout      *Test::mixed;

void Test::init ()
{
	fpp = new FPPLayout;
	convex = new ConvexLayout;
	convex_draw = new ConvexDrawLayout;
	straight = new PlanarStraightLayout;
	mixed = new MixedModelLayout;
}

void Test::test (GridLayoutModule *algo, void (*Options)())
{
	bool directed = true;
	int input, max_input;

	array<int> code(0,7);
	code[0] = 0;
	code[1] = 1;

	algo->set_coordinate_mapper (MaxAllMapper(10,5,5));

	do {
		cout << "\n*******************************\n";
		cout << "Teste " << algo->name() << "-Algorithmus\n\n";

		cout << "<1> Graph ";
		if (directed) cout << "gerichtet\n";
		else cout << "ungerichtet\n";
		max_input = 1;

		if (Options != 0) {
			max_input++;
			cout << "<" << max_input << "> Options\n";
			code [max_input] = 7;
		}
		cout << endl;

		if (algo->need_property(1, "triconnected") == false) {
			max_input++;
			cout << "<" << max_input << "> Test: planarer Zufallsgraph mit fester Knotenzahl\n";
			code [max_input] = 2;
		}

		max_input++;
		cout << "<" << max_input << "> Test: planarer, 3-zshgd. Zufallsgraph mit fester Knotenzahl\n";
		code [max_input] = 3;

		if (algo->need_property(1, "triconnected") == false) {
			max_input++;
			cout << "<" << max_input << "> Gitter-Test: planarer Zufallsgraph mit fester Knotenzahl\n";
			code [max_input] = 4;
		}

		max_input++;
		cout << "<" << max_input << "> Gitter-Test: planarer, 3-zshgd. Zufallsgraph mit fester Knotenzahl\n";
		code [max_input] = 5;

		max_input++;
		cout << "<" << max_input << "> Gitter-Test mit gespeichertem Graph\n";
		code [max_input] = 6;

		cout << "\n<0> Zurueck\n";

		cout.flush();

		cout << "\nEingabe: ";
		cin  >> input;
		cout << endl;

		if (input <= max_input) {
			switch (code[input]) {
			case 1:
				directed = !directed;
				break;
			case 2:
				test_with_generator (*algo, directed, "planar", &get_num_nodes,
					&call_random_planar);
				break;
			case 3:
				test_with_generator (*algo, directed, "planar, dreizusammenhaengend",
					&get_num_nodes_edges, &call_random_planar_tric);
				break;
			case 4:
				test_grid_with_generator (*algo, directed, "planar",
					&get_num_nodes, &call_random_planar);
				break;
			case 5:
				test_grid_with_generator (*algo, directed, "planar, dreizusammenhaengend",
					&get_num_nodes_edges, &call_random_planar_tric);
				break;
			case 6:
				test_grid_graph (*algo);
				break;
			case 7:
				Options();
				break;
			}
		
		}

	} while (input != 0);
}

void Test::OptionsMixedModel ()
{
	int input, i;
	double br;

	do {
		cout << "Optionen fuer MixedModel:\n";
		cout << "<1> base_ratio [" << mixed->base_ratio() << "]\n";
		cout << "<2> co_type [" << 
			((mixed->co_type() == co_biconnected) ? "co_biconnected" : "co_triconnected")
			<< "]\n";

		cout << "\n<0> Zurueck\n\n";
		cout << "\nEingabe: ";
		cin >> input;
		cout << endl;

		switch (input) {
		case 1:
			cout << "base_ratio: ";
			cin >> br;
			mixed->base_ratio (br);
			break;
		case 2:
			cout << "co_type [0 = co_biconnected,1 = co_triconnected] :";
			cin >> i;
			switch (i) {
			case 0: mixed->co_type(co_biconnected); break;
			case 1: mixed->co_type(co_triconnected); break;
			}
		}
	} while (input != 0);
}

void Test::test_with_generator (GridLayoutModule& algo,
	bool directed,
	const char *name,
	void (*get_param)(),
	void (*call)(graph&))
{
	int x;

	cout << "Testalgorithmus:\n";
	cout << "  Rufe wiederholt Layout-Algorithmus " << (algo.name()) << endl;
	cout << "  mit generiertem Graphen auf.\n";
	cout << "  Graph-Generator: " << name << "\n\n";
	
	cout << "Anzahl Wiederholungen: ";
	cin >> x;
	get_param();

	int count_nodes = 0, count_edges = 0;
	int count_check_fails = 0;
	float used_t, count_time = 0.0f, count_check_time = 0.0f;

	for (int i = 1; i <= x; i++) {
		cout << i << "   \r";
		cout.flush();

		graph G;
		if (directed == false) G.make_undirected();

		call (G);

		count_nodes += G.number_of_nodes();
		count_edges += G.number_of_edges();

		node_map<double> x(G), y(G);
		edge_map<DPolyline> ep(G);
		//MapInterface mi (x, y, ep);
		AGDLayout mi(G);
		string msg;

		used_time (used_t);
		bool result = algo.check (G, msg);
		count_check_time += used_time (used_t);
		if (result) {
			used_time (used_t);
			algo.call (G, mi);
			count_time += used_time (used_t);
		} else {
			count_check_fails++;
			cout << "check returned false: " << msg << "\n";
		}
	}

	int num_calls = x - count_check_fails;

	cout << "\n\ndurchschnittl. Anzahl Knoten    : " << ((double)count_nodes)/x;
	cout <<   "\ndurchschnittl. Anzahl Kanten    : " << ((double)count_edges)/x;
	cout <<   "\ndurchschnittl. Zeit fuer Check  : " << count_check_time/x;
	cout <<   "\nCheck schlug fehl               : " << count_check_fails;
	if (num_calls > 0) {
		cout <<   "\ndurchschnittl. Zeit fuer Aufruf : " <<
			count_time/num_calls << endl;
	}
}

void Test::test_grid_with_generator (GridLayoutModule& algo,
	bool directed,
	const char *name,
	void (*get_param)(),
	void (*call)(graph&))
{
	int x;

	cout << "Testalgorithmus:\n";
	cout << "  Generiere Zufallsgraph mit Generator: " << name << ".\n";
	cout << "  Rufe call_grid des Layout-Algorithmus " << algo.name() << " auf.\n";
	cout << "  Teste, ob alle Knoten- und Kantenknickpunkt auf verschiedenen Punkten\n";
	cout << "  liegen.\n\n";
	
	cout << "Anzahl Wiederholungen: ";
	cin >> x;
	get_param();

	int count_nodes = 0, count_edges = 0;
	int count_test_fails = 0;
	float used_t, count_time = 0.0f;

	for (int i = 1; i <= x; i++) {
		cout << i << "   \r";
		cout.flush();

		graph G;
		if (directed == false) G.make_undirected();

		call (G);

		count_nodes += G.number_of_nodes();
		count_edges += G.number_of_edges();

		GridLayout gl(G);

		used_time (used_t);
		algo.call_grid (G, gl);
		count_time += used_time (used_t);
		if (do_grid_test (G, algo, gl.x(), gl.y(), gl.bends()) == false) {
			count_test_fails++;
		}
	}

	cout << "\n\ndurchschnittl. Anzahl Knoten    : " << ((double)count_nodes)/x;
	cout <<   "\ndurchschnittl. Anzahl Kanten    : " << ((double)count_edges)/x;
	cout <<   "\nTest schlug fehl                : " << count_test_fails;
	if (x> 0) {
		cout <<   "\ndurchschnittl. Zeit fuer Aufruf : " <<
			count_time/x << endl;
	}

}

void Test::test_grid_graph (GridLayoutModule& algo)
{
	char name[80];
	graph G;

	cout << "Teste Graph mit " << algo.name() << "-Algorithmus\n\n";
	cout << "Dateiname: ";
	cin >> name;

	if (G.read (name) != 0) {
		cout << "Fehler beim Lesen des Graphen <" << name << ">\n";
		return;
	}

	string msg;
	if (algo.check(G, msg) == false) {
		cout << "Check schlug fehl: " << msg << endl;
		return;
	}

	GridLayout gl(G);
	float used_t;

	used_time (used_t);
	algo.call_grid (G, gl);
	used_t = used_time (used_t);

	if (do_grid_test (G, algo, gl.x(), gl.y(), gl.bends()) == false) {
		cout << "Fehler bei Test.\n";
	} else {
		cout << "Test ok.\n";
	}
	cout << "Berechnungszeit: " << used_t << endl;
}

class BitArray2 {
public:
	BitArray2 (int _a, int b, int _c, int d);

	void set (int i, int j) {
		int jn = j-c;
		int j_d = jn / (8*sizeof(int)),
			j_m = jn % (8*sizeof(int));
		A(i-a,j_d) |= (1 << j_m);
	}

	void clear (int i, int j) {
		int jn = j-c;
		int j_d = jn / (8*sizeof(int)),
			j_m = jn % (8*sizeof(int));
		A(i-a,j_d) &= ~(1 << j_m);
	}

	unsigned int get (int i, int j) const {
		int jn = j-c;
		int j_d = jn / (8*sizeof(int)),
			j_m = jn % (8*sizeof(int));
		return (A(i-a,j_d) & (1 << j_m)) >> j_m;
	}

	unsigned int operator() (int i, int j) {
		return get (i, j);
	}

	void print();

private:
	array2<unsigned int> A;
	int a, b, c, d;
};

BitArray2::BitArray2 (int _a, int _b, int _c, int _d) :
	A(_b-_a+1, ((_d-_c)+8*sizeof(int)-1)/(8*sizeof(int)) +1)
{
	int i, j;

	a = _a; b = _b; c = _c; d = _d;

	for (i = 0; i <= A.high1(); i++)
		for (j = 0; j <= A.high2(); j++)
			A(i,j) = 0;
}

void BitArray2::print ()
{
	int i, j;

	for (i = a; i <= b; i++) {
		for (j = c; j <= d; j++)
			cout << (get(i,j) ? 1 : 0) << " ";
		cout << endl;
	}
	cout.flush();
}

bool Test::do_grid_test (graph& G, GridLayoutModule& algo,
	node_map<int>& xpos, node_map<int>& ypos,
	edge_map<IPolyline >& edge_points)
{
	IRect bbox;
	node v;
	edge e;

	node_array<int> id(G);
	int i=1;

	forall_nodes (v, G)
		id[v] = i++;

	bbox = algo.get_grid_area ();

	BitArray2 mark(bbox.left(), bbox.right(), bbox.bottom(), bbox.top());
	int j;

	forall_nodes (v, G) {
		i = xpos[v]; j = ypos[v];
		if (mark(i,j)) return false;
		mark.set(i,j);
	}

	list_item it;

	forall_edges (e, G) {
		const IPolyline& L = edge_points[e];
		forall_items (it, L) {
			const IPoint &p = L[it];
			i = p.xcoord(); j = p.ycoord();
			if (mark(i,j))
				return false;
			mark.set(i,j);
		}
	}

	return true;
}

void Test::get_num_nodes()
{
	cout << "Anzahl Knoten: ";
	cin >> n;
}

void Test::get_num_nodes_edges()
{
	cout << "Anzahl Knoten: ";
	cin >> n;
	cout << "Anzahl Kanten: ";
	cin >> m;
}

void Test::call_random_planar (graph& G)
{
	random_planar_graph (G, n);
}

void Test::call_random_planar_tric (graph& G)
{
	random_tric_graph (G, n, m);
}
